/** 
 *******************************************************************************
 * @file    io89FS63B.h
 * @brief   Sample SFR Header File for TLCS-870/C1 Series/TMP89FM63B
 * @version V1.0.1.0
 * $Date:: 2019-06-10 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifndef	IO89FS63B_H
#define	IO89FS63B_H

#ifdef IO_MEM
/* This definition is for I/O variable and extern definition */
/* So please do not use the name "IO_MEM".*/
#define EXTERN
#else
#define EXTERN extern
#endif

/********** type define **********/
typedef struct {
    unsigned char b7:1;
    unsigned char b6:1;
    unsigned char b5:1;
    unsigned char b4:1;
    unsigned char b3:1;
    unsigned char b2:1;
    unsigned char b1:1;
    unsigned char b0:1;
} BYTE_FIELD;

/*=======  BYTE IO ========*/
typedef union {
    unsigned char   byte;
    BYTE_FIELD      bit;
} TYPE_BYTE;


typedef struct {
    unsigned int b15:1;
    unsigned int b14:1;
    unsigned int b13:1;
    unsigned int b12:1;
    unsigned int b11:1;
    unsigned int b10:1;
    unsigned int b9:1;
    unsigned int b8:1;
    unsigned int b7:1;
    unsigned int b6:1;
    unsigned int b5:1;
    unsigned int b4:1;
    unsigned int b3:1;
    unsigned int b2:1;
    unsigned int b1:1;
    unsigned int b0:1;
} WORD_FIELD;

/*======= WORD IO ========*/
typedef union {
    unsigned int    word;
    unsigned char   byte[2];
    WORD_FIELD      bit;
} TYPE_WORD;


/***[SFR1]******************************************************/
EXTERN	TYPE_BYTE		__io(0x0000)	_P0DR;			/* Port P0 output latch */
EXTERN	TYPE_BYTE		__io(0x0001)	_P1DR;			/* Port P1 output latch */
EXTERN	TYPE_BYTE		__io(0x0002)	_P2DR;			/* Port P2 output latch */
/*---   (0x0003)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0004)	_P4DR;			/* Port P4 output latch */
EXTERN	TYPE_BYTE		__io(0x0005)	_P5DR;			/* Port P5 output latch */
/*---   (0x0006)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0007)	_P7DR;			/* Port P7 output latch */
EXTERN	TYPE_BYTE		__io(0x0008)	_P8DR;			/* Port P8 output latch */
EXTERN	TYPE_BYTE		__io(0x0009)	_P9DR;			/* Port P9 output latch */
/*---   (0x000A)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x000B)	_PBDR;			/* Port PB output latch */
/*---   (0x000C)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x000D)	_P0PRD;			/* Port P0 input data */
EXTERN	TYPE_BYTE		__io(0x000E)	_P1PRD;			/* Port P1 input data */
EXTERN	TYPE_BYTE		__io(0x000F)	_P2PRD;			/* Port P2 input data */
/*---   (0x0010)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0011)	_P4PRD;			/* Port P4 input data */
EXTERN	TYPE_BYTE		__io(0x0012)	_P5PRD;			/* Port P5 input data */
/*---   (0x0013)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0014)	_P7PRD;			/* Port P7 input data */
EXTERN	TYPE_BYTE		__io(0x0015)	_P8PRD;			/* Port P8 input data */
EXTERN	TYPE_BYTE		__io(0x0016)	_P9PRD;			/* Port P9 input data */
/*---   (0x0017)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0018)	_PBPRD;			/* Port PB input data */
/*---   (0x0019)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x001A)	_UART0CR1;		/* UART0 control register 1 */
EXTERN	TYPE_BYTE		__io(0x001B)	_UART0CR2;		/* UART0 control register 2 */
EXTERN	unsigned char	__io(0x001C)	UART0DR;		/* UART0 baud rate register */
EXTERN	TYPE_BYTE		__io(0x001D)	_UART0SR;		/* UART0 status register */
EXTERN	TYPE_BYTE		__io(0x001E)	U0BF;
#define	TD0BUF	U0BF.byte								/* UART0 transmit data buffer */
#define	RD0BUF	U0BF.byte								/* UART0 receive data buffer */
EXTERN	TYPE_BYTE		__io(0x001F)	_SIO0CR;		/* Serial interface 0 control register */
EXTERN	TYPE_BYTE		__io(0x0020)	_SIO0SR;		/* Serial interface 0 status register */
EXTERN	unsigned char	__io(0x0021)	SIO0BUF;		/* Serial interface 0 buffer register */
EXTERN	TYPE_BYTE		__io(0x0022)	_SBI0CR1;		/* Serial bus interface 0 control register 1 */
EXTERN	TYPE_BYTE		__io(0x0023)	S0CS;
#define SBI0CR2 S0CS.byte								/* Serial bus interface 0 control register 2 */
#define SBI0SR2 S0CS.byte								/* Serial bus interface 0 status register 2 */
EXTERN	TYPE_BYTE		__io(0x0024)	_I2C0AR;		/* I2C bus address 0 register */
EXTERN	unsigned char	__io(0x0025)	SBI0DBR;		/* Serial bus interface 0 data buffer register */
EXTERN	unsigned char	__io(0x0026)	T00REG;			/* Timer register 00 */
EXTERN	unsigned char	__io(0x0027)	T01REG;			/* Timer register 01 */
EXTERN	unsigned char	__io(0x0028)	T00PWM;			/* Timer register 00 */
EXTERN	unsigned char	__io(0x0029)	T01PWM;			/* Timer register 01 */
EXTERN	TYPE_BYTE		__io(0x002A)	_T00MOD;		/* Timer counter 00 mode register */
EXTERN	TYPE_BYTE		__io(0x002B)	_T01MOD;		/* Timer counter 01 mode register */
EXTERN	TYPE_BYTE		__io(0x002C)	_T001CR;		/* Timer counter 001 control register */
EXTERN	TYPE_WORD		__io(0x002D)	_TA0DRA;
#define	TA0DRA	_TA0DRA.word
#define	TA0DRAL	_TA0DRA.byte[0]							/* Timer counter A0 register AL */
#define	TA0DRAH	_TA0DRA.byte[1]							/* Timer counter A0 register AH */
EXTERN	TYPE_WORD		__io(0x002F)	_TA0DRB;
#define	TA0DRB	_TA0DRB.word
#define	TA0DRBL	_TA0DRB.byte[0]							/* Timer counter A0 register BL */
#define	TA0DRBH	_TA0DRB.byte[1]							/* Timer counter A0 register BH */
EXTERN	TYPE_BYTE		__io(0x0031)	_TA0MOD;		/* Timer counter A0 mode register */
EXTERN	TYPE_BYTE		__io(0x0032)	_TA0CR;			/* Timer counter A0 control register */
EXTERN	TYPE_BYTE		__io(0x0033)	_TA0SR;			/* Timer counter A0 status register */
EXTERN	TYPE_BYTE		__io(0x0034)	_ADCCR1;		/* AD converter control register 1 */
EXTERN	TYPE_BYTE		__io(0x0035)	_ADCCR2;		/* AD converter control register 2 */
EXTERN	unsigned char	__io(0x0036)	ADCDRL;			/* AD converted value register (lower side) */
EXTERN	unsigned char	__io(0x0037)	ADCDRH;			/* AD converted value register (upper side) */
EXTERN	TYPE_BYTE		__io(0x0038)	_DVOCR;			/* Divider output control register */
EXTERN	TYPE_BYTE		__io(0x0039)	_TBTCR;			/* Time base timer control register */
EXTERN	TYPE_WORD		__io(0x003A)	_EIR;
#define	EIR	_EIR.word
#define	EIRL	_EIR.byte[0]							/* Interrupt enable register (EIRL) */
#define	EIRH	_EIR.byte[1]							/* Interrupt enable register (EIRH) */
EXTERN	TYPE_WORD	    __io(0x003C)	_EEIR;
#define	EEIR	_EEIR.word
#define	EIRE	_EEIR.byte[0]							/* Interrupt enable register (EIRE) */
#define	EIRD	_EEIR.byte[1]							/* Interrupt enable register (EIRD) */
/*---   (0x003E)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x003F)	_PSW;			/* Program status word */

/***[SFR3]******************************************************/
/*---   (0x0E40)    Reserved   ---*/
/*---   (0x0E41)    Reserved   ---*/
/*---   (0x0E42)    Reserved   ---*/
/*---   (0x0E43)    Reserved   ---*/
/*---   (0x0E44)    Reserved   ---*/
/*---   (0x0E45)    Reserved   ---*/
/*---   (0x0E46)    Reserved   ---*/
/*---   (0x0E47)    Reserved   ---*/
/*---   (0x0E48)    Reserved   ---*/
/*---   (0x0E49)    Reserved   ---*/
/*---   (0x0E4A)    Reserved   ---*/
/*---   (0x0E4B)    Reserved   ---*/
/*---   (0x0E4C)    Reserved   ---*/
/*---   (0x0E4D)    Reserved   ---*/
/*---   (0x0E4E)    Reserved   ---*/
/*---   (0x0E4F)    Reserved   ---*/
/*---   (0x0E50)    Reserved   ---*/
/*---   (0x0E51)    Reserved   ---*/
/*---   (0x0E52)    Reserved   ---*/
/*---   (0x0E53)    Reserved   ---*/
/*---   (0x0E54)    Reserved   ---*/
/*---   (0x0E55)    Reserved   ---*/
/*---   (0x0E56)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0E57)	_UATCNG;		/* UART IN/OUTPUT change register */
/*---   (0x0E58)    Reserved   ---*/
/*---   (0x0E59)    Reserved   ---*/
/*---   (0x0E5A)    Reserved   ---*/
/*---   (0x0E5B)    Reserved   ---*/
/*---   (0x0E5C)    Reserved   ---*/
/*---   (0x0E5D)    Reserved   ---*/
/*---   (0x0E5E)    Reserved   ---*/
/*---   (0x0E5F)    Reserved   ---*/
/*---   (0x0E60)    Reserved   ---*/
/*---   (0x0E61)    Reserved   ---*/
/*---   (0x0E62)    Reserved   ---*/
/*---   (0x0E63)    Reserved   ---*/
/*---   (0x0E64)    Reserved   ---*/
/*---   (0x0E65)    Reserved   ---*/
/*---   (0x0E66)    Reserved   ---*/
/*---   (0x0E67)    Reserved   ---*/
/*---   (0x0E68)    Reserved   ---*/
/*---   (0x0E69)    Reserved   ---*/
/*---   (0x0E6A)    Reserved   ---*/
/*---   (0x0E6B)    Reserved   ---*/
/*---   (0x0E6C)    Reserved   ---*/
/*---   (0x0E6D)    Reserved   ---*/
/*---   (0x0E6E)    Reserved   ---*/
/*---   (0x0E6F)    Reserved   ---*/
/*---   (0x0E70)    Reserved   ---*/
/*---   (0x0E71)    Reserved   ---*/
/*---   (0x0E72)    Reserved   ---*/
/*---   (0x0E73)    Reserved   ---*/
/*---   (0x0E74)    Reserved   ---*/
/*---   (0x0E75)    Reserved   ---*/
/*---   (0x0E76)    Reserved   ---*/
/*---   (0x0E77)    Reserved   ---*/
/*---   (0x0E78)    Reserved   ---*/
/*---   (0x0E79)    Reserved   ---*/
/*---   (0x0E7A)    Reserved   ---*/
/*---   (0x0E7B)    Reserved   ---*/
/*---   (0x0E7C)    Reserved   ---*/
/*---   (0x0E7D)    Reserved   ---*/
/*---   (0x0E7E)    Reserved   ---*/
/*---   (0x0E7F)    Reserved   ---*/
/*---   (0x0E80)    Reserved   ---*/
/*---   (0x0E81)    Reserved   ---*/
/*---   (0x0E82)    Reserved   ---*/
/*---   (0x0E83)    Reserved   ---*/
/*---   (0x0E84)    Reserved   ---*/
/*---   (0x0E85)    Reserved   ---*/
/*---   (0x0E86)    Reserved   ---*/
/*---   (0x0E87)    Reserved   ---*/
/*---   (0x0E88)    Reserved   ---*/
/*---   (0x0E89)    Reserved   ---*/
/*---   (0x0E8A)    Reserved   ---*/
/*---   (0x0E8B)    Reserved   ---*/
/*---   (0x0E8C)    Reserved   ---*/
/*---   (0x0E8D)    Reserved   ---*/
/*---   (0x0E8E)    Reserved   ---*/
/*---   (0x0E8F)    Reserved   ---*/
/*---   (0x0E90)    Reserved   ---*/
/*---   (0x0E91)    Reserved   ---*/
/*---   (0x0E92)    Reserved   ---*/
/*---   (0x0E93)    Reserved   ---*/
/*---   (0x0E94)    Reserved   ---*/
/*---   (0x0E95)    Reserved   ---*/
/*---   (0x0E96)    Reserved   ---*/
/*---   (0x0E97)    Reserved   ---*/
/*---   (0x0E98)    Reserved   ---*/
/*---   (0x0E99)    Reserved   ---*/
/*---   (0x0E9A)    Reserved   ---*/
/*---   (0x0E9B)    Reserved   ---*/
/*---   (0x0E9C)    Reserved   ---*/
/*---   (0x0E9D)    Reserved   ---*/
/*---   (0x0E9E)    Reserved   ---*/
/*---   (0x0E9F)    Reserved   ---*/
/*---   (0x0EA0)    Reserved   ---*/
/*---   (0x0EA1)    Reserved   ---*/
/*---   (0x0EA2)    Reserved   ---*/
/*---   (0x0EA3)    Reserved   ---*/
/*---   (0x0EA4)    Reserved   ---*/
/*---   (0x0EA5)    Reserved   ---*/
/*---   (0x0EA6)    Reserved   ---*/
/*---   (0x0EA7)    Reserved   ---*/
/*---   (0x0EA8)    Reserved   ---*/
/*---   (0x0EA9)    Reserved   ---*/
/*---   (0x0EAA)    Reserved   ---*/
/*---   (0x0EAB)    Reserved   ---*/
/*---   (0x0EAC)    Reserved   ---*/
/*---   (0x0EAD)    Reserved   ---*/
/*---   (0x0EAE)    Reserved   ---*/
/*---   (0x0EAF)    Reserved   ---*/
/*---   (0x0EB0)    Reserved   ---*/
/*---   (0x0EB1)    Reserved   ---*/
/*---   (0x0EB2)    Reserved   ---*/
/*---   (0x0EB3)    Reserved   ---*/
/*---   (0x0EB4)    Reserved   ---*/
/*---   (0x0EB5)    Reserved   ---*/
/*---   (0x0EB6)    Reserved   ---*/
/*---   (0x0EB7)    Reserved   ---*/
/*---   (0x0EB8)    Reserved   ---*/
/*---   (0x0EB9)    Reserved   ---*/
/*---   (0x0EBA)    Reserved   ---*/
/*---   (0x0EBB)    Reserved   ---*/
/*---   (0x0EBC)    Reserved   ---*/
/*---   (0x0EBD)    Reserved   ---*/
/*---   (0x0EBE)    Reserved   ---*/
/*---   (0x0EBF)    Reserved   ---*/
/*---   (0x0EC0)    Reserved   ---*/
/*---   (0x0EC1)    Reserved   ---*/
/*---   (0x0EC2)    Reserved   ---*/
/*---   (0x0EC3)    Reserved   ---*/
/*---   (0x0EC4)    Reserved   ---*/
/*---   (0x0EC5)    Reserved   ---*/
/*---   (0x0EC6)    Reserved   ---*/
/*---   (0x0EC7)    Reserved   ---*/
/*---   (0x0EC8)    Reserved   ---*/
/*---   (0x0EC9)    Reserved   ---*/
/*---   (0x0ECA)    Reserved   ---*/
/*---   (0x0ECB)    Reserved   ---*/
/*---   (0x0ECC)    Reserved   ---*/
/*---   (0x0ECD)    Reserved   ---*/
/*---   (0x0ECE)    Reserved   ---*/
/*---   (0x0ECF)    Reserved   ---*/
/*---   (0x0ED0)    Reserved   ---*/
/*---   (0x0ED1)    Reserved   ---*/
/*---   (0x0ED2)    Reserved   ---*/
/*---   (0x0ED3)    Reserved   ---*/
/*---   (0x0ED4)    Reserved   ---*/
/*---   (0x0ED5)    Reserved   ---*/
/*---   (0x0ED6)    Reserved   ---*/
/*---   (0x0ED7)    Reserved   ---*/
/*---   (0x0ED8)    Reserved   ---*/
/*---   (0x0ED9)    Reserved   ---*/
/*---   (0x0EDA)    Reserved   ---*/
/*---   (0x0EDB)    Reserved   ---*/
/*---   (0x0EDC)    Reserved   ---*/
/*---   (0x0EDD)    Reserved   ---*/
/*---   (0x0EDE)    Reserved   ---*/
/*---   (0x0EDF)    Reserved   ---*/
/*---   (0x0EE0)    Reserved   ---*/
/*---   (0x0EE1)    Reserved   ---*/
/*---   (0x0EE2)    Reserved   ---*/
/*---   (0x0EE3)    Reserved   ---*/
/*---   (0x0EE4)    Reserved   ---*/
/*---   (0x0EE5)    Reserved   ---*/
/*---   (0x0EE6)    Reserved   ---*/
/*---   (0x0EE7)    Reserved   ---*/
/*---   (0x0EE8)    Reserved   ---*/
/*---   (0x0EE9)    Reserved   ---*/
/*---   (0x0EEA)    Reserved   ---*/
/*---   (0x0EEB)    Reserved   ---*/
/*---   (0x0EEC)    Reserved   ---*/
/*---   (0x0EED)    Reserved   ---*/
/*---   (0x0EEE)    Reserved   ---*/
/*---   (0x0EEF)    Reserved   ---*/
/*---   (0x0EF0)    Reserved   ---*/
/*---   (0x0EF1)    Reserved   ---*/
/*---   (0x0EF2)    Reserved   ---*/
/*---   (0x0EF3)    Reserved   ---*/
/*---   (0x0EF4)    Reserved   ---*/
/*---   (0x0EF5)    Reserved   ---*/
/*---   (0x0EF6)    Reserved   ---*/
/*---   (0x0EF7)    Reserved   ---*/
/*---   (0x0EF8)    Reserved   ---*/
/*---   (0x0EF9)    Reserved   ---*/
/*---   (0x0EFA)    Reserved   ---*/
/*---   (0x0EFB)    Reserved   ---*/
/*---   (0x0EFC)    Reserved   ---*/
/*---   (0x0EFD)    Reserved   ---*/
/*---   (0x0EFE)    Reserved   ---*/
/*---   (0x0EFF)    Reserved   ---*/

/***[SFR2]******************************************************/
/*---   (0x0F00)    Reserved   ---*/
/*---   (0x0F01)    Reserved   ---*/
/*---   (0x0F02)    Reserved   ---*/
/*---   (0x0F03)    Reserved   ---*/
/*---   (0x0F04)    Reserved   ---*/
/*---   (0x0F05)    Reserved   ---*/
/*---   (0x0F06)    Reserved   ---*/
/*---   (0x0F07)    Reserved   ---*/
/*---   (0x0F08)    Reserved   ---*/
/*---   (0x0F09)    Reserved   ---*/
/*---   (0x0F0A)    Reserved   ---*/
/*---   (0x0F0B)    Reserved   ---*/
/*---   (0x0F0C)    Reserved   ---*/
/*---   (0x0F0D)    Reserved   ---*/
/*---   (0x0F0E)    Reserved   ---*/
/*---   (0x0F0F)    Reserved   ---*/
/*---   (0x0F10)    Reserved   ---*/
/*---   (0x0F11)    Reserved   ---*/
/*---   (0x0F12)    Reserved   ---*/
/*---   (0x0F13)    Reserved   ---*/
/*---   (0x0F14)    Reserved   ---*/
/*---   (0x0F15)    Reserved   ---*/
/*---   (0x0F16)    Reserved   ---*/
/*---   (0x0F17)    Reserved   ---*/
/*---   (0x0F18)    Reserved   ---*/
/*---   (0x0F19)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0F1A)	_P0CR;			/* Port P0 input/output control */
EXTERN	TYPE_BYTE		__io(0x0F1B)	_P1CR;			/* Port P1 input/output control */
EXTERN	TYPE_BYTE		__io(0x0F1C)	_P2CR;			/* Port P2 input/output control */
/*---   (0x0F1D)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0F1E)	_P4CR;			/* Port P4 input/output control */
EXTERN	TYPE_BYTE		__io(0x0F1F)	_P5CR;			/* Port P5 input/output control */
/*---   (0x0F20)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0F21)	_P7CR;			/* Port P7 input/output control */
EXTERN	TYPE_BYTE		__io(0x0F22)	_P8CR;			/* Port P8 input/output control */
EXTERN	TYPE_BYTE		__io(0x0F23)	_P9CR;			/* Port P9 input/output control */
/*---   (0x0F24)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0F25)	_PBCR;			/* Port PB input/output control */
/*---   (0x0F26)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0F27)	_P0PU;			/* Port P0 built-in pull-up resistor control */
EXTERN	TYPE_BYTE		__io(0x0F28)	_P1PU;			/* Port P1 built-in pull-up resistor control */
EXTERN	TYPE_BYTE		__io(0x0F29)	_P2PU;			/* Port P2 built-in pull-up resistor control */
/*---   (0x0F2A)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0F2B)	_P4PU;			/* Port P4 built-in pull-up resistor control */
/*---   (0x0F2C)    Reserved   ---*/
/*---   (0x0F2D)    Reserved   ---*/
/*---   (0x0F2E)    Reserved   ---*/
/*---   (0x0F2F)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0F30)	_P9PU;			/* Port P9 built-in pull-up resistor control */
/*---   (0x0F31)    Reserved   ---*/
/*---   (0x0F32)    Reserved   ---*/
/*---   (0x0F33)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0F34)	_P0FC;			/* Port P0 function control */
/*---   (0x0F35)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0F36)	_P2FC;			/* Port P2 function control */
/*---   (0x0F37)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0F38)	_P4FC;			/* Port P4 function control */
EXTERN	TYPE_BYTE		__io(0x0F39)	_P5FC;			/* Port P5 function control */
/*---   (0x0F3A)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0F3B)	_P7FC;			/* Port P7 function control */
EXTERN	TYPE_BYTE		__io(0x0F3C)	_P8FC;			/* Port P8 function control */
EXTERN	TYPE_BYTE		__io(0x0F3D)	_P9FC;			/* Port P9 function control */
/*---   (0x0F3E)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0F3F)	_PBFC;			/* Port PB function control */
/*---   (0x0F40)    Reserved   ---*/
/*---   (0x0F41)    Reserved   ---*/
/*---   (0x0F42)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0F43)	_P2OUTCR;		/* Port P2 output control */
/*---   (0x0F44)    Reserved   ---*/
/*---   (0x0F45)    Reserved   ---*/
/*---   (0x0F46)    Reserved   ---*/
/*---   (0x0F47)    Reserved   ---*/
/*---   (0x0F48)    Reserved   ---*/
/*---   (0x0F49)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0F4A)	_P9OUTCR;		/* Port P9 output control */
/*---   (0x0F4B)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0F4C)	_PBOUTCR;		/* Port PB output control */
/*---   (0x0F4D)    Reserved   ---*/
/*---   (0x0F4E)    Reserved   ---*/
/*---   (0x0F4F)    Reserved   ---*/
/*---   (0x0F50)    Reserved   ---*/
/*---   (0x0F51)    Reserved   ---*/
/*---   (0x0F52)    Reserved   ---*/
/*---   (0x0F53)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0F54)	_UART1CR1;		/* UART1 control register 1 */
EXTERN	TYPE_BYTE		__io(0x0F55)	_UART1CR2;		/* UART1 control register 2 */
EXTERN	unsigned char	__io(0x0F56)	UART1DR;		/* UART1 baud rate register */
EXTERN	TYPE_BYTE		__io(0x0F57)	_UART1SR;		/* UART1 status register */
EXTERN	TYPE_BYTE		__io(0x0F58)	U1BF;
#define	TD1BUF	U1BF.byte								/* UART1 transmit data buffer */
#define	RD1BUF	U1BF.byte								/* UART1 receive data buffer */
/*---   (0x0F59)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0F5A)	_UART2CR1;		/* UART2 control register 1 */
EXTERN	TYPE_BYTE		__io(0x0F5B)	_UART2CR2;		/* UART2 control register 2 */
EXTERN	unsigned char	__io(0x0F5C)	UART2DR;		/* UART2 baud rate register */
EXTERN	TYPE_BYTE		__io(0x0F5D)	_UART2SR;		/* UART2 status register */
EXTERN	TYPE_BYTE		__io(0x0F5E)	U2BF;
#define	TD2BUF	U2BF.byte								/* UART2 transmit data buffer */
#define	RD2BUF	U2BF.byte								/* UART2 receive data buffer */
/*---   (0x0F5F)    Reserved   ---*/
/*---   (0x0F60)    Reserved   ---*/
/*---   (0x0F61)    Reserved   ---*/
/*---   (0x0F62)    Reserved   ---*/
/*---   (0x0F63)    Reserved   ---*/
/*---   (0x0F64)    Reserved   ---*/
/*---   (0x0F65)    Reserved   ---*/
/*---   (0x0F66)    Reserved   ---*/
/*---   (0x0F67)    Reserved   ---*/
/*---   (0x0F68)    Reserved   ---*/
/*---   (0x0F69)    Reserved   ---*/
/*---   (0x0F6A)    Reserved   ---*/
/*---   (0x0F6B)    Reserved   ---*/
/*---   (0x0F6C)    Reserved   ---*/
/*---   (0x0F6D)    Reserved   ---*/
/*---   (0x0F6E)    Reserved   ---*/
/*---   (0x0F6F)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0F70)	_SIO1CR;		/* Serial interface 1 control register */
EXTERN	TYPE_BYTE		__io(0x0F71)	_SIO1SR;		/* Serial interface 1 status register */
EXTERN	unsigned char	__io(0x0F72)	SIO1BUF;		/* Serial interface 1 buffer register */
/*---   (0x0F73)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0F74)	_POFFCR0;		/* Low power consumption register 0 */
EXTERN	TYPE_BYTE		__io(0x0F75)	_POFFCR1;		/* Low power consumption register 1 */
EXTERN	TYPE_BYTE		__io(0x0F76)	_POFFCR2;		/* Low power consumption register 2 */
EXTERN	TYPE_BYTE		__io(0x0F77)	_POFFCR3;		/* Low power consumption register 3 */
/*---   (0x0F78)    Reserved   ---*/
/*---   (0x0F79)    Reserved   ---*/
/*---   (0x0F7A)    Reserved   ---*/
/*---   (0x0F7B)    Reserved   ---*/
/*---   (0x0F7C)    Reserved   ---*/
/*---   (0x0F7D)    Reserved   ---*/
/*---   (0x0F7E)    Reserved   ---*/
/*---   (0x0F7F)    Reserved   ---*/
/*---   (0x0F80)    Reserved   ---*/
/*---   (0x0F81)    Reserved   ---*/
/*---   (0x0F82)    Reserved   ---*/
/*---   (0x0F83)    Reserved   ---*/
/*---   (0x0F84)    Reserved   ---*/
/*---   (0x0F85)    Reserved   ---*/
/*---   (0x0F86)    Reserved   ---*/
/*---   (0x0F87)    Reserved   ---*/
EXTERN	unsigned char	__io(0x0F88)	T02REG;			/* Timer register 02 */
EXTERN	unsigned char	__io(0x0F89)	T03REG;			/* Timer register 03 */
EXTERN	unsigned char	__io(0x0F8A)	T02PWM;			/* Timer register 02 */
EXTERN	unsigned char	__io(0x0F8B)	T03PWM;			/* Timer register 03 */
EXTERN	TYPE_BYTE		__io(0x0F8C)	_T02MOD;		/* Timer counter 02 mode register */
EXTERN	TYPE_BYTE		__io(0x0F8D)	_T03MOD;		/* Timer counter 03 mode register */
EXTERN	TYPE_BYTE		__io(0x0F8E)	_T023CR;		/* Timer counter 023 control register */
/*---   (0x0F8F)    Reserved   ---*/
/*---   (0x0F90)    Reserved   ---*/
/*---   (0x0F91)    Reserved   ---*/
/*---   (0x0F92)    Reserved   ---*/
/*---   (0x0F93)    Reserved   ---*/
/*---   (0x0F94)    Reserved   ---*/
/*---   (0x0F95)    Reserved   ---*/
/*---   (0x0F96)    Reserved   ---*/
/*---   (0x0F97)    Reserved   ---*/
/*---   (0x0F98)    Reserved   ---*/
/*---   (0x0F99)    Reserved   ---*/
/*---   (0x0F9A)    Reserved   ---*/
/*---   (0x0F9B)    Reserved   ---*/
/*---   (0x0F9C)    Reserved   ---*/
/*---   (0x0F9D)    Reserved   ---*/
/*---   (0x0F9E)    Reserved   ---*/
/*---   (0x0F9F)    Reserved   ---*/
/*---   (0x0FA0)    Reserved   ---*/
/*---   (0x0FA1)    Reserved   ---*/
/*---   (0x0FA2)    Reserved   ---*/
/*---   (0x0FA3)    Reserved   ---*/
/*---   (0x0FA4)    Reserved   ---*/
/*---   (0x0FA5)    Reserved   ---*/
/*---   (0x0FA6)    Reserved   ---*/
/*---   (0x0FA7)    Reserved   ---*/
EXTERN	TYPE_WORD		__io(0x0FA8)	_TA1DRA;
#define	TA1DRA	_TA1DRA.word
#define	TA1DRAL	_TA1DRA.byte[0]							/* Timer counter A1 register AL */
#define	TA1DRAH	_TA1DRA.byte[1]							/* Timer counter A1 register AH */
EXTERN	TYPE_WORD		__io(0x0FAA)	_TA1DRB;
#define	TA1DRB	_TA1DRB.word
#define	TA1DRBL	_TA1DRB.byte[0]							/* Timer counter A1 register BL */
#define	TA1DRBH	_TA1DRB.byte[1]							/* Timer counter A1 register BH */
EXTERN	TYPE_BYTE		__io(0x0FAC)	_TA1MOD;		/* Timer counter A1 mode register */
EXTERN	TYPE_BYTE		__io(0x0FAD)	_TA1CR;			/* Timer counter A1 control register */
EXTERN	TYPE_BYTE		__io(0x0FAE)	_TA1SR;			/* Timer counter A1 status register */
/*---   (0x0FAF)    Reserved   ---*/
/*---   (0x0FB0)    Reserved   ---*/
/*---   (0x0FB1)    Reserved   ---*/
/*---   (0x0FB2)    Reserved   ---*/
/*---   (0x0FB3)    Reserved   ---*/
/*---   (0x0FB4)    Reserved   ---*/
/*---   (0x0FB5)    Reserved   ---*/
/*---   (0x0FB6)    Reserved   ---*/
/*---   (0x0FB7)    Reserved   ---*/
/*---   (0x0FB8)    Reserved   ---*/
/*---   (0x0FB9)    Reserved   ---*/
/*---   (0x0FBA)    Reserved   ---*/
/*---   (0x0FBB)    Reserved   ---*/
/*---   (0x0FBC)    Reserved   ---*/
/*---   (0x0FBD)    Reserved   ---*/
/*---   (0x0FBE)    Reserved   ---*/
/*---   (0x0FBF)    Reserved   ---*/
/*---   (0x0FC0)    Reserved   ---*/
/*---   (0x0FC1)    Reserved   ---*/
/*---   (0x0FC2)    Reserved   ---*/
/*---   (0x0FC3)    Reserved   ---*/
EXTERN TYPE_BYTE		__io(0x0FC4)	_KWUCR0;		/* Key-on wakeup control register 0 */
EXTERN TYPE_BYTE		__io(0x0FC5)	_KWUCR1;		/* Key-on wakeup control register 1 */
EXTERN TYPE_BYTE		__io(0x0FC6)	_VDCR1;			/* Voltage detection control register 1 */
EXTERN TYPE_BYTE		__io(0x0FC7)	_VDCR2;			/* Voltage detection control register 2 */
EXTERN TYPE_BYTE		__io(0x0FC8)	_RTCCR;			/* Real time clock control register */
/*---   (0x0FC9)    Reserved   ---*/
/*---   (0x0FCA)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0FCB)	_SERSEL;		/* Serial interface selection control register */
EXTERN	TYPE_BYTE		__io(0x0FCC)	_IRSTSR;		/* Internal factor reset detection status register */
EXTERN	TYPE_BYTE		__io(0x0FCD)	_WUCCR;			/* Warm-up counter control register */

EXTERN	unsigned char	__io(0x0FCE)	WUCDR;			/* Warm-up counter data register */
EXTERN	TYPE_BYTE		__io(0x0FCF)	_CGCR;			/* Clock gear control register */
EXTERN	TYPE_BYTE		__io(0x0FD0)	_FLSCR1;		/* Flash memory control register 1 */

EXTERN	TYPE_BYTE		__io(0x0FD1)	FLCM;
#define	FLSCR2 FLCM.byte								/* Flash memory control register 2 */
#define	FLSCRM FLCM.byte								/* Flash memory control register 1 monitor */
EXTERN	TYPE_BYTE		__io(0x0FD2)	_FLSSTB;		/* Flash memory standby control register */
/*---   (0x0FD3)    Reserved   ---*/
EXTERN	TYPE_BYTE	__io(0x0FD4)	_WDCTR;				/* Watchdog timer control register */
EXTERN	unsigned char	__io(0x0FD5)	WDCDR;			/* Watchdog timer control code register */
EXTERN	unsigned char	__io(0x0FD6)	WDCNT;			/* 8-bit up counter monitor */
EXTERN	TYPE_BYTE		__io(0x0FD7)	_WDST;			/* Watchdog timer status */
EXTERN	TYPE_BYTE		__io(0x0FD8)	_EINTCR1;		/* External interrupt control register 1 */
EXTERN	TYPE_BYTE		__io(0x0FD9)	_EINTCR2;		/* External interrupt control register 2 */
EXTERN	TYPE_BYTE		__io(0x0FDA)	_EINTCR3;		/* External interrupt control register 3 */
EXTERN	TYPE_BYTE		__io(0x0FDB)	_EINTCR4;		/* External interrupt control register 4 */
EXTERN	TYPE_BYTE		__io(0x0FDC)	_SYSCR1;		/* System control register 1 */
EXTERN	TYPE_BYTE		__io(0x0FDD)	_SYSCR2;		/* System control register 2 */
EXTERN	TYPE_BYTE		__io(0x0FDE)	_SYSCR3;		/* System control register 3 */
EXTERN	TYPE_BYTE		__io(0x0FDF)	SYS4;
#define	SYSCR4	SYS4.byte								/* System control register 4 */
#define	SYSSR4	SYS4.byte								/* System control status register 4 */
EXTERN	TYPE_WORD		__io(0x0FE0)	_IL;
#define	IL	_IL.word
#define	ILL	_IL.byte[0]									/* Interrupt latch (ILL) */
#define	ILH	_IL.byte[1]									/* Interrupt latch (ILH) */
EXTERN	TYPE_WORD		__io(0x0FE2)	_EIL;
#define	EIL	_EIL.word
#define	ILE	_EIL.byte[0]								/* Interrupt latch (ILE) */
#define	ILD	_EIL.byte[1]								/* Interrupt latch (ILD) */
/*---   (0x0FE4)    Reserved   ---*/
/*---   (0x0FE5)    Reserved   ---*/
/*---   (0x0FE6)    Reserved   ---*/
/*---   (0x0FE7)    Reserved   ---*/
/*---   (0x0FE8)    Reserved   ---*/
/*---   (0x0FE9)    Reserved   ---*/
/*---   (0x0FEA)    Reserved   ---*/
/*---   (0x0FEB)    Reserved   ---*/
/*---   (0x0FEC)    Reserved   ---*/
/*---   (0x0FED)    Reserved   ---*/
/*---   (0x0FEE)    Reserved   ---*/
/*---   (0x0FEF)    Reserved   ---*/
EXTERN	TYPE_BYTE		__io(0x0FF0)	_ILPRS1;		/* Interrupt priority change control register 1 */
EXTERN	TYPE_BYTE		__io(0x0FF1)	_ILPRS2;		/* Interrupt priority change control register 2 */
EXTERN	TYPE_BYTE		__io(0x0FF2)	_ILPRS3;		/* Interrupt priority change control register 3 */
EXTERN	TYPE_BYTE		__io(0x0FF3)	_ILPRS4;		/* Interrupt priority change control register 4 */
EXTERN	TYPE_BYTE		__io(0x0FF4)	_ILPRS5;		/* Interrupt priority change control register 5 */
EXTERN	TYPE_BYTE		__io(0x0FF5)	_ILPRS6;		/* Interrupt priority change control register 6 */
/*---   (0x0FF6)    Reserved   ---*/
/*---   (0x0FF7)    Reserved   ---*/
/*---   (0x0FF8)    Reserved   ---*/
/*---   (0x0FF9)    Reserved   ---*/
/*---   (0x0FFA)    Reserved   ---*/
/*---   (0x0FFB)    Reserved   ---*/
/*---   (0x0FFC)    Reserved   ---*/
/*---   (0x0FFD)    Reserved   ---*/
/*---   (0x0FFE)    Reserved   ---*/
/*---   (0x0FFF)    Reserved   ---*/

#endif	/* IO89FS63B_H */
/*-eof-*/
