/** 
 *******************************************************************************
 * @file    main.c
 * @brief   ADC_REPEAT Sample Application.
 * @version V1.0.4.0
 * $Date:: 2019-09-18 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdio.h>
#include	<string.h>
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"
#include	"uart_io.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup ADC_REPEAT ADC_REPEAT Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_REPEAT_Private_macro ADC_REPEAT Private Macro
 *  @{
 */
/** 
 *  @defgroup RTC_RTCEN RTC Enable
 *  @brief    RTC Control Enable.
 *  @{
 */
#define RTC_RTCEN_ENABLE				((UINT8_t)0x20)			/*!< RTC enable */
/**
 *  @}
 */ /* End of group RTC_RTCEN */
/** 
 *  @defgroup RTC_RTCSEL Macro Definition
 *  @brief    RTC Interval.
 *  @{
 */
#define RTC_SEL_2_15_FS					((UINT8_t)0x00)			/*!< RTC Interval(= 2^15/fs = 1.000[s]) */
/**
 *  @}
 */ /* End of group RTC_RTCSEL Macro Definition */
/** 
 *  @defgroup RTC_RTCRUN RTC Start
 *  @brief    RTC Start.
 *  @{
 */
#define RTC_RTCRUN_START				((UINT8_t)0x01)			/*!< RTC Start */
/**
 *  @}
 */ /* End of group RTC_RTCRUN */
/** 
 *  @defgroup ADC_ADRS ADC Convert Start
 *  @brief    ADC Convert Start.
 *  @{
 */
#define ADC_ADRS_START					((UINT8_t)0x01)			/*!< ADC Convert Start.  */
/**
 *  @}
 */ /* End of group ADC_ADRS */
/** 
 *  @defgroup ADC_AMD MODE Setting
 *  @brief    MODE Setting.
 *  @{
 */
#define ADC_AMD_REPEAT					((UINT8_t)0x60)			/*!< ADC Mode:  Repeat Mode  */
/**
 *  @}
 */ /* End of group ADC_AMD */
/** 
 *  @defgroup ADC_AINEN ADC AINEN Setting
 *  @brief    ADC AINEN Setting.
 *  @{
 */
#define ADC_AINEN_ENABLE				((UINT8_t)0x10)			/*!< ADC Input:  Enable  */
/**
 *  @}
 */ /* End of group ADC_AINEN */
/** 
 *  @defgroup ADC_SAIN ADC AINEN Setting
 *  @brief    ADC AINEN Setting.
 *  @{
 */
#define ADC_CHANNEL_AIN7				((UINT8_t)0x07)			/*!< ADC Ch:  AIN7  */
/**
 *  @}
 */ /* End of group ADC_SAIN */
/** 
 *  @defgroup ADC_REPEAT_TIME ADC Convert Time
 *  @brief    ADC Convert Time.
 *  @{
 */
#define ADC_REPEAT_TIME					((UINT8_t)0x03)			/*!< ADC Convert time: 312/fcgck */
/**
 *  @}
 */ /* End of group ADC_REPEAT_TIME */
/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_SINGLE_Private_define ADC Private Define
 *  @{
 */
const UINT8_t disp_tbl[][48] = {
	{"10bit AD Converter Sample Program\n"},
	{"----------------------------------------------\n"},
	{"Operating mode: Repeat\n"},
	{"Conversion time Select: 312/fcgck\n"},
	{"AD converted value:\n"}
};

#define TX_BUFF_ALL			(UINT8_t)5
/** 
 *  @}
 */ /* End of group ADC_SINGLE_Private_define */
/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_REPEAT_Private_variables ADC Private Variables
 *  @{
 */
static UINT8_t adc_repeat_count = (UINT8_t)0;
static UINT16_t adc_repeat_data[10];
static UINT16_t adc_repeat_data_ave = (UINT16_t)0;
static UINT32_t adc_data_total = (UINT32_t)0;
static UINT8_t timer_flag = (UINT8_t)0;
static UINT8_t end_flag = (UINT8_t)0;
/**
 *  @}
 */ /* End of group ADC_REPEAT_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_REPEAT_Private_fuctions ADC Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void ADC_Config(void);
static void ADC_Start(void);
static void ADC_Get_Data(void);
static void RTC_Config(void);
static void RTC_Start(void);

/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Port Input/Output Control(AIN7) : Input Setting */
	P4CR = 0x00;
	/* (2) Port Function Control(AIN7) : AIN7 Setting */
	P4FC = 0x80;
}

/*--------------------------------------------------*/
/** 
  * @brief  ADC_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void ADC_Config(void)
{
	/* Enable interrupt of ADC */
	__DI();
	EIR_INTADC = SET;
	__EI();
	/* AD Converter Register 1 Setting */
	/* (1) Select AIN Chennel,  ADCCR1[3:0]=0111(AIN7 Select) */
	/* (2) ADC Enable/Disable,  ADCCR1[4]=1(Enable)  */
	/* (3) Select Mode,         ADCCR1[6:5]=11(Repeat Mode)  */
	ADCCR1 = ADC_CHANNEL_AIN7 | ADC_AINEN_ENABLE | ADC_AMD_REPEAT;
	
	/* AD Converter Register 2 Setting */
	/* Select Convert Time, ADCCR2[2:0]=011(312/fcgck) */
	ADCCR2 = ADC_REPEAT_TIME;
}

/*--------------------------------------------------*/
/** 
  * @brief  RTC_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void RTC_Config(void)
{
	/* (1) Enable RTC Control */
	/* (2) Enable interrupt of RTC */
	/* (3) Setting interrupt frequency,RTCCR[3:1]=(2^15/fs) */
	POFFCR2 = RTC_RTCEN_ENABLE;		/* (1) */
	__DI();
	EIR_INTRTC = 1;					/* (2) */
	__EI();
	RTCCR  = RTC_SEL_2_15_FS;		/* (3) */
}
/*--------------------------------------------------*/
/** 
  * @brief  RTC_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void RTC_Start(void)
{
	/* RTCRUN = 1 */
	RTCRUN = RTC_RTCRUN_START;
}

/*--------------------------------------------------*/
/** 
  * @brief  ADC_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void ADC_Start(void)
{
	adc_repeat_count = (UINT8_t)0;
	adc_data_total = (UINT32_t)0;
	end_flag = CLEAR;
	
	/* ADRS = 1 */
	ADRS = ADC_ADRS_START;
}
/*--------------------------------------------------*/
/** 
  * @brief  ADC_Get_Data
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void ADC_Get_Data(void)
{
	UINT16_t result;
	
	result = ADCDRL;
	result |= (UINT16_t)(ADCDRH << 8);			/* output the top 8 bits in 10 bits */
	adc_repeat_data[adc_repeat_count] = result;
	
	adc_data_total += adc_repeat_data[adc_repeat_count];
}

/**
 *  @}
 */ /* End of group ADC_REPEAT_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_REPEAT_Exported_functions ADC Exported Functions
 *  @{
 */
 
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	UINT8_t i;
	UINT8_t j;
	UINT8_t t_state = (UINT8_t)0;
	UINT8_t t_len = (UINT8_t)0;
	UINT8_t a = (UINT8_t)0;
	
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	
	RTC_Config();
	RTC_Start();
	/*------------------------*/
	/* UART Setting           */
	/*------------------------*/
	uart_initialize();
	
	for(j=0;j<TX_BUFF_ALL;j++) {
		t_len = sizeof(disp_tbl[j]);
		uart_data_set(t_len);
		for(i=0;i<=t_len;i++) {
			tx_data[i] = disp_tbl[j][i];
		}
		t_state = UART2SR;
		while((t_state & 0x02) != 0) {
			t_state = UART2SR;
		}
		TX_Start();
		TD2BUF = tx_data[write];
		
		for(;;) {
			if(write == 0xFF) {
				break;
			} else {
				/* Do nothing */
			}
		}
	}
	/*------------------------*/
	/* ADC Setting            */
	/*------------------------*/
	ADC_Config();
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
		if(timer_flag == SET) {
			timer_flag = CLEAR;
			ADC_Config();
			ADC_Start();
			
			for(;;){
				if(end_flag == SET) {
					end_flag = CLEAR;
					adc_repeat_data_ave = (UINT16_t)(adc_data_total / 10);
					if(adc_repeat_data_ave > 0x03FF) {
						adc_repeat_data_ave = (UINT16_t)0x03FF;
					} else {
						/* Do nothing */
					}
					break;
				} else {
					/* Do nothing */
				}
			}
			/*------------------------*/
			/* UART Send              */
			/*------------------------*/
			write = 0;
			tx_length = (UINT8_t)6;
			
			tx_data[0] = (CHAR_t)'0';
			tx_data[1] = (CHAR_t)'x';
			
			/* change AD */
			a = (UINT8_t)(adc_repeat_data_ave / 256);
			if(a >= 10) {
				a -= 10;
				a += 65;
			}
			else {
				a += 48;
			}
			tx_data[2] = a;
			a = (UINT8_t)((adc_repeat_data_ave % 256) / 16);
			if(a >= 10) {
				a -= 10;
				a += 65;
			}
			else {
				a += 48;
			}
			tx_data[3] = a;
			a = (UINT8_t)((adc_repeat_data_ave % 256) % 16);
			if(a >= 10) {
				a -= 10;
				a += 65;
			}
			else {
				a += 48;
			}
			tx_data[4] = a;
			tx_data[5] = '\n';
			TX_Start();
			TD2BUF = tx_data[write];
		} else {
			/* Do nothing */
		}
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  INTADC
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntADC(void)
{
	ADC_Get_Data();
	adc_repeat_count++;
	if(adc_repeat_count > 9) {
		/* AD change end */
		end_flag = SET;
		ADCCR1 &= 0x9F;
	} else {
		/* Do nothing */
	}
}
/*--------------------------------------------------*/
/** 
  * @brief  INTRTC
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntRTC(void)
{
	timer_flag = SET;
}

/**
 *  @}
 */ /* End of group ADC_REPEAT_Exported_functions */

/**
 *  @}
 */ /* End of group ADC_REPEAT */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
