/** 
 *******************************************************************************
 * @file    main.c
 * @brief   ADC_SINGLE Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-07-19 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdio.h>
#include	<string.h>
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"
#include	"uart_io.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup ADC_SINGLE ADC_SINGLE Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_SINGLE_Private_macro ADC_SINGLE Private Macro
 *  @{
 */
/** 
 *  @defgroup ADC_ADRS ADC Convert Start
 *  @brief    ADC Convert Start.
 *  @{
 */
#define ADC_ADRS_START					((UINT8_t)0x01)			/*!< ADC Convert Start.  */
/**
 *  @}
 */ /* End of group ADC_ADRS */
/** 
 *  @defgroup ADC_AMD MODE Setting
 *  @brief    MODE Setting.
 *  @{
 */
#define ADC_AMD_SINGLE					((UINT8_t)0x20)			/*!< ADC Mode:  Single Mode  */
/**
 *  @}
 */ /* End of group ADC_AMD */
/** 
 *  @defgroup ADC_AINEN ADC AINEN Setting
 *  @brief    ADC AINEN Setting.
 *  @{
 */
#define ADC_AINEN_ENABLE				((UINT8_t)0x10)			/*!< ADC Input:  Enable  */
/**
 *  @}
 */ /* End of group ADC_AINEN */
/** 
 *  @defgroup ADC_SAIN ADC AINEN Setting
 *  @brief    ADC AINEN Setting.
 *  @{
 */
#define ADC_CHANNEL_AIN7				((UINT8_t)0x07)			/*!< ADC Ch:  AIN7  */
/**
 *  @}
 */ /* End of group ADC_SAIN */
/** 
 *  @defgroup ADC_SINGLE_TIME ADC Convert Time
 *  @brief    ADC Convert Time.
 *  @{
 */
#define ADC_SINGLE_TIME					((UINT8_t)0x02)			/*!< ADC Convert time: 156/fcgck */
/**
 *  @}
 */ /* End of group ADC_SINGLE_TIME */
/** 
 *  @}
 */ /* End of group ADC_SINGLE_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_SINGLE_Private_define ADC Private Define
 *  @{
 */
const UINT8_t disp_tbl[][48] = {
	{"10bit AD Converter Sample Program\n"},
	{"----------------------------------------------\n"},
	{"Operating mode: Single\n"},
	{"Conversion time Select: 156/fcgck\n"},
	{"AD converted value:\n"}
};

#define TX_BUFF_ALL			(UINT8_t)5
/** 
 *  @}
 */ /* End of group ADC_SINGLE_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_SINGLE_Private_variables ADC Private Variables
 *  @{
 */
static UINT16_t adc_single_data = (UINT16_t)0;
/**
 *  @}
 */ /* End of group ADC_SINGLE_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_SINGLE_Private_fuctions ADC Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void ADC_Config(void);
static void ADC_Start(void);
static void ADC_Get_Data(void);

/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Port Function Control(AIN7) : AIN7 Setting */
	P4FC = 0x80;
	/* (2) Port Input/Output Control(AIN7) : Input Setting */
	P4CR = 0x00;
}

/*--------------------------------------------------*/
/** 
  * @brief  ADC_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void ADC_Config(void)
{
	/* AD Converter Register 1 Setting */
	/* (1) Select AIN Chennel,  ADCCR1[3:0]=0111(AIN7 Select) */
	/* (2) ADC Enable/Disable,  ADCCR1[4]=1(Enable)  */
	/* (3) Select Mode,         ADCCR1[6:5]=01(Single Mode)  */
	ADCCR1 = ADC_CHANNEL_AIN7 | ADC_AINEN_ENABLE | ADC_AMD_SINGLE;
	
	/* AD Converter Register 2 Setting */
	/* Select Convert Time, ADCCR2[2:0]=010(156/fcgck) */
	ADCCR2 = ADC_SINGLE_TIME;
}

/*--------------------------------------------------*/
/** 
  * @brief  ADC_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void ADC_Start(void)
{
	/* ADRS = 1 */
	ADRS = ADC_ADRS_START;
}

/*--------------------------------------------------*/
/** 
  * @brief  ADC_Get_Data
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void ADC_Get_Data(void)
{
	UINT16_t result;
	
	result = ADCDRL;
	result |= (UINT16_t)(ADCDRH << 8);			/* output the top 8 bits in 10 bits */
	adc_single_data = result;
	if(adc_single_data > 0x03FF) {
		adc_single_data = 0x03FF;
	}
	else {
		/* Do nothing */
	}
}

/**
 *  @}
 */ /* End of group ADC_SINGLE_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_SINGLE_Exported_functions ADC Exported Functions
 *  @{
 */
 
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	UINT8_t i;
	UINT8_t j;
	UINT8_t t_state = (UINT8_t)0;
	UINT8_t t_len = (UINT8_t)0;
	UINT8_t a = (UINT8_t)0;
	
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	/*------------------------*/
	/* UART Setting            */
	/*------------------------*/
	uart_initialize();
	
	for(j=0;j<TX_BUFF_ALL;j++) {
		t_len = sizeof(disp_tbl[j]);
		uart_data_set(t_len);
		for(i=0;i<=t_len;i++) {
			tx_data[i] = disp_tbl[j][i];
		}
		t_state = UART2SR;
		while((t_state & 0x02) != 0) {
			t_state = UART2SR;
		}
		UART2CR1_TXE = UART_TXE_ENABLE;
		TX_Start();
		TD2BUF = tx_data[write];
		
		for(;;) {
			if(write == 0xFF) {
				break;
			}
			else {
				/* Do nothing */
			}
		}
	}
	/*------------------------*/
	/* ADC Setting            */
	/*------------------------*/
	ADC_Config();
	
	/* Start ADC Convert */
	ADC_Start();
	/*------------------------*/
	/* Wait Convert           */
	/*------------------------*/
	while (EOCF == CLEAR) {
	}
	while (ADBF != CLEAR) {
	}
	
	/*------------------------*/
	/* Get Data               */
	/*------------------------*/
	ADC_Get_Data();
	
	/*------------------------*/
	/* UART Send              */
	/*------------------------*/
	uart_initialize();
	write = 0;
	tx_length = (UINT8_t)5;
	
	tx_data[0] = (CHAR_t)'0';
	tx_data[1] = (CHAR_t)'x';
	
	/* change AD */
	a = (UINT8_t)(adc_single_data / 256);
	if(a >= 10) {
		a -= 10;
		a += 65;
	}
	else {
		a += 48;
	}
	tx_data[2] = a;
	a = (UINT8_t)((adc_single_data % 256) / 16);
	if(a >= 10) {
		a -= 10;
		a += 65;
	}
	else {
		a += 48;
	}
	tx_data[3] = a;
	a = (UINT8_t)((adc_single_data % 256) % 16);
	if(a >= 10) {
		a -= 10;
		a += 65;
	}
	else {
		a += 48;
	}
	tx_data[4] = a;
	tx_data[5] = '\n';
	TX_Start();
	TD2BUF = tx_data[write];
}
/**
 *  @}
 */ /* End of group ADC_SINGLE_Exported_functions */

/**
 *  @}
 */ /* End of group ADC_SINGLE */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
