/** 
 *******************************************************************************
 * @file    main.c
 * @brief   DVO Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-07-19 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup DVO DVO Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup DVO_Private_macro DVO Private Macro
 *  @{
 */

/** 
 *  @defgroup DVO_Mode Macro Definition
 *  @brief    Mode Setting.
 *  @{
 */
#define DVO_MODE_SEL					DVO_MODE_NORMAL_1
#define DVO_MODE_NORMAL_1				((UINT8_t)0x00)			/*!< (NORMAL1[h) */
#define DVO_MODE_IDLE_1					((UINT8_t)0x01)			/*!< (IDLE1[h) */
#define DVO_MODE_NORMAL_2				((UINT8_t)0x02)			/*!< (NORMAL2[h) */
#define DVO_MODE_IDLE_2					((UINT8_t)0x03)			/*!< (IDLE2[h) */
#define DVO_MODE_SLOW_2					((UINT8_t)0x04)			/*!< (SLOW2[h) */
#define DVO_MODE_SLOW_1					((UINT8_t)0x05)			/*!< (SLOW1[h) */
#define DVO_MODE_SLEEP_1				((UINT8_t)0x06)			/*!< (SLEEP1[h) */
/**
 *  @}
 */ /* End of group DVO_Mode Macro Definition */
/** 
 *  @defgroup DVO_SYS_CLOCK Macro Definition
 *  @brief    System clock Setting.
 *  @{
 */
#define DVO_DV9CK_FCGCK					((UINT8_t)0x00)			/*!< (MANbN) */
#define DVO_DV9CK_FS					((UINT8_t)0x10)			/*!< (gNbN) */
/**
 *  @}
 */ /* End of group DVO_SYS_CLOCK Macro Definition */
/** 
 *  @defgroup DVO_DVOCR Macro Definition
 *  @brief    DVOCR Setting.
 *  @{
 */
#define DVO_CK_00						((UINT8_t)0x00)			/*!< (fcgck/2^12Afs/2^5) */
#define DVO_CK_01						((UINT8_t)0x01)			/*!< (fcgck/2^11Afs/2^4) */
#define DVO_CK_10						((UINT8_t)0x02)			/*!< (fcgck/2^10Afs/2^3) */
#define DVO_CK_11						((UINT8_t)0x03)			/*!< (fcgck/2^9AfsReserved) */
/**
 *  @}
 */ /* End of group DVO_DVOCR Macro Definition */
/** 
 *  @defgroup DVO_DVOEN DVO Start
 *  @brief    DVO Start.
 *  @{
 */
#define DVO_START						((UINT8_t)0x01)			/*!< DVO Start */
/**
 *  @}
 */ /* End of group DVO_DVOEN */
/** 
 *  @defgroup  Port Port Setting
 *  @brief     Port Setting
 *  @{
 */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
#define PORT_DVO						((UINT8_t)0x01)			/*!< Port Function DVO */
/** 
 *  @}
 */ /* End of group Port */

/** 
 *  @}
 */ /* End of group DVO_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup DVO_Private_fuctions DVO Private Fuctions
 *  @{
 */
static void System_Init(void);
static void Port_Config(void);
static void DVO_Config(void);
static void DVO_Start(void);
/*--------------------------------------------------*/
/** 
  * @brief  System_Init
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void System_Init(void)
{
	static UINT8_t mode = DVO_MODE_SEL;
	
	/* (1) Mode setting */
	if(mode < DVO_MODE_NORMAL_2) {
		if(mode == DVO_MODE_IDLE_1) {
			/* IDLE1 mode */
			__DI();
			IDLE = SET;
		}
		else {
		/* NORMAL1 mode */	/* SINGLE CLOCK mode == no action */
		}
	}
	else {
		/* NORMAL2 mode */

		ILL = 0xEF;
		WUCCR = 0x80;
		WUCDR = 0x2F;

		XTEN = SET;								/* Enable the Low-frequency Clock */
		
		while((ILL & 0x10) == 0){
		}

		ILL = 0xEF;
		
		if(mode == DVO_MODE_IDLE_2) {
			/* IDLE2 mode */
		__DI();
		IDLE = SET;

		}
		else if(mode >= DVO_MODE_SLOW_2){
			/* SLOW2 mode */
			SYSCK = SET;
			__asm("NOP");
			__asm("NOP");

			if(mode != DVO_MODE_SLOW_2) {
				if(mode >= DVO_MODE_SLOW_1) {
					/* SLOW1 */
					XEN = CLEAR;					/* Disable the High-frequency Clock */
					
					if(mode == DVO_MODE_SLEEP_1) {
						__DI();
						IDLE = SET;
					}
					else {
						/* Do nothing */
					}
				}
				else {
					/* Do nothing */
				}
			}
			else {
				/* Do nothing */
			}
		}
		else {
			/* Do nothing */
		}
	}
	/* (2) Select system clock */
	SYSCR1 |= DVO_DV9CK_FCGCK;
}
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Port Function Control : DVO */
	P7FC4 = PORT_DVO;
	/* (2) Select output mode on PORT 74 (DVO) */
	P7CR4 = PORT_OUTPUT;
}

/*--------------------------------------------------*/
/** 
  * @brief  DVO_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void DVO_Config(void)
{
	/* Setting clock DVOCR[1:0]=01(fcgck/2^11) */
	DVOCR = DVO_CK_01;
}
/*--------------------------------------------------*/
/** 
  * @brief  DVO_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void DVO_Start(void)
{
	/* DVOEN = 1 */
	DVOEN = DVO_START;
}
    
/**
 *  @}
 */ /* End of group DVO_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup DVO_Exported_functions DVO Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	
	/*------------------------*/
	/* DVO Setting            */
	/*------------------------*/ 
	DVO_Config();
	DVO_Start();
	
	/*------------------------*/
	/* Sys Setting            */
	/*------------------------*/
	System_Init();
	
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
	}
}

/**
 *  @}
 */ /* End of group DVO_Exported_functions */

/**
 *  @}
 */ /* End of group DVO */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
