/** 
 *******************************************************************************
 * @file    main.c
 * @brief   EXT_INT Sample Application.
 * @version V1.0.1.0
 * $Date:: 2019-06-10 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup EXT_INT EXT_INT Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EXT_INT_Private_macro EXT_INT Private Macro
 *  @{
 */

/** 
 *  @defgroup INT INT0 Enable
 *  @brief    INT0 Control Enable.
 *  @{
 */
#define INT_INT0_ENABLE					((UINT8_t)0x01)			/*!< INT0 enable */
/**
 *  @}
 */ /* End of group INT0 */
/** 
 *  @defgroup  Port Port Setting
 *  @brief     Port Setting
 *  @{
 */
#define PORT_OFF						((UINT8_t)0x00)			/*!< Port OFF */
#define PORT_ON							((UINT8_t)0x01)			/*!< Port ON */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
#define PORT_INPUT						((UINT8_t)0x00)			/*!< Port Mode Input */
/** 
 *  @}
 */ /* End of group EXT_INT */

/** 
 *  @}
 */ /* End of group EXT_INT_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EXT_INT_Private_fuctions EXT_INT Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void EXT_INT_Config(void);
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Select output mode on PORT 25 (LED0) */
	P2CR5 = PORT_OUTPUT;
	/* (2) Turn off PORT 25 (LED0) */
	P25 = PORT_OFF;
	/* (3) Select input mode on PORT 12 (SW0) */
	P1CR2 = PORT_INPUT;
}

/*--------------------------------------------------*/
/** 
  * @brief  EXT_INT_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void EXT_INT_Config(void)
{
	/* Enable INT0 control */
	POFFCR3 = INT_INT0_ENABLE;
	
	/* Enable interrupt of INT0 */
	__DI();
	EIR_INT0 = 1;
	__EI();
}

/**
 *  @}
 */ /* End of group EXT_INT_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_EXT_TRIG_Exported_functions TCA_EXT_TRIG Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	
	/*------------------------*/
	/* EXT_INT Setting        */
	/*------------------------*/ 
	EXT_INT_Config();
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  INT0
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt INT0(void)
{
	P25 = PORT_ON;
}

/**
 *  @}
 */ /* End of group EXT_INT_Exported_functions */

/**
 *  @}
 */ /* End of group EXT_INT */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
