/** 
 *******************************************************************************
 * @file    main.c
 * @brief   FLASH_MCU_RAM Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-08-21 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"TMP89Fx.h"  /*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"
#include	"uart_io.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup FLASH_MCU_RAM FLASH_MCU_RAM Sample Appli
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_MCU_RAM_Private_define FLASH_MCU_RAM Private Define
 *  @{
 */
#define TX_BUFF_ALL			(UINT8_t)3
const UINT8_t disp_tbl[TX_BUFF_ALL][56] = {
	{"Flash MCU Product ID mode Sample Program\n"},
	{"------------------------------------------------------\n"},
	{"Transfer a control program to the RAM.\n"}
};

#define TX_BUFF_ALL2		(UINT8_t)4
const UINT8_t disp_tbl2[TX_BUFF_ALL2][19] = {
	{"Vendor ID:0x"},
	{"Flash ID:0x"},
	{"Security status:0x"},
	{"Finished\n"}
};

#define	RAMCODE_MAPPING_ON				(0x06)						/*!< mapping ram to code area */
#define	RAMCODE_MAPPING_OFF				(0x00)						/*!< release mapping ram to code area */
#define	RAMCODE_ENABLE					(0xD4)						/*!< enable ramcode setting */

/** 
 *  @}
 */ /* End of group FLASH_MCU_RAM_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_MCU_RAM_Private_typedef FLASH_MCU_RAM Private Typedef
 *  @{
 */
 
static void __interrupt RamOnlyReti(void);
static void __interrupt_n RamOnlyRetn(void);
static void RamSTARTUP(void);
extern	void STARTUP( void );
/** 
 *  @}
 */ /* End of group FLASH_MCU_RAM_Private_define */
/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_MCU_RAM_Private_variables FLASH_MCU_RAM Private Variables
 *  @{
 */
#pragma	section	const	r_vct
#if defined(TMP89FS60B)
void * const _rivect[] = {
	RamOnlyReti,			/* 0xFFC8 INTTXD2			*/
	RamOnlyReti,			/* 0xFFCA INTRXD2			*/
	RamOnlyReti,			/* 0xFFCC INTTC03			*/
	RamOnlyReti,			/* 0xFFCE INTTC02			*/
	RamOnlyReti,			/* 0xFFD0 INTTXD1			*/
	RamOnlyReti,			/* 0xFFD2 INTRXD1/INTSIO1	*/
	RamOnlyReti,			/* 0xFFD4 INTTCA1			*/
	RamOnlyReti,			/* 0xFFD6 INT4				*/
	RamOnlyReti,			/* 0xFFD8 INT3				*/
	RamOnlyReti,			/* 0xFFDA INT2				*/
	RamOnlyReti,			/* 0xFFDC INT1				*/
	RamOnlyReti,			/* 0xFFDE INT0				*/
	RamOnlyReti,			/* 0xFFE0 INTSBI0/INTSIO0	*/
	RamOnlyReti,			/* 0xFFE2 INTTCA0			*/
	RamOnlyReti,			/* 0xFFE4 INTTC01			*/
	RamOnlyReti,			/* 0xFFE6 INTTC00			*/
	RamOnlyReti,			/* 0xFFE8 INTRTC			*/
	RamOnlyReti,			/* 0xFFEA INTADC			*/
	RamOnlyReti,			/* 0xFFEC INTVLTD			*/
	RamOnlyReti,			/* 0xFFEE INT5				*/
	RamOnlyReti,			/* 0xFFF0 INTTXD0			*/
	RamOnlyReti,			/* 0xFFF2 INTRXD0/INTSIO0	*/
	RamOnlyReti,			/* 0xFFF4 INTTBT			*/
	RamOnlyReti,			/* 0xFFF6 INTWUC			*/
	RamOnlyRetn,			/* 0xFFF8 INTWDT			*/
	RamOnlyRetn,			/* 0xFFFA Reserved			*/
	RamOnlyRetn,			/* 0xFFFC INTUNDEF/INTSWI	*/
	RamSTARTUP				/* 0xFFFE RESET				*/
};
#elif defined(TMP89FS62B)
void * const _rivect[] = {
	RamOnlyReti,			/* 0xFFC8 INTTXD2			*/
	RamOnlyReti,			/* 0xFFCA INTRXD2			*/
	RamOnlyReti,			/* 0xFFCC INTTC03			*/
	RamOnlyReti,			/* 0xFFCE INTTC02			*/
	RamOnlyReti,			/* 0xFFD0 INTTXD1			*/
	RamOnlyReti,			/* 0xFFD2 INTRXD1/INTSIO1	*/
	RamOnlyReti,			/* 0xFFD4 INTTCA1			*/
	RamOnlyReti,			/* 0xFFD6 INT4				*/
	RamOnlyReti,			/* 0xFFD8 INT3				*/
	RamOnlyReti,			/* 0xFFDA INT2				*/
	RamOnlyReti,			/* 0xFFDC INT1				*/
	RamOnlyReti,			/* 0xFFDE INT0				*/
	RamOnlyReti,			/* 0xFFE0 INTSBI0/INTSIO0	*/
	RamOnlyReti,			/* 0xFFE2 INTTCA0			*/
	RamOnlyReti,			/* 0xFFE4 INTTC01			*/
	RamOnlyReti,			/* 0xFFE6 INTTC00			*/
	RamOnlyReti,			/* 0xFFE8 INTRTC			*/
	RamOnlyReti,			/* 0xFFEA INTADC			*/
	RamOnlyReti,			/* 0xFFEC INTVLTD			*/
	RamOnlyReti,			/* 0xFFEE INT5				*/
	RamOnlyReti,			/* 0xFFF0 INTTXD0			*/
	RamOnlyReti,			/* 0xFFF2 INTRXD0/INTSIO0	*/
	RamOnlyReti,			/* 0xFFF4 INTTBT			*/
	RamOnlyReti,			/* 0xFFF6 INTWUC			*/
	RamOnlyRetn,			/* 0xFFF8 INTWDT			*/
	RamOnlyRetn,			/* 0xFFFA Reserved			*/
	RamOnlyRetn,			/* 0xFFFC INTUNDEF/INTSWI	*/
	RamSTARTUP				/* 0xFFFE RESET				*/
};
#elif defined(TMP89FS63B)
void * const _rivect[] = {
	RamOnlyReti,			/* 0xFFC8 INTTXD2			*/
	RamOnlyReti,			/* 0xFFCA INTRXD2			*/
	RamOnlyReti,			/* 0xFFCC INTTC03			*/
	RamOnlyReti,			/* 0xFFCE INTTC02			*/
	RamOnlyReti,			/* 0xFFD0 INTTXD1			*/
	RamOnlyReti,			/* 0xFFD2 INTRXD1/INTSIO1	*/
	RamOnlyReti,			/* 0xFFD4 INTTCA1			*/
	RamOnlyReti,			/* 0xFFD6 INT4				*/
	RamOnlyReti,			/* 0xFFD8 INT3				*/
	RamOnlyReti,			/* 0xFFDA INT2				*/
	RamOnlyReti,			/* 0xFFDC INT1				*/
	RamOnlyReti,			/* 0xFFDE INT0				*/
	RamOnlyReti,			/* 0xFFE0 INTSBI0/INTSIO0	*/
	RamOnlyReti,			/* 0xFFE2 INTTCA0			*/
	RamOnlyReti,			/* 0xFFE4 INTTC01			*/
	RamOnlyReti,			/* 0xFFE6 INTTC00			*/
	RamOnlyReti,			/* 0xFFE8 INTRTC			*/
	RamOnlyReti,			/* 0xFFEA INTADC			*/
	RamOnlyReti,			/* 0xFFEC INTVLTD			*/
	RamOnlyReti,			/* 0xFFEE INT5				*/
	RamOnlyReti,			/* 0xFFF0 INTTXD0			*/
	RamOnlyReti,			/* 0xFFF2 INTRXD0/INTSIO0	*/
	RamOnlyReti,			/* 0xFFF4 INTTBT			*/
	RamOnlyReti,			/* 0xFFF6 INTWUC			*/
	RamOnlyRetn,			/* 0xFFF8 INTWDT			*/
	RamOnlyRetn,			/* 0xFFFA Reserved			*/
	RamOnlyRetn,			/* 0xFFFC INTUNDEF/INTSWI	*/
	RamSTARTUP				/* 0xFFFE RESET				*/
};
#else
	#error "target device is non-select."
#endif

static UINT8_t read_data[3];						/*!< read data            */
/**
 *  @}
 */ /* End of group FLASH_MCU_RAM_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_MCU_RAM_Private_fuctions FLASH_MCU_RAM Private Fuctions
 *  @{
 */
extern	UINT8_t		_CPdataAddr[];
extern	UINT8_t		_CPdataOrg[];
extern	UINT16_t	_CPdataSize;
extern	UINT8_t		_CPvectAddr[];
extern	UINT8_t		_CPvectOrg[];
extern	UINT16_t	_CPvectSize;


static void Product_ID_Read(void);


#pragma section code r_copy
/*--------------------------------------------------*/
/** 
  * @brief  Product ID Read
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Product_ID_Read(void)
{
	/* sector erase */
#pragma asm
	PUSH	WA
	PUSH	HL

	LD		(0x0FD0),0xA8
	LD		(0x0FD1),0xD5

	LD		(0xF555),0xAA
	LD		(0xFAAA),0x55
	LD		(0xF555),0x90

	NOP
	NOP
	NOP

	LD		HL,_read_data

	LD		A,(0xF000)
	LD		(HL),A

	INC		HL

	LD		A,(0xF001)
	LD		(HL),A

	INC		HL

	LD		A,(0xFF7F)
	LD		(HL),A

	LD		A,(0xFF7F)

	LD		(0xF000),0xF0

	NOP
	NOP
	NOP

	LD		(0x0FD0),0x40
	LD		(0x0FD1),0xD5

	POP		HL
	POP		WA
#pragma endasm
}

/*--------------------------------------------------*/
/** 
  * @brief  Ram Only Reti
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void __interrupt RamOnlyReti(void)
{
	__asm("	nop	");
}

/*--------------------------------------------------*/
/** 
  * @brief  Ram Only Retn
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void __interrupt_n RamOnlyRetn(void)
{
	__asm("	nop	");
}

/*--------------------------------------------------*/
/** 
  * @brief  Ram STARTUP
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void RamSTARTUP(void)
{
	STARTUP();
}
/**
 *  @}
 */ /* End of group FLASH_MCU_RAM_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_MCU_RAM_Exported_functions FLASH_MCU_RAM Exported Functions
 *  @{
 */
#pragma	section	code
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	UINT8_t 	i;
	UINT8_t 	temp;
	UINT8_t 	send_data[3];
	UINT8_t		*addr;
	UINT8_t		*org;
	UINT16_t	size;
	
	/*------------------------*/
	/* UART Setting            */
	/*------------------------*/
	uart_initialize();
	
	for(i=0;i<TX_BUFF_ALL;i++) {
		Send_Display_Data(sizeof(disp_tbl[i]), (UINT8_t *)disp_tbl[i]);
	}

	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	/* copy ram program  */
	size = (UINT16_t)&_CPdataSize;
	addr = (UINT8_t *)&_CPdataAddr;
	org = (UINT8_t *)&_CPdataOrg;
	for(i = 0; i < size; i++){
		addr[i] = org[i];
	}
	
	/* copy ram vector */
	size = (UINT16_t)&_CPvectSize;
	addr = (UINT8_t *)&_CPvectAddr;
	org = (UINT8_t *)&_CPvectOrg;
	for(i = 0; i < size; i++){
		addr[i] = org[i];
	}

	/* mapping ram to code area */
	SYSCR3 = RAMCODE_MAPPING_ON;
	SYSCR4 = RAMCODE_ENABLE;

	__DI();
	/* read product id */
	Product_ID_Read();
	__EI();

	/* release mapping ram to code area */
	SYSCR3 = RAMCODE_MAPPING_OFF;
	SYSCR4 = RAMCODE_ENABLE;

	for(i=0;i<TX_BUFF_ALL2;i++) {
		Send_Display_Data(sizeof(disp_tbl2[i]), (UINT8_t *)disp_tbl2[i]);

		if(i < (TX_BUFF_ALL2 - 1)) {
			temp = (UINT8_t)((read_data[i] >> 4) & 0x0F);
			if(temp >= 10) {
				temp -= 10;
				temp += 'A';
			}
			else {
				temp += '0';
			}
			send_data[0] = temp;
			temp = (UINT8_t)(read_data[i] & 0x0F);
			if(temp >= 10) {
				temp -= 10;
				temp += 'A';
			}
			else {
				temp += '0';
			}
			send_data[1] = temp;
			send_data[2] = (CHAR_t)'\n';
			Send_Display_Data(3, send_data);
		}
		else {
			/* Do nothing */
		}
	}

	for(;;){
	}
}
/**
 *  @}
 */ /* End of group FLASH_MCU_RAM_Exported_functions */

/**
 *  @}
 */ /* End of group FLASH_MCU_RAM */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
