/** 
 *******************************************************************************
 * @file    StartUp.C
 * @brief   TMP89Fx Sample Application.
 *            fc : 10MHz
 *            fs : 32.768kHz
 * @version V1.0.2.0
 * $Date:: 2019-08-27 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
/* ************************************************************************** */

/* ************************************************************************** */
/*
 * --------------------------------------------------------------------------
 *   HEADER INCLUDE DEFINITIONS
 * --------------------------------------------------------------------------
 */
#define IO_MEM

#include	<stdlib.h>
#include	"TMP89Fx.h"  /*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"
/*
 * --------------------------------------------------------------------------
 *   MACRO DEFINITIONS
 * --------------------------------------------------------------------------
 */
#define	_BaseSP				0x0C3F
#define	_BYTE_OF_CLEAR		0x0BFF

#define	_RAM_TOP_ADDRESS	0x0040

extern INT8_t		_NDataAddr[];	/* for n_data initialize, defined in link command file  */
extern INT8_t		_NDataOrg[];	/* for n_data initialize, defined in link command file  */
extern INT16_t		_NDataSize;		/* for n_data initialize, defined in link command file  */

/*
 * --------------------------------------------------------------------------
 *   FUNCTION DECLARATIONS
 * --------------------------------------------------------------------------
 */
extern void main(void);
extern void __interrupt IntTXD2(void);
extern void __interrupt IntRXD2(void);

/* ************************************************************************** */

/*
 ***************************************************************************
 * NAME				:STARTUP
 *--------------------------------------------------------------------------
 * DESCRIPTION		:Startup routine
 * PARAMETER		:-
 *
 * RETURN VALUE		:-
 *
 ***************************************************************************
 */
void STARTUP( void )
{
	__DI();
	__SP = _BaseSP;

	__HL = _RAM_TOP_ADDRESS;
	__BC = _BYTE_OF_CLEAR-1;
	__asm(" LD A,0x00");
	__asm("CLR_RAM:");
	__asm(" LD (HL),A");
	__asm(" INC HL");
	__asm(" DEC BC");
	__asm(" JRS F,CLR_RAM");		/* Clear the RAM */

	/* Initialize of n_data section */
	__asm("	LD	IX,__NDataOrg");
	__asm("	LD	IY,__NDataAddr");
	__asm("	LD	WA,__NDataSize");
	__asm("	DEC	WA");
	__asm("	J	T,non_n_data");
	__asm("move_n_data:");
	__asm("	LD	C,(IX)");
	__asm("	LD	(IY),C");
	__asm("	INC	IX");
	__asm("	INC	IY");
	__asm("	DEC	WA");
	__asm("	J	F,move_n_data");
	__asm("non_n_data:");

	
	WUCCR = 0x80;					/* WUCRST = SET fc = 10MHz */
	WUCDR = 0x2F;					/* 300us/6.4us = 46.875  47 */
	
	
	P0FC2 = SET;
	
	XTEN = SET;						/* Enable the Low-frequency Clock */
	
	while((ILL & 0x10) == 0){
	}
	ILL = 0xEF;
	__asm("nop");					/* Wait 2 machine cycles */
	__asm("nop");
	
	CGCR = 0x02;					/* fcgck = fc = 10MHZ */

 	WDCTR = 0x00;
	WDCDR = 0xB1;					/* WDT disable */

#if defined(TMP89FS62B)
	/* Port initialize */
	P1CR = 0x0C;
	P2CR = 0xF0;
	P5CR = 0xFF;
	P8CR = 0x1C;
	PBCR = 0xC0;
#elif defined(TMP89FS63B)
	/* Port initialize */
	P1CR = 0x0C;
	P2CR = 0xC0;
	P5CR = 0xE0;
	P8CR = 0x1C;
	PBCR = 0xC0;
#endif

	/* Latch clear */
	ILL = 0;
	ILH = 0;
	ILE = 0;
	ILD = 0;

	__EI();

	main();
	for (;;) {
		;
	}
}

void __interrupt OnlyReti(void)
{
	__asm("NOP");
}

void __interrupt_n OnlyRetn(void)
{
	__asm("NOP");
}

#pragma section const _BaseIntTbl
/*
 * --------------------------------------------------------------------------
 *   VECTOR DECLARATIONS
 * --------------------------------------------------------------------------
 */
#if defined(TMP89FS60B)
void * const _ivect[] = {
	IntTXD2,				/* 0xFFC8 INTTXD2			*/
	IntRXD2,				/* 0xFFCA INTRXD2			*/
	OnlyReti,				/* 0xFFCC INTTC03			*/
	OnlyReti,				/* 0xFFCE INTTC02			*/
	OnlyReti,				/* 0xFFD0 INTTXD1			*/
	OnlyReti,				/* 0xFFD2 INTRXD1/INTSIO1	*/
	OnlyReti,				/* 0xFFD4 INTTCA1			*/
	OnlyReti,				/* 0xFFD6 INT4				*/
	OnlyReti,				/* 0xFFD8 INT3				*/
	OnlyReti,				/* 0xFFDA INT2				*/
	OnlyReti,				/* 0xFFDC INT1				*/
	OnlyReti,				/* 0xFFDE INT0				*/
	OnlyReti,				/* 0xFFE0 INTSBI0/INTSIO0	*/
	OnlyReti,				/* 0xFFE2 INTTCA0			*/
	OnlyReti,				/* 0xFFE4 INTTC01			*/
	OnlyReti,				/* 0xFFE6 INTTC00			*/
	OnlyReti,				/* 0xFFE8 INTRTC			*/
	OnlyReti,				/* 0xFFEA INTADC			*/
	OnlyReti,				/* 0xFFEC INTVLTD			*/
	OnlyReti,				/* 0xFFEE INT5				*/
	OnlyReti,				/* 0xFFF0 INTTXD0			*/
	OnlyReti,				/* 0xFFF2 INTRXD0/INTSIO0	*/
	OnlyReti,				/* 0xFFF4 INTTBT			*/
	OnlyReti,				/* 0xFFF6 INTWUC			*/
	OnlyRetn,				/* 0xFFF8 INTWDT			*/
	(void *)0xFFFF,			/* 0xFFFA Option Code		*/
	OnlyRetn,				/* 0xFFFC INTUNDEF/INTSWI	*/
	STARTUP					/* 0xFFFE RESET				*/
};
#elif defined(TMP89FS62B)
void * const _ivect[] = {
	IntTXD2,				/* 0xFFC8 INTTXD2			*/
	IntRXD2,				/* 0xFFCA INTRXD2			*/
	OnlyReti,				/* 0xFFCC INTTC03			*/
	OnlyReti,				/* 0xFFCE INTTC02			*/
	OnlyReti,				/* 0xFFD0 INTTXD1			*/
	OnlyReti,				/* 0xFFD2 INTRXD1/INTSIO1	*/
	OnlyReti,				/* 0xFFD4 INTTCA1			*/
	OnlyReti,				/* 0xFFD6 INT4				*/
	OnlyReti,				/* 0xFFD8 INT3				*/
	OnlyReti,				/* 0xFFDA INT2				*/
	OnlyReti,				/* 0xFFDC INT1				*/
	OnlyReti,				/* 0xFFDE INT0				*/
	OnlyReti,				/* 0xFFE0 INTSBI0/INTSIO0	*/
	OnlyReti,				/* 0xFFE2 INTTCA0			*/
	OnlyReti,				/* 0xFFE4 INTTC01			*/
	OnlyReti,				/* 0xFFE6 INTTC00			*/
	OnlyReti,				/* 0xFFE8 INTRTC			*/
	OnlyReti,				/* 0xFFEA INTADC			*/
	OnlyReti,				/* 0xFFEC INTVLTD			*/
	OnlyReti,				/* 0xFFEE INT5				*/
	OnlyReti,				/* 0xFFF0 INTTXD0			*/
	OnlyReti,				/* 0xFFF2 INTRXD0/INTSIO0	*/
	OnlyReti,				/* 0xFFF4 INTTBT			*/
	OnlyReti,				/* 0xFFF6 INTWUC			*/
	OnlyRetn,				/* 0xFFF8 INTWDT			*/
	(void *)0xFFFF,			/* 0xFFFA Option Code		*/
	OnlyRetn,				/* 0xFFFC INTUNDEF/INTSWI	*/
	STARTUP					/* 0xFFFE RESET				*/
};
#elif defined(TMP89FS63B)
void * const _ivect[] = {
	IntTXD2,				/* 0xFFC8 INTTXD2			*/
	IntRXD2,				/* 0xFFCA INTRXD2			*/
	OnlyReti,				/* 0xFFCC INTTC03			*/
	OnlyReti,				/* 0xFFCE INTTC02			*/
	OnlyReti,				/* 0xFFD0 INTTXD1			*/
	OnlyReti,				/* 0xFFD2 INTRXD1/INTSIO1	*/
	OnlyReti,				/* 0xFFD4 INTTCA1			*/
	OnlyReti,				/* 0xFFD6 INT4				*/
	OnlyReti,				/* 0xFFD8 INT3				*/
	OnlyReti,				/* 0xFFDA INT2				*/
	OnlyReti,				/* 0xFFDC INT1				*/
	OnlyReti,				/* 0xFFDE INT0				*/
	OnlyReti,				/* 0xFFE0 INTSBI0/INTSIO0	*/
	OnlyReti,				/* 0xFFE2 INTTCA0			*/
	OnlyReti,				/* 0xFFE4 INTTC01			*/
	OnlyReti,				/* 0xFFE6 INTTC00			*/
	OnlyReti,				/* 0xFFE8 INTRTC			*/
	OnlyReti,				/* 0xFFEA INTADC			*/
	OnlyReti,				/* 0xFFEC INTVLTD			*/
	OnlyReti,				/* 0xFFEE INT5				*/
	OnlyReti,				/* 0xFFF0 INTTXD0			*/
	OnlyReti,				/* 0xFFF2 INTRXD0/INTSIO0	*/
	OnlyReti,				/* 0xFFF4 INTTBT			*/
	OnlyReti,				/* 0xFFF6 INTWUC			*/
	OnlyRetn,				/* 0xFFF8 INTWDT			*/
	(void *)0xFFFF,			/* 0xFFFA Option Code		*/
	OnlyRetn,				/* 0xFFFC INTUNDEF/INTSWI	*/
	STARTUP					/* 0xFFFE RESET				*/
};
#else
	#error "target device is non-select."
#endif

/* ************************************************************************** */
