/** 
 *******************************************************************************
 * @file    main.c
 * @brief   FLASH_MCU_RAM Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-08-21 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"TMP89Fx.h"  /*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"
#include	"uart_io.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup FLASH_MCU_RAM FLASH_MCU_RAM Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_MCU_RAM_Private_macro FLASH_MCU_RAM Private Macro
 *  @{
 */
/** 
 *  @defgroup  Key 
 *  @brief     Key Setting
 *  @{
 */
#define PORT_INPUT						((UINT8_t)0x00)			/*!< Port Mode Input */
/** 
 *  @}
 */ /* End of group Key */

/** 
 *  @}
 */ /* End of group FLASH_MCU_RAM_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_MCU_RAM_Private_define FLASH_MCU_RAM Private Define
 *  @{
 */
#define KEYIN_CNT						((UINT8_t)2)			/*!< keychat=2 */

const UINT8_t dsp_tbl_1[] = "FLASH MCU mode(executing a control program in RAM) Sample\n";
const UINT8_t dsp_tbl_2[] = "-----------------------------------------------------------------\n";
const UINT8_t dsp_tbl_3[] = "Transfer a control program to the RAM.\n";
const UINT8_t dsp_tbl_4[] = "Sector Erase:0xE000-0xEFFF\n";
const UINT8_t dsp_tbl_5[] = "Byte Program:0xE500\n";
const UINT8_t dsp_tbl_6[] = "Read Data:(Data)0x3F,(RAM Addr)0xE500\n";
const UINT8_t dsp_tbl_7[] = "Finished\n";

#define	RAMCODE_MAPPING_ON				(0x06)						/*!< mapping ram to code area */
#define	RAMCODE_MAPPING_OFF				(0x00)						/*!< release mapping ram to code area */
#define	RAMCODE_ENABLE					(0xD4)						/*!< enable ramcode setting */

#define	READ_DATA						(0x3F)						/*!< read data */

#pragma asm
read_data		DSB						1							;/*!< read data buffer */

.BTWrite		EQU						0x1010						;/*!< API address (byte write) */
.BTEraseSec		EQU						0x1012						;/*!< API address (flash erase) */

WRITE_ADDR		EQU						0xE500						;/*!< API parameter (write addr) */
WRITE_DATA		EQU						0x3F						;/*!< API parameter (write data) */
ERASE_SEC		EQU						0x0E						;/*!< API parameter (erase sector) */
ENABLE_CODE		EQU						0xD5						;/*!< API parameter (enable code) */
#pragma endasm
/** 
 *  @}
 */ /* End of group FLASH_MCU_RAM_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_MCU_RAM_Private_typedef FLASH_MCU_RAM Private Typedef
 *  @{
 */


static void __interrupt RamOnlyReti(void);
static void __interrupt_n RamOnlyRetn(void);
static void RamSTARTUP(void);
extern	void STARTUP( void );
/** 
 *  @}
 */ /* End of group FLASH_MCU_RAM_Private_define */
/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_MCU_RAM_Private_variables FLASH_MCU_RAM Private Variables
 *  @{
 */

#pragma	section	const	r_vct
#if defined(TMP89FS60B)
void * const _rivect[] = {
	RamOnlyReti,			/* 0xFFC8 INTTXD2			*/
	RamOnlyReti,			/* 0xFFCA INTRXD2			*/
	RamOnlyReti,			/* 0xFFCC INTTC03			*/
	RamOnlyReti,			/* 0xFFCE INTTC02			*/
	RamOnlyReti,			/* 0xFFD0 INTTXD1			*/
	RamOnlyReti,			/* 0xFFD2 INTRXD1/INTSIO1	*/
	RamOnlyReti,			/* 0xFFD4 INTTCA1			*/
	RamOnlyReti,			/* 0xFFD6 INT4				*/
	RamOnlyReti,			/* 0xFFD8 INT3				*/
	RamOnlyReti,			/* 0xFFDA INT2				*/
	RamOnlyReti,			/* 0xFFDC INT1				*/
	RamOnlyReti,			/* 0xFFDE INT0				*/
	RamOnlyReti,			/* 0xFFE0 INTSBI0/INTSIO0	*/
	RamOnlyReti,			/* 0xFFE2 INTTCA0			*/
	RamOnlyReti,			/* 0xFFE4 INTTC01			*/
	RamOnlyReti,			/* 0xFFE6 INTTC00			*/
	RamOnlyReti,			/* 0xFFE8 INTRTC			*/
	RamOnlyReti,			/* 0xFFEA INTADC			*/
	RamOnlyReti,			/* 0xFFEC INTVLTD			*/
	RamOnlyReti,			/* 0xFFEE INT5				*/
	RamOnlyReti,			/* 0xFFF0 INTTXD0			*/
	RamOnlyReti,			/* 0xFFF2 INTRXD0/INTSIO0	*/
	RamOnlyReti,			/* 0xFFF4 INTTBT			*/
	RamOnlyReti,			/* 0xFFF6 INTWUC			*/
	RamOnlyRetn,			/* 0xFFF8 INTWDT			*/
	RamOnlyRetn,			/* 0xFFFA Reserved			*/
	RamOnlyRetn,			/* 0xFFFC INTUNDEF/INTSWI	*/
	RamSTARTUP				/* 0xFFFE RESET				*/
};
#elif defined(TMP89FS62B)
void * const _rivect[] = {
	RamOnlyReti,			/* 0xFFC8 INTTXD2			*/
	RamOnlyReti,			/* 0xFFCA INTRXD2			*/
	RamOnlyReti,			/* 0xFFCC INTTC03			*/
	RamOnlyReti,			/* 0xFFCE INTTC02			*/
	RamOnlyReti,			/* 0xFFD0 INTTXD1			*/
	RamOnlyReti,			/* 0xFFD2 INTRXD1/INTSIO1	*/
	RamOnlyReti,			/* 0xFFD4 INTTCA1			*/
	RamOnlyReti,			/* 0xFFD6 INT4				*/
	RamOnlyReti,			/* 0xFFD8 INT3				*/
	RamOnlyReti,			/* 0xFFDA INT2				*/
	RamOnlyReti,			/* 0xFFDC INT1				*/
	RamOnlyReti,			/* 0xFFDE INT0				*/
	RamOnlyReti,			/* 0xFFE0 INTSBI0/INTSIO0	*/
	RamOnlyReti,			/* 0xFFE2 INTTCA0			*/
	RamOnlyReti,			/* 0xFFE4 INTTC01			*/
	RamOnlyReti,			/* 0xFFE6 INTTC00			*/
	RamOnlyReti,			/* 0xFFE8 INTRTC			*/
	RamOnlyReti,			/* 0xFFEA INTADC			*/
	RamOnlyReti,			/* 0xFFEC INTVLTD			*/
	RamOnlyReti,			/* 0xFFEE INT5				*/
	RamOnlyReti,			/* 0xFFF0 INTTXD0			*/
	RamOnlyReti,			/* 0xFFF2 INTRXD0/INTSIO0	*/
	RamOnlyReti,			/* 0xFFF4 INTTBT			*/
	RamOnlyReti,			/* 0xFFF6 INTWUC			*/
	RamOnlyRetn,			/* 0xFFF8 INTWDT			*/
	RamOnlyRetn,			/* 0xFFFA Reserved			*/
	RamOnlyRetn,			/* 0xFFFC INTUNDEF/INTSWI	*/
	RamSTARTUP				/* 0xFFFE RESET				*/
};
#elif defined(TMP89FS63B)
void * const _rivect[] = {
	RamOnlyReti,			/* 0xFFC8 INTTXD2			*/
	RamOnlyReti,			/* 0xFFCA INTRXD2			*/
	RamOnlyReti,			/* 0xFFCC INTTC03			*/
	RamOnlyReti,			/* 0xFFCE INTTC02			*/
	RamOnlyReti,			/* 0xFFD0 INTTXD1			*/
	RamOnlyReti,			/* 0xFFD2 INTRXD1/INTSIO1	*/
	RamOnlyReti,			/* 0xFFD4 INTTCA1			*/
	RamOnlyReti,			/* 0xFFD6 INT4				*/
	RamOnlyReti,			/* 0xFFD8 INT3				*/
	RamOnlyReti,			/* 0xFFDA INT2				*/
	RamOnlyReti,			/* 0xFFDC INT1				*/
	RamOnlyReti,			/* 0xFFDE INT0				*/
	RamOnlyReti,			/* 0xFFE0 INTSBI0/INTSIO0	*/
	RamOnlyReti,			/* 0xFFE2 INTTCA0			*/
	RamOnlyReti,			/* 0xFFE4 INTTC01			*/
	RamOnlyReti,			/* 0xFFE6 INTTC00			*/
	RamOnlyReti,			/* 0xFFE8 INTRTC			*/
	RamOnlyReti,			/* 0xFFEA INTADC			*/
	RamOnlyReti,			/* 0xFFEC INTVLTD			*/
	RamOnlyReti,			/* 0xFFEE INT5				*/
	RamOnlyReti,			/* 0xFFF0 INTTXD0			*/
	RamOnlyReti,			/* 0xFFF2 INTRXD0/INTSIO0	*/
	RamOnlyReti,			/* 0xFFF4 INTTBT			*/
	RamOnlyReti,			/* 0xFFF6 INTWUC			*/
	RamOnlyRetn,			/* 0xFFF8 INTWDT			*/
	RamOnlyRetn,			/* 0xFFFA Reserved			*/
	RamOnlyRetn,			/* 0xFFFC INTUNDEF/INTSWI	*/
	RamSTARTUP				/* 0xFFFE RESET				*/
};
#else
	#error "target device is non-select."
#endif

static UINT8_t keybuf = (UINT8_t)0;					/*!<  key buffer          */
static UINT8_t keycode = (UINT8_t)0;				/*!<  key confirm         */
static UINT8_t keychat = KEYIN_CNT;					/*!<  key now             */
static UINT8_t read_data   = 0;						/*!< read data            */
/**
 *  @}
 */ /* End of group FLASH_MCU_RAM_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_MCU_RAM_Private_fuctions FLASH_MCU_RAM Private Fuctions
 *  @{
 */

extern	UINT8_t		_CPdataAddr[];
extern	UINT8_t		_CPdataOrg[];
extern	UINT16_t	_CPdataSize;
extern	UINT8_t		_CPvectAddr[];
extern	UINT8_t		_CPvectOrg[];
extern	UINT16_t	_CPvectSize;

static void Port_Config(void);
static void Key(void);
static void Sec_Program(void);
static void Write_Program(void);
#pragma	section	code
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Select input mode on PORT 12 (SW0) */
	P1CR2 = PORT_INPUT;
}

/*--------------------------------------------------*/
/** 
  * @brief  Key
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Key(void)
{
	UINT8_t keyTemp;

	keyTemp = (UINT8_t)(P1PRD & 0x04);
	keyTemp ^= 0xFF;
	
	if(keybuf == keyTemp) {
		if(--keychat == 0) {
			keychat = KEYIN_CNT;
			keycode = keybuf;
		}
	}
	else {
		keychat = KEYIN_CNT;
		keybuf = keyTemp;
	}
}
/*#pragma section code ram_copy*/
#pragma section code r_copy
/*#pragma	section	code*/
/*--------------------------------------------------*/
/** 
  * @brief  Sector Erase
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Sec_Program(void)
{
	/* sector erase */
#pragma asm
	PUSH	WA
	PUSH	BC
	PUSH	DE
	PUSH	IX

	LD		(0x0FD0),0x50
	LD		(0x0FD1),0xD5

	LD		A,ERASE_SEC
	LD		C,ENABLE_CODE
	CALL	(.BTEraseSec)

	LD		(0x0FD0),0x40
	LD		(0x0FD1),0xD5

	POP		IX
	POP		DE
	POP		BC
	POP		WA
#pragma endasm
}
/*--------------------------------------------------*/
/** 
  * @brief  Byte Write
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Write_Program(void)
{
	/* byte write */
#pragma asm
	PUSH	WA
	PUSH	BC
	PUSH	DE
	PUSH	IX

	LD		(0x0FD0),0x50
	LD		(0x0FD1),0xD5

	LD		WA,WRITE_ADDR
	LD		C,0x00
	LD		E,WRITE_DATA
	LD		(SP-),ENABLE_CODE
	CALL	(.BTWrite)

	LD		(0x0FD0),0x40
	LD		(0x0FD1),0xD5

	POP		IX
	POP		DE
	POP		BC
	POP		WA
#pragma endasm
}
/*--------------------------------------------------*/
/** 
  * @brief  Ram Only Reti
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void __interrupt RamOnlyReti(void)
{
	__asm("	nop	");
}

/*--------------------------------------------------*/
/** 
  * @brief  Ram Only Retn
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void __interrupt_n RamOnlyRetn(void)
{
	__asm("	nop	");
}

/*--------------------------------------------------*/
/** 
  * @brief  Ram STARTUP
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void RamSTARTUP(void)
{
	STARTUP();
}

/**
 *  @}
 */ /* End of group FLASH_MCU_RAM_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_MCU_RAM_Exported_functions FLASH_MCU_RAM Exported Functions
 *  @{
 */
#pragma	section	code
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	UINT8_t i;
	UINT8_t		*addr;
	UINT8_t		*org;
	UINT16_t	size;
	
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	
	/*------------------------*/
	/* UART Setting            */
	/*------------------------*/
	uart_initialize();
	
	Send_Display_Data((UINT8_t)sizeof(dsp_tbl_1), (UINT8_t *)dsp_tbl_1);
	Send_Display_Data((UINT8_t)sizeof(dsp_tbl_2), (UINT8_t *)dsp_tbl_2);
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
		/*------------------------*/
		/* Key Check              */
		/*------------------------*/
		Key();
		if((keycode & 0x04) != 0) {
			/* copy ram program  */
			size = (UINT16_t)&_CPdataSize;
			addr = (UINT8_t *)&_CPdataAddr;
			org = (UINT8_t *)&_CPdataOrg;
			for(i = 0; i < size; i++){
				addr[i] = org[i];
			}
			
			/* copy ram vector */
			size = (UINT16_t)&_CPvectSize;
			addr = (UINT8_t *)&_CPvectAddr;
			org = (UINT8_t *)&_CPvectOrg;
			for(i = 0; i < size; i++){
				addr[i] = org[i];
			}
			
			Send_Display_Data((UINT8_t)sizeof(dsp_tbl_3), (UINT8_t *)dsp_tbl_3);
			
			/* mapping ram to code area */
			SYSCR3 = RAMCODE_MAPPING_ON;
			SYSCR4 = RAMCODE_ENABLE;
			
			__DI();
			/* sector erase */
			Sec_Program();
			__EI();
			
			/* release mapping ram to code area */
			SYSCR3 = RAMCODE_MAPPING_OFF;
			SYSCR4 = RAMCODE_ENABLE;
			
			Send_Display_Data((UINT8_t)sizeof(dsp_tbl_4), (UINT8_t *)dsp_tbl_4);
			
			/* mapping ram to code area */
			SYSCR3 = RAMCODE_MAPPING_ON;
			SYSCR4 = RAMCODE_ENABLE;
			
			__DI();
			/* byte write */
			Write_Program();
			__EI();
			
			/* release mapping ram to code area */
			SYSCR3 = RAMCODE_MAPPING_OFF;
			SYSCR4 = RAMCODE_ENABLE;
			
			Send_Display_Data((UINT8_t)sizeof(dsp_tbl_5), (UINT8_t *)dsp_tbl_5);
			
			read_data = *(unsigned char *)0xE500;
			
			if(read_data == READ_DATA) {
				Send_Display_Data((UINT8_t)sizeof(dsp_tbl_6), (UINT8_t *)dsp_tbl_6);
				Send_Display_Data((UINT8_t)sizeof(dsp_tbl_7), (UINT8_t *)dsp_tbl_7);
			}
			else {
				/* Do nothing */
			}
			
			break;
		}
		else {
			/* Do nothing */
		}
	}
}
/**
 *  @}
 */ /* End of group FLASH_MCU_RAM_Exported_functions */

/**
 *  @}
 */ /* End of group FLASH_MCU_RAM */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
