/** 
 *******************************************************************************
 * @file    main.c
 * @brief   FLASH_SROM_UART Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-08-21 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"TMP89Fx.h"  /*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup FLASH_MCU_RAM FLASH_MCU_RAM Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_SROM_UART_Private_macro FLASH_SROM_UART Private Macro
 *  @{
 */
/** 
 *  @defgroup  Port Setting
 *  @brief     Port Setting I/O,FC,ON/OFF
 *  @{
 */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
#define PORT_INPUT						((UINT8_t)0x00)			/*!< Port Mode Input */
#define PORT_TXD2						((UINT8_t)0x01)			/*!< Port Function TXD1 */
/** 
 *  @}
 */ /* End of group Port Setting */

/** 
 *  @defgroup  Key Input
 *  @brief     Key Input counter
 *  @{
 */
#define KEYIN_CNT						((UINT8_t)2)			/*!< keychat=2 */
/** 
 *  @}
 */ /* End of group Key Input */

/** 
 *  @defgroup UART_DataConfiguration  Data Configuration
 *  @brief    Data Configuration Macro Definisiton.
 *  @{
 */
#define UART2_CONTORL_ENABLE			((UINT8_t)0x04)			/*!< UART0 enable */
#define UART2_TRANS_STOP_BIT_LENGTH		((UINT8_t)0x00)			/*!< 1 bit */
#define UART2_PARITY					((UINT8_t)0x00)			/*!< No Parity  */
#define UART2_TXD_UART2_OUTPUT			((UINT8_t)0x00)			/*!< UART Output */
#define UART2_CLOCK						((UINT8_t)0x00)			/*!< fcgck */
#define UART2_TXD_ENABLE				((UINT8_t)0x80)			/*!< TXD enable */
#define UART2_RXD_ENABLE				((UINT8_t)0x40)			/*!< RXD enable */
#define UART2_RT_CLOCK					((UINT8_t)0x20)			/*!< 17/17clock */
#define UART2_NOIZE_CANCEL_TIME			((UINT8_t)0x00)			/*!< No Noize Removal */
#define UART2_RECIEVE_STOP_BIT_LENGTH	((UINT8_t)0x00)			/*!< 1 bit */
#define UART2_BAUD_RATE					((UINT8_t)0x04)			/*!< 115200 bps */
/**
 *  @}
 */ /* End of group UART_DataConfiguration */

/** 
 *  @defgroup  SROM Command
 *  @brief     SPROM mode command
 *  @{
 */
#define MATCHING_DATA1					((UINT8_t)0x86)				/*!< matching data1 */
#define MATCHING_DATA2					((UINT8_t)0x79)				/*!< matching data2 */
#define COMMAND_ERASE					((UINT8_t)0xF0)				/*!< erase command */
#define COMMAND_WRITE					((UINT8_t)0x30)				/*!< write command */
#define COMMAND_READ					((UINT8_t)0x40)				/*!< read command */

#define WRITE_OK						((UINT8_t)0x55)				/*!< write result ok */

#define PARA_CHIP_ERASE					((UINT8_t)0xFF)				/*!< chip erase parameter */

#define RADDRH							((UINT8_t)0x00)				/*!< read address high */
#define RADDRM							((UINT8_t)0xE1)				/*!< read address middle */
#define RADDRL							((UINT8_t)0x00)				/*!< read address low */
#define RSIZEH							((UINT8_t)0x00)				/*!< read size high */
#define RSIZEM							((UINT8_t)0x00)				/*!< read size middle */
#define RSIZEL							((UINT8_t)0x42)				/*!< read size loe */

#define PSIZE_ADDRH						((UINT8_t)0x00)				/*!< password size storage address high */
#define PSIZE_ADDRM						((UINT8_t)0xFE)				/*!< password size storage address middle */
#define PSIZE_ADDRL						((UINT8_t)0x00)				/*!< password size storage address low */
#define PDATA_ADDRH						((UINT8_t)0x00)				/*!< password data storage address high */
#define PDATA_ADDRM						((UINT8_t)0xFE)				/*!< password data storage address middle */
#define PDATA_ADDRL						((UINT8_t)0x01)				/*!< password data storage address low */
#if defined(TMP89FS60B)
#define PASSWORD1						((UINT8_t)'T')				/*!< password data1 */
#define PASSWORD2						((UINT8_t)'M')				/*!< password data2 */
#define PASSWORD3						((UINT8_t)'P')				/*!< password data3 */
#define PASSWORD4						((UINT8_t)'8')				/*!< password data4 */
#define PASSWORD5						((UINT8_t)'9')				/*!< password data5 */
#define PASSWORD6						((UINT8_t)'F')				/*!< password data6 */
#define PASSWORD7						((UINT8_t)'S')				/*!< password data7 */
#define PASSWORD8						((UINT8_t)'6')				/*!< password data8 */
#define PASSWORD9						((UINT8_t)'0')				/*!< password data9 */
#define PASSWORD10						((UINT8_t)'B')				/*!< password data10 */
#elif defined(TMP89FS62B)
#define PASSWORD1						((UINT8_t)'T')				/*!< password data1 */
#define PASSWORD2						((UINT8_t)'M')				/*!< password data2 */
#define PASSWORD3						((UINT8_t)'P')				/*!< password data3 */
#define PASSWORD4						((UINT8_t)'8')				/*!< password data4 */
#define PASSWORD5						((UINT8_t)'9')				/*!< password data5 */
#define PASSWORD6						((UINT8_t)'F')				/*!< password data6 */
#define PASSWORD7						((UINT8_t)'S')				/*!< password data7 */
#define PASSWORD8						((UINT8_t)'6')				/*!< password data8 */
#define PASSWORD9						((UINT8_t)'2')				/*!< password data9 */
#define PASSWORD10						((UINT8_t)'B')				/*!< password data10 */
#elif defined(TMP89FS63B)
#define PASSWORD1						((UINT8_t)'T')				/*!< password data1 */
#define PASSWORD2						((UINT8_t)'M')				/*!< password data2 */
#define PASSWORD3						((UINT8_t)'P')				/*!< password data3 */
#define PASSWORD4						((UINT8_t)'8')				/*!< password data4 */
#define PASSWORD5						((UINT8_t)'9')				/*!< password data5 */
#define PASSWORD6						((UINT8_t)'F')				/*!< password data6 */
#define PASSWORD7						((UINT8_t)'S')				/*!< password data7 */
#define PASSWORD8						((UINT8_t)'6')				/*!< password data8 */
#define PASSWORD9						((UINT8_t)'3')				/*!< password data9 */
#define PASSWORD10						((UINT8_t)'B')				/*!< password data10 */
#else
	#error "target device is non-select."
#endif
/** 
 *  @}
 */ /* End of group SROM Command */


/** 
 *  @}
 */ /* End of group FLASH_SROM_UART_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_SROM_UART_Private_variables FLASH_SROM_UART Private Variables
 *  @{
 */
static UINT8_t keybuf;											/*!< key buffer */
static UINT8_t keycode;											/*!< key confirm */
static UINT8_t keychat = KEYIN_CNT;								/*!< key chat */
static UINT8_t Step;											/*!< sequence step */
static UINT8_t send_cnt;										/*!< send data counter */
static UINT8_t send_size;										/*!< send data size */
static UINT8_t send_flg;										/*!< send flag */
static UINT8_t txbuf[30];										/*!< send data buffer */
static UINT8_t *send_pnt;										/*!< send data pointer */
static UINT8_t receive_cnt;										/*!< receive data counter */
static UINT8_t receive_size;									/*!< receive data size */
static UINT8_t receive_flg;										/*!< receive flag */
static UINT8_t receive_error;									/*!< receive error info */
static UINT8_t rxbuf[130];										/*!< receive data buffer */
static unsigned int CNT = 0;									/*!< key chat */


static const UINT8_t write_data_tbl[] = {
/*             :             size      addr                type      data       */
              ':',           0x40,     0xE1,0x00,          0x00,     0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
              0x1F,
 /*           sum                     */
              0x1F
};

static const UINT8_t write_enddata_tbl[] = {
/*	:		size	addr		type	data	sum		*/
	':',	0x00,	0x00,0x00,	0x01,	0xFF
};
/**
 *  @}
 */ /* End of group FLASH_SROM_UART_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_SROM_UART_Private_fuctions FLASH_SROM_UART Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void UART_Config(void);
static void Key(void);
static void UART_Data_Send(UINT8_t *buf, UINT8_t txsize, UINT8_t rxsize);

/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Setting UART2 */
	UATCNG = CLEAR;
	P93 = SET;
	P9FC3 = PORT_TXD2;
	P9CR3 = PORT_OUTPUT;
	P9CR4 = PORT_INPUT;

	/* (2) Setting SW */
	P1CR2 = PORT_INPUT;
}

/*--------------------------------------------------*/
/** 
  * @brief  UART Config
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void UART_Config(void)
{
	/* (1) Enable UART2 Control */
	POFFCR1 = UART2_CONTORL_ENABLE;

	/* (2) Setting UART2 boudrate 115200bps, data length 8bit, no parity, stop bit 1bit */
	UART2CR1 = UART2_TRANS_STOP_BIT_LENGTH | UART2_PARITY | UART2_TXD_UART2_OUTPUT | UART2_CLOCK;
	UART2CR2 = UART2_RT_CLOCK | UART2_NOIZE_CANCEL_TIME | UART2_RECIEVE_STOP_BIT_LENGTH;
	UART2DR = UART2_BAUD_RATE;
	UART2CR1_TXE  = SET;
	UART2CR1_RXE  = SET;

	/* (3) Enable interrupt of TXD2/RXD2 */
	__DI();
	EIR_INTTXD2 = SET;
	EIR_INTRXD2 = SET;
	__EI();
}

/*--------------------------------------------------*/
/** 
  * @brief  Key Input
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Key(void)
{
	UINT8_t keyTemp;

	keyTemp = (UINT8_t)(P1PRD & 0x04);
	keyTemp ^= 0xFF;

	if(keybuf == keyTemp) {
		if(--keychat == 0) {
			keychat = KEYIN_CNT;
			keycode = keybuf;
		}
		else {
			/* Do nothing */
		}
	}
	else {
		keychat = KEYIN_CNT;
		keybuf = keyTemp;
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  UART data send
  * @param  *buf:send data buffer pointer
  * @param  txsize:send data size
  * @param  rxsize:response data size
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void UART_Data_Send(UINT8_t *buf, UINT8_t txsize, UINT8_t rxsize)
{
	send_cnt = 0;
	send_size = txsize;
	receive_size = rxsize;
	send_pnt = buf;
	send_flg = SET;

	TD2BUF = send_pnt[send_cnt];
}
/**
 *  @}
 */ /* End of group FLASH_SROM_UART_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_SROM_UART_Exported_functions FLASH_SROM_UART Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();

	/*------------------------*/
	/* UART Setting           */
	/*------------------------*/
	UART_Config();

	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
		Key();

		switch(Step){
			case 0:
				/* SW input wait */
				if((keycode & 0x04) != 0){
					/* send matching data 1 */
					txbuf[0] = MATCHING_DATA1;
					UART_Data_Send(txbuf, 1, 1);
					Step++;
				}
				else {
					/* Do nothing */
				}
				break;

			case 1:
				/* data receive wait */
				if((send_flg == CLEAR) && (receive_flg == SET)){
					if((receive_error == CLEAR) && (rxbuf[0] == MATCHING_DATA1)){
						/* send matching data 2 */
						txbuf[0] = MATCHING_DATA2;
						UART_Data_Send(txbuf, 1, 1);
						Step++;
					}
					else{
						/* error */
						Step = 0xFF;
					}
				}
				else {
					/* Do nothing */
				}
				break;

			case 2:
				/* data receive wait */
				if((send_flg == CLEAR) && (receive_flg == SET)){
					if((receive_error == CLEAR) && (rxbuf[0] == MATCHING_DATA2)){
						/* send erase command */
						txbuf[0] = COMMAND_ERASE;
						UART_Data_Send(txbuf, 1, 1);
						Step++;
					}
					else{
						/* error */
						Step = 0xFF;
					}
				}
				else {
					/* Do nothing */
				}
				break;

			case 3:
				/* data receive wait */
				if((send_flg == CLEAR) && (receive_flg == SET)){
					if((receive_error == CLEAR) && (rxbuf[0] == COMMAND_ERASE)){
						/* send password, erase area */
						txbuf[0] = PSIZE_ADDRH;
						txbuf[1] = PSIZE_ADDRM;
						txbuf[2] = PSIZE_ADDRL;

						txbuf[3] = PDATA_ADDRH;
						txbuf[4] = PDATA_ADDRM;
						txbuf[5] = PDATA_ADDRL;

						txbuf[6] = PASSWORD1;
						txbuf[7] = PASSWORD2;
						txbuf[8] = PASSWORD3;
						txbuf[9] = PASSWORD4;
						txbuf[10] = PASSWORD5;
						txbuf[11] = PASSWORD6;
						txbuf[12] = PASSWORD7;
						txbuf[13] = PASSWORD8;
						txbuf[14] = PASSWORD9;
						txbuf[15] = PASSWORD10;

						txbuf[16] = PARA_CHIP_ERASE;
						UART_Data_Send(txbuf, 17, 2);
						Step++;
					}
					else{
						/* error */
						Step = 0xFF;
					}
				}
				else {
					/* Do nothing */
				}
				break;

			case 4:
				/* data receive wait */
				if((send_flg == CLEAR) && (receive_flg == SET)){
					if(receive_error == CLEAR){
						/* send write command */
						txbuf[0] = COMMAND_WRITE;
						UART_Data_Send(txbuf, 1, 1);
						Step++;
					}
					else{
						/* error */
						Step = 0xFF;
					}
				}
				else {
					/* Do nothing */
				}
				break;

			case 5:
				/* data receive wait */
				if((send_flg == CLEAR) && (receive_flg == SET)){
					if((receive_error == CLEAR) && (rxbuf[0] == COMMAND_WRITE)){
						/* send password */
						txbuf[0] = PSIZE_ADDRH;
						txbuf[1] = PSIZE_ADDRM;
						txbuf[2] = PSIZE_ADDRL;

						txbuf[3] = PDATA_ADDRH;
						txbuf[4] = PDATA_ADDRM;
						txbuf[5] = PDATA_ADDRL;

						UART_Data_Send(txbuf, 6, 0);
						Step++;
					}
					else{
						/* error */
						Step = 0xFF;
					}
				}
				else {
					/* Do nothing */
				}
				break;

			case 6:
				/* data send wait */
				if(send_flg  == CLEAR){
					/* send write data */
					UART_Data_Send((UINT8_t *)write_data_tbl, sizeof(write_data_tbl), 0);
					CNT=0;
					Step++;
				}
				else {
					/* Do nothing */
				}
				break;

			case 7:
			    /* 3ms wait */
				if(CNT < 0x200)
				 {
                     CNT++;
				 }
				 else
				 {
                     CNT=0;
					 Step++;
				 }
				break;

			case 8:
				/* data send wait */

				if(send_flg  == CLEAR){
					/* send write data */
					UART_Data_Send((UINT8_t *)write_enddata_tbl, sizeof(write_enddata_tbl), 3);
					Step++;
				}
				else {
					/* Do nothing */
				}
				break;

			case 9:
				/* data receive wait */
				if((send_flg == CLEAR) && (receive_flg == SET)){
					if((receive_error == CLEAR) && (rxbuf[0] == WRITE_OK)){
						/* send read command */
						txbuf[0] = COMMAND_READ;
						UART_Data_Send(txbuf, 1, 1);
						Step++;
					}
					else{
						/* error */
						Step = 0xFF;
					}
				}
				else {
					/* Do nothing */
				}
				break;

			case 10:
				/* data receive wait */
				if((send_flg == CLEAR) && (receive_flg == SET)){
					if((receive_error == CLEAR) && (rxbuf[0] == COMMAND_READ)){
						/* send password, read area */
						txbuf[0] = PSIZE_ADDRH;
						txbuf[1] = PSIZE_ADDRM;
						txbuf[2] = PSIZE_ADDRL;

						txbuf[3] = PDATA_ADDRH;
						txbuf[4] = PDATA_ADDRM;
						txbuf[5] = PDATA_ADDRL;

						txbuf[6] = RADDRH;
						txbuf[7] = RADDRM;
						txbuf[8] = RADDRL;

						txbuf[9] = RSIZEH;
						txbuf[10] = RSIZEM;
						txbuf[11] = RSIZEL;

						UART_Data_Send(txbuf, 12, (UINT8_t)((RSIZEH << 8) + (RSIZEM << 4) + (RSIZEL << 0)));
						Step++;
					}
					else{
						/* error */
						Step = 0xFF;
					}
				}
				else {
					/* Do nothing */
				}
				break;

			case 11:
				/* data receive wait */
				if((send_flg == CLEAR) && (receive_flg == SET)){
					if(receive_error == CLEAR){
						Step++;
					}
					else{
						/* error */
						Step = 0xFF;
					}
				}
				else {
					/* Do nothing */
				}
				break;

			case 12:
				/* Do nothing */
				break;

			default:
				/* Do nothing */
				break;
		}
	}
}


/*--------------------------------------------------*/
/** 
  * @brief  INTTXD2
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntTXD2(void)
{
	send_cnt++;
	if(send_cnt < send_size){
		TD2BUF = send_pnt[send_cnt];
	}
	else{
		send_flg = CLEAR;
		receive_flg = CLEAR;
		receive_cnt = 0;
		if(receive_size == 0){
			receive_flg = SET;
		}
		else {
			/* Do nothing */
		}
	}
}


/*--------------------------------------------------*/
/** 
  * @brief  INTRXD2
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntRXD2(void)
{
	UINT8_t state;
	UINT8_t data;
	
	state = UART2SR;
	data = RD2BUF;

	/* RBFL0 & error check */
	if(((state & 0xE0) == 0) && (send_flg == CLEAR)){
		rxbuf[receive_cnt] = data;
		receive_cnt++;
		if(receive_cnt >= receive_size){
			receive_flg = SET;
		}
		else {
			/* Do nothing */
		}
	}
	else{
		receive_error = SET;
		receive_flg = SET;
	}
}

/**
 *  @}
 */ /* End of group FLASH_SROM_UART_Exported_functions */

/**
 *  @}
 */ /* End of group FLASH_SROM_UART */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
