/** 
 *******************************************************************************
 * @file    main.c
 * @brief   KWU Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-07-19 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup KWU KWU Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup KWU_Private_macro KWU Private Macro
 *  @{
 */

/** 
 *  @defgroup KWU_KWxEN Macro Definition
 *  @brief    KWU Control Enable.
 *  @{
 */
#define KWI								((UINT8_t)0x01)			/*!< KWI1 enable */
/**
 *  @}
 */ /* End of group KWU_KWxEN Macro Definition */
/** 
 *  @defgroup KWU_KWxLE Macro Definition
 *  @brief    KWU STOP Release Level.
 *  @{
 */
#define KWU_LEVEL						((UINT8_t)0x00)			/*!< Lo level */
/**
 *  @}
 */ /* End of group KWU_KWxLE Macro Definition */
/** 
 *  @defgroup  Port Port Setting
 *  @brief     Port Setting
 *  @{
 */
#define PORT_OFF						((UINT8_t)0x00)			/*!< Port OFF */
#define PORT_ON							((UINT8_t)0x01)			/*!< Port ON */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
#define PORT_INPUT						((UINT8_t)0x00)			/*!< Port Mode Input */
#define PORT_PU_ON						((UINT8_t)0x01)			/*!< Port Pull up ON */
#define PORT_PU_OFF						((UINT8_t)0x00)			/*!< Port Pull up OFF */
/** 
 *  @}
 */ /* End of group Port */

/** 
 *  @}
 */ /* End of group KWU_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup KWU_Private_define KWU Private Define
 *  @{
 */
#define KEYIN_CNT						((UINT8_t)20)			/*!< keychat=20 */
/** 
 *  @}
 */ /* End of group KWU_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup KWU_Private_variables KWU Private Variables
 *  @{
 */
static UINT8_t key_already_flag = (UINT8_t)0;			/*!<  flag.        */
static UINT8_t confirm_flag     = (UINT8_t)0;			/*!<  flag.        */
static UINT8_t keybuf           = (UINT8_t)0;			/* key buffer */
static UINT8_t keycode          = (UINT8_t)0;			/* key confirm */
static UINT8_t keychat          = KEYIN_CNT;			/* key chat */
/**
 *  @}
 */ /* End of group KWU_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup KWU_Private_fuctions KWU Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void KWU_Config(void);
static void Key(void);
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Select output mode on PORT 25 (LED0) */
	P2CR5 = PORT_OUTPUT;
	/* (2) Turn on PORT 25 (LED0) */
	P25 = PORT_ON;
	/* (3) Select output mode on PORT 26 (LED1) */
	P2CR6 = PORT_OUTPUT;
	/* (4) Turn off PORT 26 (LED1) */
	P26 = PORT_OFF;
	/* (5) Select input mode on PORT 41 (SW0) */
	P4CR1 = PORT_INPUT;
}

/*--------------------------------------------------*/
/** 
  * @brief  KWU_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void KWU_Config(void)
{
	UINT8_t i;
	UINT8_t j;
	
	/* PULL UP ON PORT 41 (KWI1) */
	P4PU1 = PORT_PU_ON;
	
	i = KWI;
	j = KWU_LEVEL;
	/* (1) Enable KWU Control */
	/* (2) Setting release level */
	if(i < 4) {
		KWUCR0 = (UINT8_t)(0x01 << (i * 2));				/* (1) */
		KWUCR0 |= (UINT8_t)(j * (0x01 << ((i * 2) + 1)));			/* (2) */
	}
	else if(i < 8) {
		KWUCR1 = (UINT8_t)(0x01 << ((i - 4) * 2));				/* (1) */
		KWUCR1 |= (UINT8_t)(j * (0x01 << (((i - 4) * 2) + 1)));	/* (2) */
	}
	else {
		/* Do nothing */
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  Key
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Key(void)
{
	UINT8_t keyTemp;
	
	keyTemp = (UINT8_t)(0x00);
	keyTemp |= (UINT8_t)(P4PRD & 0x02);
	keyTemp ^= 0xff;
	keyTemp &= 0x02;
	
	if(keybuf == keyTemp) {
		if(--keychat == 0) {
			keychat = KEYIN_CNT;
			keycode = keybuf;
			if(keycode == 0) {
				if(key_already_flag == SET) {
					key_already_flag = CLEAR;
					confirm_flag = SET;
				}
				else {
					/* Do nothing */
				}
			}
			else {
				if(key_already_flag == 0) {
					key_already_flag = SET;
				}
				else {
					/* Do nothing */
				}
			}
		}
		else {
			/* Do nothing */
		}
	}
	else {
		keychat = KEYIN_CNT;
		keybuf = keyTemp;
	}
}

/**
 *  @}
 */ /* End of group TCA_EXT_TRIG_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup KWU_Exported_functions KWU Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
		/*------------------------*/
		/* Key Check              */
		/*------------------------*/
		Key();
		
		if(confirm_flag == SET) {
			confirm_flag = CLEAR;
			
			__DI();
			P25 = PORT_OFF;				/* LED0 OFF */
			P26 = PORT_ON;				/* LED1 ON */
			KWU_Config();				/* KWU enable */
			
			OUTEN = SET;
			RELM = SET;
			
			WUCCR = 0x80;				/* fc = 10MHz */
			WUCDR = 0x2F;				/* 300us/6.4us = 46.875  47 */
			
			STOP = SET;					/* STOP mode */
			
			__asm("NOP");
			__asm("NOP");
			__asm("NOP");
			
			while(P4PRD1 == CLEAR) {
			}
			/* KWU disable */
			KWUCR0 = (UINT8_t)0x00;
			KWUCR1 = (UINT8_t)0x00;
			
			P4PU1 = PORT_PU_OFF;
			
			P25 = PORT_ON;				/* LED0 ON */
			P26 = PORT_OFF;				/* LED1 OFF */
			
			__EI();
		}
		else {
			/* Do nothing */
		}
	}
}

/**
 *  @}
 */ /* End of group KWU_Exported_functions */

/**
 *  @}
 */ /* End of group KWU */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
