/** 
 *******************************************************************************
 * @file    main.c
 * @brief   LOW_POWER_SLOW1 Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-07-19 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdlib.h>
#include "TMP89Fx.h"  /*!< TMP89Fx Group Header file. */
#include "cmn_type.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup LOW_POWER_SLOW1 LOW_POWER_SLOW1 Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LOW_POWER_SLOW1_Private_macro LOW_POWER_SLOW1 Private Macro
 *  @{
 */

/** 
 *  @defgroup TBT_TBTCK Macro Definition
 *  @brief    TBT Interval.
 *  @{
 */
#define TBT_INTERVAL					((UINT8_t)0x00)			/*!< TBT Interrupt Clock(=fs/2^15) */
/**
 *  @}
 */ /* End of group TBT_TBTCK Macro Definition */
/** 
 *  @defgroup TBT_TBTEN TBT Start
 *  @brief    TBT Start.
 *  @{
 */
#define TBT_TBTEN_START					((UINT8_t)0x01)			/*!< TBT Start(Innterrupt request signal enable) */
/**
 *  @}
 */ /* End of group TBT_TBTEN */
/** 
 *  @defgroup RTC_RTCEN RTC Enable
 *  @brief    RTC Control Enable.
 *  @{
 */
#define RTC_RTCEN_ENABLE				((UINT8_t)0x20)			/*!< RTC enable */
/**
 *  @}
 */ /* End of group RTC_RTCEN */
/** 
 *  @defgroup RTC_RTCSEL Macro Definition
 *  @brief    RTC Interval.
 *  @{
 */
#define RTC_INTERVAL					((UINT8_t)0x00)			/*!< RTC Interval(= 2^15/fs = 1.000[s]) */
/**
 *  @}
 */ /* End of group RTC_RTCSEL Macro Definition */
/** 
 *  @defgroup RTC_RTCRUN RTC Start
 *  @brief    RTC Start.
 *  @{
 */
#define RTC_RTCRUN_START				((UINT8_t)0x01)			/*!< RTC Start */
/**
 *  @}
 */ /* End of group RTC_RTCRUN */
/** 
 *  @defgroup  Port Port Setting
 *  @brief     Port Setting
 *  @{
 */
#define PORT_OFF						((UINT8_t)0x00)			/*!< Port OFF */
#define PORT_ON							((UINT8_t)0x01)			/*!< Port ON */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
#define PORT_INPUT						((UINT8_t)0x00)			/*!< Port Mode Input */
#define PORT_PU_ON						((UINT8_t)0x01)			/*!< Port Pull up ON */
/** 
 *  @}
 */ /* End of group Port */
/** 
 *  @defgroup  Mode
 *  @brief     Mode Setting
 *  @{
 */
#define MODE_STOP						((UINT8_t)0x00)			/*!< Mode : STOP */
#define MODE_SLOW1						((UINT8_t)0x01)			/*!< Mode : SLOW1 */
#define MODE_SLOW2						((UINT8_t)0x02)			/*!< Mode : SLOW2 */
#define MODE_IDLE0						((UINT8_t)0x03)			/*!< Mode : IDLE0 */
#define MODE_IDLE1						((UINT8_t)0x04)			/*!< Mode : IDLE1 */
#define MODE_IDLE2						((UINT8_t)0x05)			/*!< Mode : IDLE2 */
#define MODE_SLEEP0						((UINT8_t)0x06)			/*!< Mode : SLEEP0 */
#define MODE_SLEEP1						((UINT8_t)0x07)			/*!< Mode : SLEEP1 */
#define MODE_NORMAL1					((UINT8_t)0x08)			/*!< Mode : NORMAL1 */
#define MODE_NORMAL2					((UINT8_t)0x09)			/*!< Mode : NORMAL2 */

#define KEYIN_CNT						((UINT8_t)3)			/*!< keychat=3 */

/** 
 *  @}
 */ /* End of group Mode */

/** 
 *  @}
 */ /* End of group LOW_POWER_SLOW1_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LOW_POWER_SLOW1_Private_variables LOW_POWER_SLOW1 Private Variables
 *  @{
 */
static UINT8_t keybuf = (UINT8_t)0;								/*!<  key buffer */
static UINT8_t keycode = (UINT8_t)0;							/*!<  key confirm */
static UINT8_t keychat = KEYIN_CNT;								/*!<  key now */
static UINT8_t confirm_flag = (UINT8_t)0;						/*!<  flag.        */
static UINT8_t key_already_flag = (UINT8_t)0;					/*!<  flag.        */
static UINT8_t long_flag = (UINT8_t)0;							/*!<  flag.        */
static UINT8_t long_comfirm_flag = (UINT8_t)0;					/*!<  flag.        */
static UINT8_t mode = MODE_SLOW1;								/*!<  mode.        */
static UINT8_t mode_flag = (UINT8_t)0;							/*!<  flag.        */
static UINT16_t tbt_cnt = (UINT16_t)0;							/*!<  tbt count.   */
static UINT8_t rtc_cnt = (UINT8_t)0;							/*!<  rtc count.   */
static UINT8_t sleep0_cnt = (UINT8_t)0;							/*!<  sleep0 count.   */
static UINT8_t sleep1_cnt = (UINT8_t)0;							/*!<  sleep1 count.   */
/**
 *  @}
 */ /* End of group LOW_POWER_SLOW1_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LOW_POWER_SLOW1_Private_fuctions LOW_POWER_SLOW1 Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void KWU_Config(void);
static void Key(void);
static void Slow1_Mode_Config(void);
static void Sleep0_Mode_Config(void);
static void Sleep1_Mode_Config(void);
static void RTC_Config(void);
static void TBT_Config(void);
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Select output mode on PORT 25 (LED0) */
	P2CR5 = PORT_OUTPUT;
	/* (2) Turn off PORT 25 (LED0) */
	P25 = PORT_OFF;
	/* (3) Select output mode on PORT 26 (LED1) */
	P2CR6 = PORT_OUTPUT;
	/* (4) Turn on PORT 26 (LED1) */
	P26 = PORT_OFF;
	/* (5) Select output mode on PORT 27 (LED2) */
	P2CR7 = PORT_OUTPUT;
	/* (6) Turn on PORT 27 (LED2) */
	P27 = PORT_OFF;
	/* (7) Select input mode on PORT 41 (SW0) */
	P4CR1 = PORT_INPUT;
	/* (8) Select input mode on PORT 13 (SW1) */
	P1CR3 = PORT_INPUT;
}
/*--------------------------------------------------*/
/** 
  * @brief  KWU_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void KWU_Config(void)
{
	/* PULL UP ON PORT 41 (KWI1) */
	P4PU1 = PORT_PU_ON;
	
	/* (1) Enable KWU Control */
	/* (2) Setting release level */
	KWUCR0 = (UINT8_t)0x04;							/* (1) */
	KWUCR0 |= (UINT8_t)0x00;						/* (2) */
}
/*--------------------------------------------------*/
/** 
  * @brief  Key
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Key(void)
{
	UINT8_t keyTemp;
	UINT8_t keycnt=(UINT8_t)0;
	UINT8_t i;
	UINT8_t chk;
	
	keyTemp = (UINT8_t)((P1PRD & 0x08) >> 2);
	keyTemp |= (UINT8_t)((P4PRD & 0x02) >> 1);
	keyTemp ^= (UINT8_t)0xFF;
	keyTemp &= 0x03;
	
	if(keybuf == keyTemp) {
		if(--keychat == 0) {
			keychat = KEYIN_CNT;
			for(i=0 ; i<4 ; i++) {
				chk = (UINT8_t)(0x01 << i);
				if((keybuf & chk) != 0) {
					keycnt++;
				}
				else {
					/* Do nothing */
				}
			}
			if(keycnt == 1) {
				if(key_already_flag == CLEAR){
					TBTEN = CLEAR;
					TBT_Config();
					tbt_cnt = (UINT16_t)0;
					keycode = keybuf;
				}
				key_already_flag = SET;
				if(tbt_cnt >= 3) {
					long_flag = SET;
				}
				else
				{
					/* Do nothing */
				}
			}
			else if(keycnt == 0) {
				if(long_flag == SET) {
					long_comfirm_flag = SET;
					long_flag = CLEAR;
					tbt_cnt = CLEAR;
					key_already_flag = CLEAR;
				}
				else {
					if(key_already_flag == SET){
						key_already_flag = CLEAR;
						confirm_flag = SET;
					}
					else {
						/* Do nothing */
					}
				}
			}
			else {
				/* multi disable */
				long_flag = CLEAR;
				tbt_cnt = CLEAR;
				key_already_flag = CLEAR;
				keybuf = CLEAR;
			}
		}
		else {
			/* Do nothing */
		}
	}
	else {
		keychat = KEYIN_CNT;
		keybuf = keyTemp;
	}
}
/*--------------------------------------------------*/
/** 
  * @brief  NORMAL Mode Config
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Slow1_Mode_Config(void)
{
	Port_Config();
}
/*--------------------------------------------------*/
/** 
  * @brief  SLEEP0 Mode Config
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Sleep0_Mode_Config(void)
{
	mode = MODE_SLEEP0;			/*!<  SLEEP0 mode.        */
	
	P25 = PORT_OFF;				/* LED0 OFF */
	P26 = PORT_OFF;				/* LED1 OFF */
	P27 = PORT_ON;				/* LED2 ON */
	
	/* RTCRUN = 0 */
	RTCRUN = CLEAR;
	
	TBT_Config();
	TGHALT = SET;
	
	mode_flag = SET;
}
/*--------------------------------------------------*/
/** 
  * @brief  SLEEP1 Mode Config
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Sleep1_Mode_Config(void)
{
	TBTEN = CLEAR;
	
	mode = MODE_SLEEP1;			/*!<  SLEEP1 mode.        */
	
	P25 = PORT_OFF;				/* LED0 OFF */
	P26 = PORT_ON;				/* LED1 ON */
	P27 = PORT_OFF;				/* LED2 OFF */
	
	RTC_Config();
	IDLE = SET;
	
	mode_flag = SET;
}
/*--------------------------------------------------*/
/** 
  * @brief  RTC_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void RTC_Config(void)
{
	/* (1) Enable RTC Control */
	/* (2) Enable interrupt of RTC */
	/* (3) Setting interrupt frequency */
	POFFCR2 = RTC_RTCEN_ENABLE;		/* (1) */
	__DI();
	EIR_INTRTC = 1;					/* (2) */
	__EI();
	RTCCR  = RTC_INTERVAL;			/* (3) */
	/* RTCRUN = 1 */
	RTCRUN = RTC_RTCRUN_START;
}
/*--------------------------------------------------*/
/** 
  * @brief  TBT_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TBT_Config(void)
{
	/* (1) Enable interrupt of TBT */
	/* (2) Setting interrupt frequency,TBTCR[2:0]=(fs/2^15) */
	/* (3) Enable interrupt request signal,TBTCR[3]=1 */
	__DI();
	EIR_INTTBT = SET;				/* (1) */
	__EI();
	TBTCR  = TBT_INTERVAL;			/* (2) */
	TBTEN  = TBT_TBTEN_START;		/* (3) */
}

/**
 *  @}
 */ /* End of group TCA_EXT_TRIG_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LOW_POWER_SLOW1_Exported_functions LOW_POWER_SLOW1 Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	/* SLOW2 mode */
	SYSCK = SET;
	__asm("NOP");
	__asm("NOP");

	/* SLOW1 */
	XEN = CLEAR;					/* Disable the High-frequency Clock */

	RTC_Config();
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
		/*------------------------*/
		/* Key Check              */
		/*------------------------*/
		Key();
		if(mode_flag == 0) {
			if(confirm_flag == SET) {
				confirm_flag = CLEAR;
				if(mode == MODE_SLOW1) {
					if((keycode & 0x01) != 0) {
						/* SLOW2 */
						keycode = CLEAR;
						mode = MODE_SLOW2;			/*!<  SLOW2 mode.        */
						rtc_cnt = CLEAR;
						TBTEN = CLEAR;

						P25 = PORT_ON;				/* LED0 ON */
						P26 = PORT_OFF;				/* LED1 OFF */
						P27 = PORT_OFF;				/* LED2 OFF */

						ILL = 0xEF;
						WUCCR = 0x80;				/* WUCRST = SET WUCSEL =fc */
						WUCDR = 0x2F;

						XEN = SET;
						while((ILL & 0x10) == 0){
						}
						ILL = 0xEF;	
																	
						RTC_Config();
					}
					else if((keycode & 0x02) != 0) {
						keycode = CLEAR;
						/* SLEEP0 */
						sleep0_cnt = CLEAR;
						Sleep0_Mode_Config();
					}
					else {
						/* Do nothing */
					}
				}else {
					/* Do nothing */
				}
			}
			else if(long_comfirm_flag == SET) {
				long_comfirm_flag = CLEAR;
				if((keycode & 0x01) != 0){	
					key_already_flag = CLEAR;
					/* STOP mode */
					OUTEN = SET;
					RELM = SET;
					while(P4PRD1 == CLEAR) {
					}
					__DI();
				
					P25 = PORT_ON;				/* LED0 ON */
					P26 = PORT_ON;				/* LED1 ON */
					P27 = PORT_ON;				/* LED2 ON */
				
					KWU_Config();

					WUCCR = 0x82;
					WUCDR = 0xE8;				/* 450 [ms] / 1.953 [ms] = 230.4 0xE8 */

					STOP = SET;					/* STOP mode */
				
					__asm("NOP");
					__asm("NOP");
					__asm("NOP");
				
					while(P4PRD1 == CLEAR) {
					}
					/* SLOW1 mode */
					Slow1_Mode_Config();
					mode = MODE_SLOW1;
					mode_flag = CLEAR;

					__EI();
					keycode = CLEAR;
				}else if((keycode & 0x02) != 0){	/* SLEEP1 */
					keycode = CLEAR;
					sleep1_cnt = CLEAR;
					Sleep1_Mode_Config();
				}
				else {
					/* Do nothing */
				}
			}
			else {
				/* Do nothing */
			}
		}
		else {
			if(mode == MODE_SLEEP0) {
				if(sleep0_cnt < 10) {
					/* SLEEP0 mode */
					Sleep0_Mode_Config();
				}
				else {
					/* SLOW1 mode */
					Slow1_Mode_Config();
					mode = MODE_SLOW1;
					mode_flag = CLEAR;
				}
			}
			else if(mode == MODE_SLEEP1) {
				if(sleep1_cnt < 10) {
					/* SLEEP1 mode */
					Sleep1_Mode_Config();
				}
				else {
					/* SLOW1 mode */
					Slow1_Mode_Config();
					mode = MODE_SLOW1;
					mode_flag = CLEAR;
				}
			}
			else {
				/* Do nothing */
			}
		}
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  INTRTC
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntRTC(void)
{
	rtc_cnt++;
	if(mode == MODE_SLEEP1) {
		/* SLEEP1 */
		sleep1_cnt++;
		Slow1_Mode_Config();
	}
	else if(mode == MODE_SLOW2) {
		/* SLOW2 */
		if(rtc_cnt >= 10) {
			/* 10s */
			rtc_cnt = CLEAR;
			mode_flag = CLEAR;
			mode = MODE_SLOW1;
			Slow1_Mode_Config();

			XEN = CLEAR;
		}
		else {
			/* Do nothing */
		}
	}
	else {
		/* Do nothing */
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  INTTBT
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntTBT(void)
{
	if(mode == MODE_SLOW1){
		tbt_cnt++;
	}
	else {
		/* Do nothing */
	}
	if(mode == MODE_SLEEP0) {
		/* SLEEP0 */
		sleep0_cnt++;
		Slow1_Mode_Config();
	}
	else {
		/* Do nothing */
	}

}
/**
 *  @}
 */ /* End of group LOW_POWER_SLOW1_Exported_functions */

/**
 *  @}
 */ /* End of group LOW_POWER_SLOW1 */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
