/** 
 *******************************************************************************
 * @file    main.c
 * @brief   PORT Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-07-19 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup PORT PORT Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup PORT_Private_macro PORT Private Macro
 *  @{
 */

/** 
 *  @defgroup  Port Port Setting
 *  @brief     Port Setting
 *  @{
 */
#define PORT_OFF						((UINT8_t)0x00)			/*!< Port OFF */
#define PORT_ON					    	((UINT8_t)0x01)			/*!< Port ON */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
#define PORT_INPUT						((UINT8_t)0x00)			/*!< Port Mode Input */
/** 
 *  @}
 */ /* End of group Port */

/** 
 *  @}
 */ /* End of group PORT_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup PORT_Private_define PORT Private Define
 *  @{
 */
#define KEYIN_CNT						((UINT8_t)2)			/*!< keychat=2 */
/** 
 *  @}
 */ /* End of group PORT_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup PORT_Private_variables PORT Private Variables
 *  @{
 */
static UINT8_t keybuf  = (UINT8_t)0;				/* key buffer */
static UINT8_t keycode = (UINT8_t)0;				/* key confirm */
static UINT8_t keychat = KEYIN_CNT;					/* key now */
static UINT8_t ledbuff = (UINT8_t)0;				/* led buffer */
/**
 *  @}
 */ /* End of group PORT_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup PORT_Private_fuctions PORT Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void Key(void);
static void Led(void);

/*--------------------------------------------------*/
/** 
  * @brief  PORT_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Select output mode on PORT 25 (LED0) */
	P2CR5 = PORT_OUTPUT;
	/* (2) Turn off PORT 25 (LED0) */
	P25 = PORT_OFF;
	/* (3) Select output mode on PORT 26 (LED1) */
	P2CR6 = PORT_OUTPUT;
	/* (4) Turn off PORT 26 (LED1) */
	P26 = PORT_OFF;
	/* (5) Select input mode on PORT 12 (SW0) */
	P1CR2 = PORT_INPUT;
	/* (6) Select input mode on PORT 13 (SW1) */
	P1CR3 = PORT_INPUT;
}

/*--------------------------------------------------*/
/** 
  * @brief  Key
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Key(void)
{
	UINT8_t keyTemp;
	
	keyTemp = (UINT8_t)((P1PRD & 0x0C) >> 2);
	keyTemp ^= (UINT8_t)0xFF;
	
	if(keybuf == keyTemp) {
		if(--keychat == 0) {
			keychat = KEYIN_CNT;
			keycode = keybuf;
		}
		else {
			/* Do nothing */
		}
	}
	else {
		keychat = KEYIN_CNT;
		keybuf = keyTemp;
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  LED
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Led(void)
{
	ledbuff = (UINT8_t)0;

	/* key confirm */
	if((keycode & 0x01) != 0) {
		ledbuff |= (UINT8_t)0x20;		/* LED0 ON */
	}
	else {
		/* Do nothing */
	}

	if((keycode & 0x02) != 0) {
		ledbuff |= (UINT8_t)0x40;		/* LED1 ON */
	}
	else {
		/* Do nothing */
	}
	P2DR = (UINT8_t)(ledbuff & 0x60);
	
}
/**
 *  @}
 */ /* End of group TCA_EXT_TRIG_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_EXT_TRIG_Exported_functions TCA_EXT_TRIG Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
		Key();
		Led();
	}
}

/**
 *  @}
 */ /* End of group PORT_Exported_functions */

/**
 *  @}
 */ /* End of group PORT */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
