/** 
 *******************************************************************************
 * @file    main.c
 * @brief   RTC Sample Application.
 * @version V1.0.1.0
 * $Date:: 2019-06-10 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup RTC RTC Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup RTC_Private_macro RTC Private Macro
 *  @{
 */

/** 
 *  @defgroup RTC_RTCEN RTC Enable
 *  @brief    RTC Control Enable.
 *  @{
 */
#define RTC_RTCEN_ENABLE				((UINT8_t)0x20)			/*!< RTC enable */
/**
 *  @}
 */ /* End of group RTC_RTCEN */
/** 
 *  @defgroup RTC_RTCSEL Macro Definition
 *  @brief    RTC Interval.
 *  @{
 */
/*#define RTC_SEL_2_15_FS				((UINT8_t)0x00)	*/		/*!< RTC Interval(= 2^15/fs = 1.000[s]) */
#define RTC_SEL_2_14_FS				((UINT8_t)0x02)			/*!< RTC Interval(= 2^14/fs = 0.500[s]) */
/*#define RTC_SEL_2_13_FS				((UINT8_t)0x04)	*/		/*!< RTC Interval(= 2^13/fs = 0.250[s]) */
/*#define RTC_SEL_2_12_FS				((UINT8_t)0x06)	*/		/*!< RTC Interval(= 2^12/fs = 0.125[s]) */
/*#define RTC_SEL_2_11_FS				((UINT8_t)0x08)	*/		/*!< RTC Interval(= 2^11/fs = 62.50[ms]) */
/*#define RTC_SEL_2_10_FS				((UINT8_t)0x0A)	*/		/*!< RTC Interval(= 2^10/fs = 31.25[ms]) */
/*#define RTC_SEL_2_9_FS				((UINT8_t)0x0C)	*/		/*!< RTC Interval(= 2^9/fs  = 15.62[ms]) */
/*#define RTC_SEL_2_8_FS				((UINT8_t)0x0E)	*/		/*!< RTC Interval(= 2^8/fs  =  7.81[ms]) */
/**
 *  @}
 */ /* End of group RTC_RTCSEL Macro Definition */
/** 
 *  @defgroup RTC_RTCRUN RTC Start
 *  @brief    RTC Start.
 *  @{
 */
#define RTC_RTCRUN_START				((UINT8_t)0x01)			/*!< RTC Start */
/**
 *  @}
 */ /* End of group RTC_RTCRUN */
/** 
 *  @defgroup  Port_Output Port Output ON/OFF
 *  @brief     Port Output ON/OFF
 *  @{
 */
#define PORT_OFF						((UINT8_t)0x00)			/*!< Port OFF */
/** 
 *  @}
 */ /* End of group Port_Output */
 /** 
 *  @defgroup  Port_Mode Port Mode
 *  @brief     Port Mode Input/Output
 *  @{
 */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
/** 
 *  @}
 */ /* End of group Port_Output */

/** 
 *  @}
 */ /* End of group RTC_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup RTC_Private_fuctions RTC Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void RTC_Config(void);
static void RTC_Start(void);
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Select output mode on PORT 25 (LED0) */
	P2CR5 = PORT_OUTPUT;
	/* (2) Turn off PORT 25 (LED0) */
	P25 = PORT_OFF;
}

/*--------------------------------------------------*/
/** 
  * @brief  RTC_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void RTC_Config(void)
{
	/* (1) Enable RTC Control */
	/* (2) Enable interrupt of RTC */
	/* (3) Setting interrupt frequency,RTCCR[3:1]=(2^14/fs) */
	POFFCR2 = RTC_RTCEN_ENABLE;		/* (1) */
	__DI();
	EIR_INTRTC = 1;					/* (2) */
	__EI();
	RTCCR  = RTC_SEL_2_14_FS;		/* (3) */
}
/*--------------------------------------------------*/
/** 
  * @brief  RTC_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void RTC_Start(void)
{
	/* RTCRUN = 1 */
	RTCRUN = RTC_RTCRUN_START;
}
    
/**
 *  @}
 */ /* End of group RTC_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup RTC_Exported_functions RTC Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	
	/*------------------------*/
	/* RTC Setting   */
	/*------------------------*/ 
	RTC_Config();
	/* Timer Start */
	RTC_Start();
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  INTRTC
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntRTC(void)
{
	static UINT8_t i = 0;
	
	i = P25;
	i ^= 1;
	P25 = i;
}
/**
 *  @}
 */ /* End of group RTC_Exported_functions */

/**
 *  @}
 */ /* End of group RTC */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
