/** 
 *******************************************************************************
 * @file    main.c
 * @brief   SBI Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-07-19 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdio.h>
#include	<string.h>
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"
#include	"uart_io.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup SBI SBI Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SBI_Private_macro SBI Private Macro
 *  @{
 */

/** 
 *  @defgroup SBI_SBI0EN SBI Enable
 *  @brief    SBI Control Enable.
 *  @{
 */
#define SBI_CONTORL_ENABLE				((UINT8_t)0x10)			/*!< SBI enable */
/**
 *  @}
 */ /* End of group SBI_SBI0EN */
/** 
 *  @defgroup SBI_SBI0CR1 Macro Definition
 *  @brief    SBI0CR1 Setting.
 *  @{
 */
#define SBI_DATA_BIT					((UINT8_t)0x00)		/*!< data bits(8bit) */
#define SBI_ACK							((UINT8_t)0x10)		/*!< ack enable */
#define SBI_NACK						((UINT8_t)0x00)		/*!< ack diable */
#define SBI_SCK							((UINT8_t)0x06)		/*!< sck */
/**
 *  @}
 */ /* End of group SBI_SBI0CR1 Macro Definition */
/** 
 *  @defgroup SBI_SBI0CR2 Macro Definition
 *  @brief    SBI0CR2 Setting.
 *  @{
 */
#define SBI_MODE_SLAVE					((UINT8_t)0x00)		/*!< slave */
#define SBI_MODE_MASTER					((UINT8_t)0x80)		/*!< master */
#define SBI_TRX_RECIEVER				((UINT8_t)0x00)		/*!< reciever */
#define SBI_TRX_TRANSMITTER				((UINT8_t)0x40)		/*!< trans */
#define SBI_BB_STOP						((UINT8_t)0x00)		/*!< stop condition */
#define SBI_BB_START					((UINT8_t)0x20)		/*!< start condition */
#define SBI_PIN_INT_REQ					((UINT8_t)0x10)		/*!< int request */
#define SBI_SBIM_PORT					((UINT8_t)0x00)		/*!< port mode */
#define SBI_SBIM_SERIALBUS				((UINT8_t)0x08)		/*!< sirial bus */
/**
 *  @}
 */ /* End of group SBI_SBI0CR2 Macro Definition */

/** 
 *  @defgroup SBI_I2C0AR Macro Definition
 *  @brief    I2C0AR Setting.
 *  @{
 */
/*#define SBI_FORMAT_FREE					((UINT8_t)0x01)	*/	/*!< free data */
#define SBI_FORMAT_I2C					((UINT8_t)0x00)		/*!< I2C */
#define SBI_SLAVE_ADDR					((UINT8_t)0x22)		/*!< slave addr : 0x22 */
/**
 *  @}
 */ /* End of group SBI_I2C0AR Macro Definition */
/** 
 *  @defgroup  Key 
 *  @brief     Key Setting
 *  @{
 */
#define PORT_ON							((UINT8_t)0x01)			/*!< Port ON */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
#define PORT_FUNC_ON					((UINT8_t)0x01)			/*!< Port Function Change */
/** 
 *  @}
 */ /* End of group Key */

/** 
 *  @}
 */ /* End of group SBI_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SBI_Private_define SBI Private Define
 *  @{
 */
#define MODE_SLAVE						((UINT8_t)0x00)			/*!< Mode : slave */
#define MODE_MASTER						((UINT8_t)0x01)			/*!< Mode : master */
#define STEP_MASTER_IDLE				((UINT8_t)0x00)			/*!< Step : master */
#define STEP_MASTER_INIT				((UINT8_t)0x01)
#define STEP_MASTER_START_CON			((UINT8_t)0x02)
#define STEP_MASTER_DATA_WRITE			((UINT8_t)0x03)
#define STEP_MASTER_START_CON_READ		((UINT8_t)0x04)
#define STEP_MASTER_READ				((UINT8_t)0x05)
#define STEP_MASTER_READ_END			((UINT8_t)0x06)

#define SBI_DATA						((UINT8_t)0xA3)			/*!< Data : 0xA3 */

const UINT8_t disp_tbl1[] = {"Command>  "};
const UINT8_t disp_tbl2[] = {"error\n"};
const UINT8_t disp_tbl3[] = {"read data: 0x"};
/** 
 *  @}
 */ /* End of group SBI_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SBI_Private_variables SBI Private Variables
 *  @{
 */
static UINT8_t mode             = MODE_SLAVE;		/*!<  mode.        */
static UINT8_t trans_step       = STEP_MASTER_INIT;	/*!<  step.        */
static UINT8_t recieve_buff_sbi_slave = (UINT8_t)0;		/*!< SBI Recieve buffer(slave).        */
static UINT8_t recieve_buff_sbi_master = (UINT8_t)0;	/*!< SBI Recieve buffer(master).        */
static UINT8_t direction_flag   = 0;				/*!< direction        */
static const UINT8_t cmpstring_m[6] = "master";		/*!< write command      */
static const UINT8_t cmpstring_w[5] = "write";		/*!< write command      */
static const UINT8_t cmpstring_r[4] = "read";		/*!< write command      */
/**
 *  @}
 */ /* End of group SBI_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SBI_Private_fuctions SBI Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void SBI_Init(void);
static void SBI_Start_Condition(void);
static void SBI_Stop_Condition(void);
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Port Function Control : SDA0 */
	P2FC3 = PORT_FUNC_ON;
	/* (2) Port Function Control : SCL0 */
	P2FC4 = PORT_FUNC_ON;
	
	P2CR3 = PORT_OUTPUT;
	P2CR4 = PORT_OUTPUT;
	P23 = PORT_ON;
	P24 = PORT_ON;
}

/*--------------------------------------------------*/
/** 
  * @brief  SBI_Init
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void SBI_Init(void)
{
	UINT8_t port = (UINT8_t)0;
	
	__DI();
	EIR_INTSBI0 = SET;
	__EI();
	
	/* check sirial bus port : Hi */
	while(port == 0) {
		port = (UINT8_t)(P2PRD & 0x18);
	}
	
	/* (1) Enable SBI Control */
	/* (2) Setting SBI0CR2,sirial port */
	/* (3) Setting SBI0CR1,acknowledge mode & sck */
	/* (4) Setting I2C0AR,I2C */
	/* (5) Setting SBI0CR2,slave & reciever */
	POFFCR1 |= SBI_CONTORL_ENABLE;									/* (1) */
	SBI0CR2 = 0x02;
	SBI0CR2 = 0x01;
	
	SBI0CR2 = SBI_PIN_INT_REQ | SBI_SBIM_SERIALBUS;					/* (2) */
	SBI0CR1 = SBI_ACK | SBI_SCK;									/* (3) */
	I2C0AR =  SBI_FORMAT_I2C | (SBI_SLAVE_ADDR << 1);				/* (4) */
	SBI0CR2 = SBI_PIN_INT_REQ | SBI_SBIM_SERIALBUS;					/* (5) */

}
/*--------------------------------------------------*/
/** 
  * @brief  SBI START condition
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void SBI_Start_Condition(void)
{
	UINT8_t state = (UINT8_t)0;
	UINT8_t cnt = (UINT8_t)0;
	
	state = SBI0SR2;
	/* busy check */
	for(cnt = 0 ; cnt < 200 ; cnt++) {
		if((state & 0x20) == 0) {
			break;
		}
		else {
			/* Do nothing */
		}
	}
	if(cnt >= 200) {
		return;
	}
	else {
		/* Do nothing */
	}
	/* Setting SBI0CR1,data bit/ack/sck */
	SBI0CR1 = SBI_DATA_BIT | SBI_ACK | SBI_SCK;
	/* Setting SBI0DBR,slave addr/direcion */
	SBI0DBR = (UINT8_t)((SBI_SLAVE_ADDR << 1) | direction_flag);
	/* Setting SBI0CR2,master/trans/start/req/sirial */
	SBI0CR2 = SBI_MODE_MASTER | SBI_TRX_TRANSMITTER | SBI_BB_START | SBI_PIN_INT_REQ |SBI_SBIM_SERIALBUS;
}
/*--------------------------------------------------*/
/** 
  * @brief  SBI STOP condition & Slave addr
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void SBI_Stop_Condition(void)
{
	/* Setting SBI0CR2,master/trans/stop/req/sirial  */
	SBI0CR2 = SBI_MODE_MASTER | SBI_TRX_TRANSMITTER | SBI_BB_STOP | SBI_PIN_INT_REQ |SBI_SBIM_SERIALBUS;
}
/**
 *  @}
 */ /* End of group SBI_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SBI_Exported_functions SBI Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	UINT8_t i;
	UINT8_t a;
	UINT8_t check = (UINT8_t)1;

	unsigned char *buff = 0;
	
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	
	uart_initialize();
	/* Rx Start */
	UART2CR1_RXE = UART_RXE_ENABLE;
	
	Send_Display_Data((UINT8_t)sizeof(disp_tbl1), (UINT8_t *)disp_tbl1);
	
	/*------------------------*/
	/* SBI Init               */
	/*------------------------*/
	SBI_Init();
	
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
		if(cmp_flag == 1) {
			check = (UINT8_t)1;
			/* UART recieve */
			cmp_flag = (UINT8_t)0;
			if(mode == MODE_SLAVE) {
				/* slave */
				buff = (unsigned char *)&cmpstring_m;					/* "master" */
				for(i=0 ; i < rx_length ; i++) {
					if(*(buff) != rx_data[i]) {
						check = 0;
					}
					else {
						/* Do nothing */
					}
					buff++;
				}
				if(check == 1) {
					/* SBI initialize */
					SBI_Init();
					mode = MODE_MASTER;
					trans_step = STEP_MASTER_INIT;
					
					Send_Display_Data((UINT8_t)sizeof(disp_tbl1), (UINT8_t *)disp_tbl1);
				}
				else {
					Send_Display_Data((UINT8_t)sizeof(disp_tbl2), (UINT8_t *)disp_tbl2);
					Send_Display_Data((UINT8_t)sizeof(disp_tbl1), (UINT8_t *)disp_tbl1);
				}
				rx_length = (UINT8_t)0;
			}
			else {
				/* master */
				if(rx_length == 5) {
					buff = (unsigned char *)&cmpstring_w;				/* "write" */
				}
				else {
					/* Do nothing */
				}
				if(rx_length == 4) {
					buff = (unsigned char *)&cmpstring_r;				/* "read" */
				}
				else {
					/* Do nothing */
				}
				for(i=0 ; i < rx_length ; i++) {
					if(*(buff) != rx_data[i]) {
						check = 0;
					}
					else {
						/* Do nothing */
					}
					buff++;
				}
				if(check == 1) {
					if(rx_length == 5) {
						/* trans(to slave) start */
						mode = MODE_MASTER;
						trans_step = STEP_MASTER_START_CON;
						direction_flag = CLEAR;
						
						Send_Display_Data((UINT8_t)sizeof(disp_tbl1), (UINT8_t *)disp_tbl1);
					}
					else {
						trans_step = STEP_MASTER_START_CON_READ;
						direction_flag = SET;
					}
					SBI_Start_Condition();
				}
				else {
					Send_Display_Data((UINT8_t)sizeof(disp_tbl2), (UINT8_t *)disp_tbl2);
					Send_Display_Data((UINT8_t)sizeof(disp_tbl1), (UINT8_t *)disp_tbl1);
				}
				rx_length = (UINT8_t)0;
			}
		}
		else {
			if(trans_step == STEP_MASTER_READ_END) {
				Send_Display_Data((UINT8_t)sizeof(disp_tbl3), (UINT8_t *)disp_tbl3);
				/* read data send */
				tx_length = (UINT8_t)3;
				
				a = (UINT8_t)(recieve_buff_sbi_master / 16);
				if(a >= 10) {
					a -= 10;
					a += 65;
				}
				else {
					a += 48;
				}
				tx_data[0] = a;
				a = (UINT8_t)(recieve_buff_sbi_master % 16);
				if(a >= 10) {
					a -= 10;
					a += 65;
				}
				else {
					a += 48;
				}
				tx_data[1] = a;
				tx_data[2] = (CHAR_t)'\n';
				
				TX_Start();
				TD2BUF = tx_data[write];
				
				for(;;){
				}
			}
			else {
				/* Do nothing */
			}
		}
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  INTSBI0
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntSBI0(void)
{
	UINT8_t sbi_state;
	UINT8_t dummy_buff;
	
	if(mode == MODE_SLAVE) {
		/* slave */
		sbi_state = SBI0SR2;
		if((SBI0SR2_TRX == 1) && (SBI0SR2_AAS == 0)) {
			/* 1word trans end */
			if(SBI0SR2_LRB == 0) {					/* last recieve bit : 1 */
				SBI0DBR = 0x00;
			}
			else {
				SBI0CR2_PIN = SET;
				SBI0CR2_TRX = CLEAR;
			}
		}
		else {
			/* Do nothing */
		}
		if((SBI0SR2_TRX == 1) && (SBI0SR2_AAS == 1)) {
			/* trans data by read request */
			SBI0CR1 &= (SBI_DATA_BIT | 0x1F);		/* 8bit */
			SBI0DBR = recieve_buff_sbi_slave;
		}
		else {
			/* Do nothing */
		}
		if((SBI0SR2_TRX == 0) && (SBI0SR2_AAS == 1)) {
			/* recieve slave addr */
			SBI0DBR = 0x00;
		}
		else {
			/* Do nothing */
		}
		if((SBI0SR2_TRX == 0) && (SBI0SR2_AAS == 0)) {
			/* 1word recieve end */
			recieve_buff_sbi_slave = SBI0DBR;
			SBI0CR1 &= (SBI_DATA_BIT | 0x1F);		/* 8bit */
			SBI0DBR = 0x00;
		}
		else {
			/* Do nothing */
		}
	}
	else {
		/* master */
		switch(trans_step) {
		/* write */
		case STEP_MASTER_START_CON:
			if(SBI0SR2_LRB == 0) {					/* recieve ack */
				trans_step = STEP_MASTER_DATA_WRITE;
				SBI0DBR = SBI_DATA;					/* set data */
			}
			else {
				/* not recieve ack & no action */
				trans_step = STEP_MASTER_IDLE;
			}
			break;
		case STEP_MASTER_DATA_WRITE:
			if(SBI0SR2_LRB == 0) {					/* recieve ack */
				SBI_Stop_Condition();
			}
			else {
				/* not recieve ack & no action */
				trans_step = STEP_MASTER_IDLE;
			}
			trans_step = STEP_MASTER_IDLE;
			break;
		case STEP_MASTER_START_CON_READ:
			if(SBI0SR2_LRB == 0) {					/* recieve ack */
				dummy_buff = SBI0DBR;				/* dummy read */
				SBI0DBR = 0x00;						/* dummy write */
				trans_step = STEP_MASTER_READ;
			}
			else {
				/* not recieve ack & no action */
				trans_step = STEP_MASTER_IDLE;
			}
			break;
		case STEP_MASTER_READ:
			recieve_buff_sbi_master = SBI0DBR;
			SBI_Stop_Condition();
			trans_step = STEP_MASTER_READ_END;
			break;
		default:
			/* Do nothing */
			break;
		}
	}
}
/**
 *  @}
 */ /* End of group SBI_Exported_functions */

/**
 *  @}
 */ /* End of group SBI */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
