/** 
 *******************************************************************************
 * @file    main.c
 * @brief   SIO_8bit_RECIEVE Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-07-19 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"
#include	"uart_io.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup SIO_8bit_RECIEVE SIO_8bit_RECIEVE Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SIO_8bit_RECIEVE_Private_macro SIO_8bit_RECIEVE Private Macro
 *  @{
 */
/** 
 *  @defgroup Serial_setting Macro Definisiton
 *  @brief    Serial setting
 *  @{
 */
#define SERIAL_SEL_UART_SIO0			((UINT8_t)0x11)				/*!< UART0 SIO Port select */
#define SERIAL_SEL_UART_SIO1			((UINT8_t)0x08)				/*!< SIO1 Port select */
/**
 *  @}
 */ /* End of group Serial_setting */
/** 
 *  @defgroup SIO_SIO1EN SIO1 Enable
 *  @brief    SIO1 Control Enable.
 *  @{
 */
#define SIO0_CONTORL_ENABLE				((UINT8_t)0x01)			/*!< SIO0 enable */
#define SIO1_CONTORL_ENABLE				((UINT8_t)0x02)			/*!< SIO1 enable */
/**
 *  @}
 */ /* End of group SIO_SIO1EN */
/** 
 *  @defgroup SIO_SIO1CR Macro Definition
 *  @brief    SIO1CR Setting.
 *  @{
 */
#define SIO_SEL_EDGE					((UINT8_t)0x00)		/*!< trans edge */
#define SIO_SCLK						((UINT8_t)0x70)		/*!< sclk(ext clock) */
#define SIO_TRANS_FORMAT				((UINT8_t)0x00)		/*!< trans format(LSB) */
#define SIO_BEHAVE_START				((UINT8_t)0x01)		/*!< trans behave start */
#define SIO_BEHAVE_STOP					((UINT8_t)0x00)		/*!< trans behave stop */
#define SIO_TRANS_MODE					((UINT8_t)0x02)		/*!< trans mode(8bit recieve) */
/**
 *  @}
 */ /* End of group SIO_SIO1CR Macro Definition */
/** 
 *  @defgroup  Key 
 *  @brief     Key Setting
 *  @{
 */
#define PORT_INPUT						((UINT8_t)0x00)			/*!< Port Mode Input */
#define PORT_FUNC_ON					((UINT8_t)0x01)			/*!< Port Function Change */
/** 
 *  @}
 */ /* End of group Key */

/** 
 *  @}
 */ /* End of group SIO_8bit_RECIEVE_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SIO_8bit_RECIEVE_Private_define SIO_8bit_RECIEVE Private Define
 *  @{
 */
const UINT8_t dsp_tbl1[] = {"SIO Sample Program\n"};
const UINT8_t dsp_tbl2[] = {"----------------------------------------------\n"};
const UINT8_t dsp_tbl3[] = {"Receive data:\n"};
/** 
 *  @}
 */ /* End of group SIO_8bit_RECIEVE_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SIO_8bit_RECIEVE_Private_variables SIO_8bit_RECIEVE Private Variables
 *  @{
 */
static UINT8_t recieve_buff = (UINT8_t)0;			/*!< SIO Recieve buffer.        */

static UINT8_t recieve_flag   = 0;					/*!< recieve end flag        */
/**
 *  @}
 */ /* End of group SIO_8bit_RECIEVE_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SIO_8bit_RECIEVE_Private_fuctions SIO_8bit_RECIEVE Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void SIO_RECIEVE_Config(void);
static void SIO_Start(void);
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Port Function Control : SCLK1 */
	P9FC2 = PORT_FUNC_ON;
	P9CR1 = PORT_INPUT;
	P9CR2 = PORT_INPUT;
}
/*--------------------------------------------------*/
/** 
  * @brief  SIO RECIEVE Config
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void SIO_RECIEVE_Config(void)
{
	SERSEL = SERIAL_SEL_UART_SIO1;
	/* (1) Enable SIO1 Control */
	/* (2) Enable interrupt of SIO1 */
	/* (3) Setting SIO1CR,trans edge,sclk,trans format,trans mode,trans behave stop*/
	POFFCR2 = SIO1_CONTORL_ENABLE;									/* (1) */
	__DI();
	EIR_INTSIO1 = 1;				     							/* (2) */
	__EI();
	SIO1CR = SIO_SEL_EDGE | SIO_SCLK | SIO_TRANS_FORMAT | SIO_TRANS_MODE | SIO_BEHAVE_STOP;
																	/* (3) */
}
/*--------------------------------------------------*/
/** 
  * @brief  SIO START
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void SIO_Start(void)
{
	/* SIOS = 1 */
	SIO1CR_SIOS = SIO_BEHAVE_START;
}

/**
 *  @}
 */ /* End of group SIO_8bit_RECIEVE_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SIO_8bit_RECIEVE_Exported_functions SIO_8bit_RECIEVE Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	UINT8_t a = (UINT8_t)0;
	
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	
	/*------------------------*/
	/* UART Setting            */
	/*------------------------*/
	uart_initialize();
	/* Rx Start */
	UART2CR1_RXE = UART_RXE_ENABLE;
	
	Send_Display_Data((UINT8_t)sizeof(dsp_tbl1), (UINT8_t *)dsp_tbl1);
	Send_Display_Data((UINT8_t)sizeof(dsp_tbl2), (UINT8_t *)dsp_tbl2);
	Send_Display_Data((UINT8_t)sizeof(dsp_tbl3), (UINT8_t *)dsp_tbl3);
	/*------------------------*/
	/* SIO Config             */
	/*------------------------*/
	SIO_RECIEVE_Config();
	
	SIO_Start();
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
		if(recieve_flag == SET) {
			recieve_flag = 0;
			
			write = 0;
			tx_length = (UINT8_t)5;
			
			tx_data[0] = (CHAR_t)'0';
			tx_data[1] = (CHAR_t)'x';
			a = (UINT8_t)(recieve_buff / 16);
			if(a >= 10) {
				a -= 10;
				a += 65;
			}
			else {
				a += 48;
			}
			tx_data[2] = a;
			a = (UINT8_t)(recieve_buff % 16);
			if(a >= 10) {
				a -= 10;
				a += 65;
			}
			else {
				a += 48;
			}
			tx_data[3] = a;
			tx_data[4] = (CHAR_t)'\n';
			
			TX_Start();
			TD2BUF = tx_data[write];
		}
		else {
			/* Do nothing */
		}
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  INTSIO1
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntSIO1(void)
{
	UINT8_t state = (UINT8_t)0;
	
	state = SIO1SR;
	if((state & 0x20) == 0) {
		/* no error && recive end && not trans && not shift */
		if(recieve_flag == 0) {
			recieve_flag = SET;
			recieve_buff = SIO1BUF;
		}
		else {
			/* Do nothing */
		}
	}
	else {
		/* Do nothing */
	}
}
/**
 *  @}
 */ /* End of group SIO_8bit_RECIEVE_Exported_functions */

/**
 *  @}
 */ /* End of group SIO_8bit_RECIEVE */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
