/** 
 *******************************************************************************
 * @file    main.c
 * @brief   SIO_8bit_TRANS Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-07-19 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"
#include	"uart_io.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup SIO_8bit_TRANS SIO_8bit_TRANS Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SIO_8bit_TRANS_Private_macro SIO_8bit_TRANS Private Macro
 *  @{
 */
/** 
 *  @defgroup Serial_setting Macro Definisiton
 *  @brief    Serial setting
 *  @{
 */

#define SERIAL_SEL_UART_SIO1			((UINT8_t)0x08)				/*!< SIO1 Port select */
/**
 *  @}
 */ /* End of group Serial_setting */
/** 
 *  @defgroup TBT_TBTCK Macro Definition
 *  @brief    TBT Interval.
 *  @{
 */
#define TBT_INTERVAL					((UINT8_t)0x03)			/*!< TBT Interrupt Clock(=2^12) */
/**
 *  @}
 */ /* End of group TBT_TBTCK Macro Definition */
/** 
 *  @defgroup TBT_TBTEN TBT Start
 *  @brief    TBT Start.
 *  @{
 */
#define TBT_TBTEN_START					((UINT8_t)0x01)			/*!< TBT Start(Innterrupt request signal enable) */
/**
 *  @}
 */ /* End of group TBT_TBTEN */
/** 
 *  @defgroup SIO_SIO1EN SIO1 Enable
 *  @brief    SIO1 Control Enable.
 *  @{
 */
#define SIO0_CONTORL_ENABLE				((UINT8_t)0x01)			/*!< SIO0 enable */
#define SIO1_CONTORL_ENABLE				((UINT8_t)0x02)			/*!< SIO1 enable */
/**
 *  @}
 */ /* End of group SIO_SIO1EN */
/** 
 *  @defgroup SIO_SIO1CR Macro Definition
 *  @brief    SIO0CR Setting.
 *  @{
 */
#define SIO_SEL_EDGE					((UINT8_t)0x00)		/*!< trans edge */
#define SIO_SCLK						((UINT8_t)0x00)		/*!< sclk(fcgck/2^9) */
#define SIO_TRANS_FORMAT				((UINT8_t)0x00)		/*!< trans format(LSB) */
#define SIO_BEHAVE_START				((UINT8_t)0x01)		/*!< trans behave start */
#define SIO_BEHAVE_STOP					((UINT8_t)0x00)		/*!< trans behave stop */
#define SIO_TRANS_MODE					((UINT8_t)0x01)		/*!< trans mode(8bit trans) */
/**
 *  @}
 */ /* End of group SIO_SIO1CR Macro Definition */
/** 
 *  @defgroup  Key 
 *  @brief     Key Setting
 *  @{
 */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
#define PORT_FUNC_ON					((UINT8_t)0x01)			/*!< Port Function Change */
/** 
 *  @}
 */ /* End of group Key */

/** 
 *  @}
 */ /* End of group SIO_8bit_TRANS_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SIO_8bit_TRANS_Private_define SIO_8bit_TRANS Private Define
 *  @{
 */
const UINT8_t dsp_tbl1[] = {"SIO Sample Program\n"};
const UINT8_t dsp_tbl2[] = {"----------------------------------------------\n"};
const UINT8_t dsp_tbl3[] = {"Command>  "};
const UINT8_t dsp_tbl4[] = {"error\n"};
/** 
 *  @}
 */ /* End of group SIO_8bit_TRANS_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SIO_8bit_TRANS_Private_variables SIO_8bit_TRANS Private Variables
 *  @{
 */

static UINT8_t trans_buff = (UINT8_t)0;				/*!< SIO Trans buffer.        */
static UINT8_t trans_flag   = 0;					/*!< trans flag        */
static const UINT8_t cmpstring[5] = "write";		/*!< write command      */
static UINT8_t timer_flag   = 0;					/*!< main timer flag        */
/**
 *  @}
 */ /* End of group SIO_8bit_TRANS_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SIO_8bit_TRANS_Private_fuctions SIO_8bit_TRANS Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void TBT_Config(void);
static void TBT_Start(void);
static void SIO_TRANS_Config(void);
static void SIO_Start(void);
static void SIO_Stop(void);
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	P9FC0 = PORT_FUNC_ON;
	P9FC2 = PORT_FUNC_ON;
	P9CR0 = PORT_OUTPUT;
	P9CR2 = PORT_OUTPUT;
}
/*--------------------------------------------------*/
/** 
  * @brief  TBT_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TBT_Config(void)
{
	/* (1) Enable interrupt of TBT */
	/* (2) Setting interrupt frequency,TBTCR[2:0]=(/2^) */
	__DI();
	EIR_INTTBT = SET;				/* (1) */
	__EI();
	TBTCR  = TBT_INTERVAL;			/* (2) */
}
/*--------------------------------------------------*/
/** 
  * @brief  TBT_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TBT_Start(void)
{
	/* TBTEN = 1 */
	TBTEN = TBT_TBTEN_START;
}
/*--------------------------------------------------*/
/** 
  * @brief  SIO TRANS Config
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void SIO_TRANS_Config(void)
{
	SERSEL = SERIAL_SEL_UART_SIO1;
	
	/* (1) Enable SIO1 Control */
	/* (2) Enable interrupt of SIO1 */
	/* (3) Setting SIO1CR,trans edge,sclk,trans format,trans mode,trans behave stop*/
	POFFCR2 = SIO1_CONTORL_ENABLE;									/* (1) */
	__DI();
	EIR_INTSIO1 = 1;												/* (2) */
	__EI();
	SIO1CR = SIO_SEL_EDGE | SIO_SCLK | SIO_TRANS_FORMAT | SIO_TRANS_MODE | SIO_BEHAVE_STOP;
}
/*--------------------------------------------------*/
/** 
  * @brief  SIO START
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void SIO_Start(void)
{
	/* SIOS = 1 */
	SIO1CR_SIOS = SIO_BEHAVE_START;
}
/*--------------------------------------------------*/
/** 
  * @brief  SIO STOP
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void SIO_Stop(void)
{
	/* SIOS = 0 */
	SIO1CR_SIOS = SIO_BEHAVE_STOP;
}
/**
 *  @}
 */ /* End of group SIO_8bit_TRANS_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SIO_8bit_TRANS_Exported_functions SIO_8bit_TRANS Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	UINT8_t i;
	UINT8_t check = (UINT8_t)1;
	UINT8_t *buff;
	
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	/*------------------------*/
	/* TBT Setting   */
	/*------------------------*/ 
	TBT_Config();
	/*------------------------*/
	/* UART Setting            */
	/*------------------------*/
	uart_initialize();
	/* Rx Start */
	UART2CR1_RXE = UART_RXE_ENABLE;
	
	Send_Display_Data((UINT8_t)sizeof(dsp_tbl1), (UINT8_t *)dsp_tbl1);
	Send_Display_Data((UINT8_t)sizeof(dsp_tbl2), (UINT8_t *)dsp_tbl2);
	Send_Display_Data((UINT8_t)sizeof(dsp_tbl3), (UINT8_t *)dsp_tbl3);
	/*------------------------*/
	/* SIO Config             */
	/*------------------------*/
	SIO_TRANS_Config();
	
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
		if(trans_flag == SET) {
			if(timer_flag == SET) {
				timer_flag = CLEAR;
				SIO1BUF = trans_buff;
				SIO_Start();
			}
			else {
				/* Do nothing */
			}
		}
		else if(cmp_flag == 1) {
			check = (UINT8_t)1;
			/* UART recieve */
			cmp_flag = 0;
			buff = (UINT8_t *)&cmpstring;							/* "write" */
			for(i=0 ; i < rx_length ; i++) {
				if(*(buff) != rx_data[i]) {
					check = 0;
				}
				else {
					/* Do nothing */
				}
				buff++;
			}
			if(check == 1) {
				/* timer & trans start */
				TBT_Start();
				trans_flag = SET;
				trans_buff = (UINT8_t)0;
				SIO1BUF = trans_buff;
				SIO_Start();
			}
			else {
				/* input command error */
				Send_Display_Data((UINT8_t)sizeof(dsp_tbl4), (UINT8_t *)dsp_tbl4);
				Send_Display_Data((UINT8_t)sizeof(dsp_tbl3), (UINT8_t *)dsp_tbl3);
			}
			rx_length = (UINT8_t)0;
		}
		else {
			/* no action */
		}
	}
}
/*--------------------------------------------------*/
/** 
  * @brief  INTTBT
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntTBT(void)
{
	static UINT8_t i = (UINT8_t)0;
	
	i++;
	if(i >= 24) {
		i = (UINT8_t)0;
		timer_flag = SET;
		if(trans_buff < 0xFF) {
			/* count up to 0xFF */
			trans_buff++;
		}
		else {
			trans_buff = (UINT8_t)0;
		}
	}
	else {
		/* Do nothing */
	}
}
/*--------------------------------------------------*/
/** 
  * @brief  INTSIO1
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntSIO1(void)
{
	UINT8_t state = (UINT8_t)0;
	
	state = SIO1SR;
	if((state & 0x04) == 0) {
		/* trans end */
		SIO_Stop();
	}
	else {
		/* Do nothing */
	}
}
/**
 *  @}
 */ /* End of group SIO_8bit_TRANS_Exported_functions */

/**
 *  @}
 */ /* End of group SIO_8bit_TRANS */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
