/** 
 *******************************************************************************
 * @file    main.c
 * @brief   TBT Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-08-28 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup TBT TBT Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TBT_Private_macro TBT Private Macro
 *  @{
 */

/** 
 *  @defgroup TBT_TBTCK Macro Definition
 *  @brief    TBT Interval.
 *  @{
 */
#define TBT_INTERVAL					((UINT8_t)0x04)			/*!< TBT Interrupt Clock(fcgck/2^12) */
/**
 *  @}
 */ /* End of group TBT_TBTCK Macro Definition */
/** 
 *  @defgroup TBT_TBTEN TBT Start
 *  @brief    TBT Start.
 *  @{
 */
#define TBT_TBTEN_START					((UINT8_t)0x01)			/*!< TBT Start(Innterrupt request signal enable) */
/**
 *  @}
 */ /* End of group TBT_TBTEN */
/** 
 *  @defgroup  Port Port Setting
 *  @brief     Port Setting
 *  @{
 */
#define PORT_OFF						((UINT8_t)0x00)			/*!< Port OFF */
#define PORT_ON							((UINT8_t)0x01)			/*!< Port ON */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
/** 
 *  @}
 */ /* End of group Port */

/** 
 *  @}
 */ /* End of group TBT_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TBT_Private_variables TBT Private Variables
 *  @{
 */
static UINT8_t blink_flag = 0;					/*!<  blink flag.        */
/**
 *  @}
 */ /* End of group TBT_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TBT_Private_fuctions TBT Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void TBT_Config(void);
static void TBT_Start(void);
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Select output mode on PORT 25 (LED0) */
	P2CR5 = PORT_OUTPUT;
	/* (2) Turn off PORT 25 (LED0) */
	P25 = PORT_OFF;
}

/*--------------------------------------------------*/
/** 
  * @brief  TBT_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TBT_Config(void)
{
	/* (1) Enable interrupt of TBT */
	/* (2) Setting interrupt frequency,TBTCR[2:0]=(/2^) */
	/* (3) Enable interrupt request signal,TBTCR[3]=1 */
	__DI();
	EIR_INTTBT = SET;				/* (1) */
	__EI();
	TBTCR  = TBT_INTERVAL;			/* (2) */
	TBTEN  = TBT_TBTEN_START;		/* (3) */
}
/*--------------------------------------------------*/
/** 
  * @brief  TBT_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TBT_Start(void)
{
	/* TBTEN = 1 */
	TBTEN = TBT_TBTEN_START;
}
/**
 *  @}
 */ /* End of group TBT_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TBT_Exported_functions TBT Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	
	/*------------------------*/
	/* TBT Setting   */
	/*------------------------*/ 
	TBT_Config();
	/* Timer Start */
	TBT_Start();
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  INTTBT
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntTBT(void)
{
	if(blink_flag == SET) {
		blink_flag = CLEAR;
		P25 = PORT_ON;
	}
	else {
		blink_flag = SET;
		P25 = PORT_OFF;
	}
}
/**
 *  @}
 */ /* End of group TBT_Exported_functions */

/**
 *  @}
 */ /* End of group TBT */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
