/** 
 *******************************************************************************
 * @file    main.c
 * @brief   TC0_12bit_PWM Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-07-19 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup TC0_12bit_PWM TC0_12bit_PWM Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_12bit_PWM_Private_macro TC0_12bit_PWM Private Macro
 *  @{
 */
/** 
 *  @defgroup TC0_TC001EN TC00/01 Enable
 *  @brief    TC00/01 Control Enable.
 *  @{
 */
#define TC0_TC00_01_ENABLE				((UINT8_t)0x10)			/*!< TC00/01 enable */
/**
 *  @}
 */ /* End of group TC0_TC001EN */
/** 
 *  @defgroup TC0_T00/01PWM Macro Definition
 *  @brief    TC0 PWM Duty.
 *  @{
 */
#define TC0_12BIT_PWM_DUTY_MIN			((UINT16_t)0x0010)		/*!< TC00/01PWM Min */
#define TC0_12BIT_PWM_DUTY_MAX			((UINT16_t)0x0FF0)		/*!< TC00/01PWM Max */
/**
 *  @}
 */ /* End of group TC0_T00REG Macro Definition */
 /** 
 *  @defgroup TC0_TCK0 Macro Definition
 *  @brief    TC0 Clock.
 *  @{
 */
#define TC0_12bit_PWM_CLK				((UINT8_t)0x30)			/*!< TCK0(110=fcgck/2) */
/**
 *  @}
 */ /* End of group TC0_TCK0 Macro Definition */
/** 
 *  @defgroup TC0_TCM0 TC0 Mode Select
 *  @brief    TC0 Start.
 *  @{
 */
#define TC0_PULSE_OUTPUT				((UINT8_t)0x08)			/*!< TC0 timer cascade set */
#define TC0_CASCADE						((UINT8_t)0x04)			/*!< TC0 timer cascade set */
/**
 *  @}
 */ /* End of group TC0_TCM0 */
/** 
 *  @defgroup TC0_T01MOD Macro Definition
 *  @brief    T01MOD Setting.
 *  @{
 */
#define TC0_T01_MODE					((UINT8_t)0x02)			/*!< TC0 12bit PWM mode */
/**
 *  @}
 */ /* End of group TC0_T01MOD Macro Definition */
/** 
 *  @defgroup TC0_T00RUN TC0 Start
 *  @brief    TC0 Start.
 *  @{
 */
#define TC0_START						((UINT8_t)0x01)			/*!< TC0 Start */
#define TC0_STOP						((UINT8_t)0x00)			/*!< TC0 Stop */
/**
 *  @}
 */ /* End of group TC0_T00RUN */
/** 
 *  @defgroup  Port Port Setting
 *  @brief     Port Setting
 *  @{
 */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
#define PORT_INPUT						((UINT8_t)0x00)			/*!< Port Mode Input */
#define PORT_PWM01						((UINT8_t)0x01)			/*!< Port Function TC00 */
/** 
 *  @}
 */ /* End of group Port */

/** 
 *  @}
 */ /* End of group TC0_12bit_PWM_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_12bit_PWM_Private_define TC0_12bit_PWM Private Define
 *  @{
 */
#define KEYIN_CNT						((UINT8_t)100)			/*!< keychat=2 */
/** 
 *  @}
 */ /* End of group TC0_12bit_PWM_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_12bit_PWM_Private_variables TC0_12bit_PWM Private Variables
 *  @{
 */
UINT16_t pwm_duty_buf;								/*!< PWM duty buffer           */
static UINT8_t keybuf = (UINT8_t)0;					/*!<  key buffer          */
static UINT8_t keycode = (UINT8_t)0;				/*!<  key confirm         */
static UINT8_t keychat = KEYIN_CNT;					/*!<  key now             */
static UINT8_t key_already_flag = (UINT8_t)0;		/*!<  flag.               */
static UINT8_t comfirm_flag = (UINT8_t)0;			/*!<  flag.               */
/**
 *  @}
 */ /* End of group TC0_12bit_PWM_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_12bit_PWM_Private_fuctions TC0_12bit_PWM Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void Key(void);
static void TC0_12bit_PWM_Config(void);
static void TC0_12bit_PWM_Start(void);
static void TC0_12bit_PWM_Stop(void);
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Port Function Control : PWM01 */
	P7FC1 = PORT_PWM01;
	/* (2) Select input mode on PORT 71 */
	P7CR1 = PORT_OUTPUT;
	/* (3) Select input mode on PORT 12 (SW0) */
	P1CR2 = PORT_INPUT;
}
/*--------------------------------------------------*/
/** 
  * @brief  Key
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Key(void)
{
	UINT8_t keyTemp;

	keyTemp = (UINT8_t)(P1PRD & 0x04);
	keyTemp ^= 0xFF;
	
	if(keybuf == keyTemp) {
		if(--keychat == 0) {
			keychat = KEYIN_CNT;
			if((keycode != 0x00) && (key_already_flag == 0)) {
				key_already_flag = SET;
				comfirm_flag = SET;
			}
			else {
				/* Do nothing */
			}
			keycode = keybuf;
		}
		else {
			/* Do nothing */
		}
	}
	else {
		keychat = KEYIN_CNT;
		keybuf = keyTemp;
		key_already_flag = CLEAR;
		comfirm_flag = CLEAR;
	}
}
/*--------------------------------------------------*/
/** 
  * @brief  TC0_12bit_PWM_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TC0_12bit_PWM_Config(void)
{
	/* (1) Enable TC00/TC01 Control */
	/* (2) Set Cascade */
	/* (3) Setting clock T01MOD[5:3]=110(fcgck/2),mode T01MOD[1:0]=10(12bit PWM mode) */
	/* (4) Setting T02/03REG */
	POFFCR0 = TC0_TC00_01_ENABLE;									/* (1) */
	T001CR = TC0_PULSE_OUTPUT | TC0_CASCADE;						/* (2) */

	T01MOD = TC0_12bit_PWM_CLK | TC0_T01_MODE;
																	/* (3) */
	T00PWM = (UINT8_t)(TC0_12BIT_PWM_DUTY_MIN & 0x00F0);			/* (4) */
	T01PWM = (UINT8_t)((TC0_12BIT_PWM_DUTY_MIN & 0x0F00) >> 8);
	pwm_duty_buf = TC0_12BIT_PWM_DUTY_MIN;
}
/*--------------------------------------------------*/
/** 
  * @brief  TC0_12bit_PWM_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TC0_12bit_PWM_Start(void)
{
	/* T01RUN = 1 */
	T01RUN = TC0_START;
}

/*--------------------------------------------------*/
/** 
  * @brief  TC0_12bit_PWM_Stop
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TC0_12bit_PWM_Stop(void)
{
	/* T01RUN = 0 */
	T01RUN = TC0_STOP;
}

/**
 *  @}
 */ /* End of group TC0_12bit_PWM_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_12bit_PWM_Exported_functions TC0_12bit_PWM Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	
	/*------------------------*/
	/* TC0_12bit_PWM Setting   */
	/*------------------------*/ 
	TC0_12bit_PWM_Config();
	/* Timer Start */
	TC0_12bit_PWM_Start();
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
		Key();
		if(comfirm_flag == 1) {
			comfirm_flag = CLEAR;
			if((keycode & 0x04) != 0) {
				/* PWM stop */
				TC0_12bit_PWM_Stop();
				
				/* PWM Duty 16 increase */
				if(pwm_duty_buf >= TC0_12BIT_PWM_DUTY_MAX) {
					pwm_duty_buf = TC0_12BIT_PWM_DUTY_MIN;
				}
				else {
					pwm_duty_buf += (UINT16_t)16;
				}
				T00PWM = (UINT8_t)(pwm_duty_buf & 0x00F0);
				T01PWM = (UINT8_t)((pwm_duty_buf & 0x0F00) >> 8);
				TC0_12bit_PWM_Start();
			}
			else {
				/* Do nothing */
			}
		}
		else {
			/* Do nothing */
		}
	}
}

/**
 *  @}
 */ /* End of group TC0_12bit_PWM_Exported_functions */

/**
 *  @}
 */ /* End of group TC0_12bit_PWM */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
