/** 
 *******************************************************************************
 * @file    main.c
 * @brief   TC0_16bit_EVENT_COUNT Sample Application.
 * @version V1.0.3.0
 * $Date:: 2019-09-13 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"


/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup TC0_16bit_EVENT_COUNT TC0_16bit_EVENT_COUNT Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_16bit_EVENT_COUNT_Private_macro TC0_16bit_EVENT_COUNT Private Macro
 *  @{
 */

/** 
 *  @defgroup TC0_TC001EN TC00/01 Enable
 *  @brief    TC00/01 Control Enable.
 *  @{
 */
#define TC0_TC00_01_ENABLE				((UINT8_t)0x10)			/*!< TC00/01 enable */
/**
 *  @}
 */ /* End of group TC0_TC001EN */
/** 
 *  @defgroup TC0_T00REG Macro Definition
 *  @brief    TC0 Interval.
 *  @{
 */
#define TC0_16bit_EVENT_COUNT_INTERVAL	((UINT16_t)0x0021)		/*!< TC00REG */
/**
 *  @}
 */ /* End of group TC0_T00REG Macro Definition */
/** 
 *  @defgroup TC0_EIN0 TC0 Clock Select
 *  @brief    TC0 Sorce Clock Select.
 *  @{
 */
#define TC0_T00_EIN						((UINT8_t)0x04)			/*!< TC0 8bit source clock(Ext)  */
/**
 *  @}
 */ /* End of group TC0_EIN0 */
/** 
 *  @defgroup TC0_TCM0 TC0 Mode Select
 *  @brief    TC0 Start.
 *  @{
 */
#define TC0_CASCADE						((UINT8_t)0x04)			/*!< TC0 timer cascade set */
/**
 *  @}
 */ /* End of group TC0_TCM0 */
/** 
 *  @defgroup TC0_T01MOD Macro Definition
 *  @brief    T01MOD Setting.
 *  @{
 */
#define TC0_T01_MODE					((UINT8_t)0x00)			/*!< TC0 16bit event count mode */
/**
 *  @}
 */ /* End of group TC0_T01MOD Macro Definition */
/** 
 *  @defgroup TC0_T00RUN TC0 Start
 *  @brief    TC0 Start.
 *  @{
 */
#define TC0_START						((UINT8_t)0x01)			/*!< TC0 Start */
/**
 *  @}
 */ /* End of group TC0_T00RUN */
/** 
 *  @defgroup  Port Port Setting
 *  @brief     Port Setting
 *  @{
 */
#define PORT_OFF						((UINT8_t)0x00)			/*!< Port OFF */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
#define PORT_INPUT						((UINT8_t)0x00)			/*!< Port Mode Input */
#define PORT_TC00						((UINT8_t)0x01)			/*!< Port Function TC00 */
/** 
 *  @}
 */ /* End of group Port */

/** 
 *  @}
 */ /* End of group TC0_16bit_EVENT_COUNT_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup WDT_Private_variables WDT Private Variables
 *  @{
 */
static UINT16_t blink_cnt = 0;									/*!<  blink flag.        */
/**
 *  @}
 */ /* End of group WDT_Private_variables */
 
/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_16bit_EVENT_COUNT_Private_fuctions TC0_16bit_EVENT_COUNT Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void TC0_16bit_EVENT_COUNT_Config(void);
static void TC0_16bit_EVENT_COUNT_Start(void);
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Select output mode on PORT 26 (LED1) */
	P2CR6 = PORT_OUTPUT;
	/* (2) Turn off PORT 26 (LED1) */
	P26 = PORT_OFF;
	/* (3) Select input mode on PORT 70 */
	P7CR0 = PORT_INPUT;
}

/*--------------------------------------------------*/
/** 
  * @brief  TC0_16bit_EVENT_COUNT_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TC0_16bit_EVENT_COUNT_Config(void)
{
	/* (1) Enable TC00/TC01 Control */
	/* (2) Set Cascade */
	/* (3) Enable interrupt of TC01 */
	/* (4) Setting mode T00MOD[1:0]=00(16bit tomer mode) */
	/* (5) Setting T02/03REG */
	POFFCR0 = TC0_TC00_01_ENABLE;									/* (1) */
	T001CR = TC0_CASCADE;											/* (2) */
	__DI();
	EIR_INTTC01 = 1;												/* (3) */
	__EI();
	T01MOD = TC0_T00_EIN | TC0_T01_MODE;							/* (4) */
	T00REG = (UINT8_t)(TC0_16bit_EVENT_COUNT_INTERVAL & 0xFF);		/* (5) */
	T01REG = (UINT8_t)((TC0_16bit_EVENT_COUNT_INTERVAL >> 8) & (0xFF));
}
/*--------------------------------------------------*/
/** 
  * @brief  TC0_16bit_EVENT_COUNT_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TC0_16bit_EVENT_COUNT_Start(void)
{
	/* T01RUN = 1 */
	T01RUN = TC0_START;
}
    
/**
 *  @}
 */ /* End of group TC0_16bit_EVENT_COUNT_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_16bit_EVENT_COUNT_Exported_functions TC0_16bit_EVENT_COUNT Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	
	/*------------------------*/
	/* TC0_16bit_EVENT_COUNT Setting   */
	/*------------------------*/ 
	TC0_16bit_EVENT_COUNT_Config();
	/* Timer Start */
	TC0_16bit_EVENT_COUNT_Start();
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  INTTC01
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntTC01(void)
{
	static UINT8_t i = 0;
	
	if(blink_cnt >= 500) {
		blink_cnt = CLEAR;
		
		i = P26;
		i ^= 1;
		P26 = i;
	}
	else {
		blink_cnt++;
	}
}
/**
 *  @}
 */ /* End of group TC0_16bit_EVENT_COUNT_Exported_functions */

/**
 *  @}
 */ /* End of group TC0_16bit_EVENT_COUNT */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
