/** 
 *******************************************************************************
 * @file    main.c
 * @brief   TC0_16bit_TIMER Sample Application.
 * @version V1.0.1.0
 * $Date:: 2019-06-10 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup TC0_16bit_TIMER TC0_16bit_TIMER Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_16bit_TIMER_Private_macro TC0_16bit_TIMER Private Macro
 *  @{
 */

/** 
 *  @defgroup TC0_TC001EN TC02/03 Enable
 *  @brief    TC00/01 Control Enable.
 *  @{
 */
#define TC0_TC02_03_ENABLE				((UINT8_t)0x20)			/*!< TC02/03 enable */
/**
 *  @}
 */ /* End of group TC0_TC001EN */
/** 
 *  @defgroup TC0_T00REG Macro Definition
 *  @brief    TC0 Interval.
 *  @{
 */
#define TC0_16BIT_TIMER_INTERVAL		((UINT16_t)0x01E0)		/*!< TC00REG */
/**
 *  @}
 */ /* End of group TC0_T00REG Macro Definition */
 /** 
 *  @defgroup TC0_TCK0 Macro Definition
 *  @brief    TC0 Clock.
 *  @{
 */
#define TC0_16BIT_TIMER_CLK				((UINT8_t)0x30)			/*!< TCK0(110=fcgck/2) */
/**
 *  @}
 */ /* End of group TC0_TCK0 Macro Definition */
/** 
 *  @defgroup TC0_TCM0 TC0 Mode Select
 *  @brief    TC0 Start.
 *  @{
 */
#define TC0_CASCADE						((UINT8_t)0x04)			/*!< TC0 timer cascade set */
/**
 *  @}
 */ /* End of group TC0_TCM0 */
/** 
 *  @defgroup TC0_T01MOD Macro Definition
 *  @brief    T01MOD Setting.
 *  @{
 */
#define TC0_T03_MODE					((UINT8_t)0x00)			/*!< TC0 16bit timer mode */
/**
 *  @}
 */ /* End of group TC0_T01MOD Macro Definition */
/** 
 *  @defgroup TC0_T00RUN TC0 Start
 *  @brief    TC0 Start.
 *  @{
 */
#define TC0_START						((UINT8_t)0x01)			/*!< TC0 Start */
/**
 *  @}
 */ /* End of group TC0_T00RUN */
 /** 
 *  @defgroup  Port_Output Port Output ON/OFF
 *  @brief     Port Output ON/OFF
 *  @{
 */
#define PORT_OFF						((UINT8_t)0x00)			/*!< Port OFF */
/** 
 *  @}
 */ /* End of group Port_Output */
 /** 
 *  @defgroup  Port_Mode Port Mode
 *  @brief     Port Mode Input/Output
 *  @{
 */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
/** 
 *  @}
 */ /* End of group Port_Output */

/** 
 *  @}
 */ /* End of group TC0_16bit_TIMER_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_16bit_TIMER_Private_fuctions TC0_16bit_TIMER Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void TC0_16bit_TIMER_Config(void);
static void TC0_16bit_TIMER_Start(void);
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Select output mode on PORT 25 (LED0) */
	P2CR5 = PORT_OUTPUT;
	/* (2) Turn off PORT 25 (LED0) */
	P25 = PORT_OFF;
}

/*--------------------------------------------------*/
/** 
  * @brief  TC0_16bit_TIMER_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TC0_16bit_TIMER_Config(void)
{
	/* (1) Enable TC02/TC03 Control */
	/* (2) Set Cascade */
	/* (3) Enable interrupt of TC03 */
	/* (4) Setting clock T00MOD[5:3]=110(fcgck/2),mode T00MOD[1:0]=00(16bit tomer mode) */
	/* (5) Setting T02/03REG */
	POFFCR0 = TC0_TC02_03_ENABLE;									/* (1) */
	T023CR = TC0_CASCADE;											/* (2) */
	__DI();
	EIR_INTTC03 = 1;												/* (3) */
	__EI();
	T03MOD = TC0_16BIT_TIMER_CLK | TC0_T03_MODE;					/* (4) */
	T02REG = (UINT8_t)(TC0_16BIT_TIMER_INTERVAL & 0xFF);			/* (5) */
	T03REG = (UINT8_t)((TC0_16BIT_TIMER_INTERVAL >> 8) & (0xFF));
}
/*--------------------------------------------------*/
/** 
  * @brief  TC0_16bit_TIMER_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TC0_16bit_TIMER_Start(void)
{
	/* T03RUN = 1 */
	T03RUN = TC0_START;
}

/**
 *  @}
 */ /* End of group TC0_16bit_TIMER_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_16bit_TIMER_Exported_functions TC0_16bit_TIMER Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	
	/*------------------------*/
	/* TC0_16bit_TIMER Setting   */
	/*------------------------*/ 
	TC0_16bit_TIMER_Config();
	/* Timer Start */
	TC0_16bit_TIMER_Start();
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  INTTC03
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntTC03(void)
{
	static UINT8_t i = 0;
	
	i = P25;
	i ^= 1;
	P25 = i;
}
/**
 *  @}
 */ /* End of group TC0_16bit_TIMER_Exported_functions */

/**
 *  @}
 */ /* End of group TC0_16bit_TIMER */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
