/** 
 *******************************************************************************
 * @file    main.c
 * @brief   TC0_8bit_PPG Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-07-19 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup TC0_8bit_PPG TC0_8bit_PPG Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_8bit_PPG_Private_macro TC0_8bit_PPG Private Macro
 *  @{
 */
/** 
 *  @defgroup TC0_TC001EN TC02/03 Enable
 *  @brief    TC02/03 Control Enable.
 *  @{
 */
#define TC0_TC02_03_ENABLE				((UINT8_t)0x20)			/*!< TC02/03 enable */
/**
 *  @}
 */ /* End of group TC0_TC001EN */
/** 
 *  @defgroup TC0_T03PPG Macro Definition
 *  @brief    TC0 PPG.
 *  @{
 */
#define TC0_8BIT_PPG_DUTY25_2441Hz		((UINT8_t)0x08)			/*!< T03PWM,2441Hz,Duty25% */
#define TC0_8BIT_PPG_DUTY50_2441Hz		((UINT8_t)0x10)			/*!< T03PWM,2441Hz,Duty50% */
#define TC0_8BIT_PPG_DUTY75_2441Hz		((UINT8_t)0x18)			/*!< T03PWM,2441Hz,Duty75% */

#define TC0_8BIT_PPG_DUTY25_1221Hz		((UINT8_t)0x10)			/*!< T03PWM,1221Hz,Duty25% */
#define TC0_8BIT_PPG_DUTY50_1221Hz		((UINT8_t)0x20)			/*!< T03PWM,1221Hz,Duty50% */
#define TC0_8BIT_PPG_DUTY75_1221Hz		((UINT8_t)0x30)			/*!< T03PWM,1221Hz,Duty75% */

#define TC0_8BIT_PPG_DUTY25_610Hz		((UINT8_t)0x20)			/*!< T03PWM,610Hz,Duty25% */
#define TC0_8BIT_PPG_DUTY50_610Hz		((UINT8_t)0x40)			/*!< T03PWM,610Hz,Duty50% */
#define TC0_8BIT_PPG_DUTY75_610Hz		((UINT8_t)0x60)			/*!< T03PWM,610Hz,Duty75% */

#define TC0_8BIT_PPG_INTERVAL_2441Hz	((UINT8_t)0x20)			/*!< T03REG,2441Hz */
#define TC0_8BIT_PPG_INTERVAL_1221Hz	((UINT8_t)0x40)			/*!< T03REG,1221Hz */
#define TC0_8BIT_PPG_INTERVAL_610Hz		((UINT8_t)0x80)			/*!< T03REG,610Hz */
/**
 *  @}
 */ /* End of group TC0_T03PPG Macro Definition */
 /** 
 *  @defgroup TC0_T03MOD Macro Definition
 *  @brief    T03MOD.
 *  @{
 */
#define TC0_T03_TTF						((UINT8_t)0x80)			/*!< TC0 8bit PPG TTF(ON=Hi start) */
#define TC0_8bit_PPG_CLK				((UINT8_t)0x18)			/*!< TCK0(011=fcgck/2^6) */
#define TC0_T03_MODE					((UINT8_t)0x03)			/*!< TC0 8bit PPG mode */
/**
 *  @}
 */ /* End of group TC0_T03MOD */
/** 
 *  @defgroup TC0_T00RUN TC0 Start
 *  @brief    TC0 Start.
 *  @{
 */
#define TC0_START						((UINT8_t)0x01)			/*!< TC0 Start */
#define TC0_STOP						((UINT8_t)0x00)			/*!< TC0 Stop */
/**
 *  @}
 */ /* End of group TC0_T00RUN */
/** 
 *  @defgroup  Port Port Setting
 *  @brief     Port Setting
 *  @{
 */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
#define PORT_INPUT						((UINT8_t)0x00)			/*!< Port Mode Input */
#define PORT_PPG03						((UINT8_t)0x01)			/*!< Port Function PPG03 */
/** 
 *  @}
 */ /* End of group Port */

/** 
 *  @}
 */ /* End of group TC0_8bit_PPG_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_8bit_PPG_Private_define TC0_8bit_PPG Private Define
 *  @{
 */
#define KEYIN_CNT						((UINT8_t)100)			/*!< keychat=2 */
/** 
 *  @}
 */ /* End of group TC0_8bit_PPG_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_8bit_PPG_Private_variables TC0_8bit_PPG Private Variables
 *  @{
 */
static UINT8_t ppg_step = 0;						/*!< PWM step.          */
static UINT8_t keybuf = (UINT8_t)0;					/*!<  key buffer          */
static UINT8_t keycode = (UINT8_t)0;				/*!<  key confirm         */
static UINT8_t keychat = KEYIN_CNT;					/*!<  key now             */
static UINT8_t key_already_flag = (UINT8_t)0;		/*!<  flag.               */
static UINT8_t comfirm_flag = (UINT8_t)0;			/*!<  flag.               */
/**
 *  @}
 */ /* End of group TC0_8bit_PPG_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_8bit_PPG_Private_fuctions TC0_8bit_PPG Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void Key(void);
static void TC0_8bit_PPG_Config(void);
static void TC0_8bit_PPG_Start(void);
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Port Function Control : PPG03 */
	P8FC1 = PORT_PPG03;
	/* (2) Select output mode on PORT 81 */
	P8CR1 = PORT_OUTPUT;
	/* (3) Select input mode on PORT 12 (SW0) */
	P1CR2 = PORT_INPUT;
}
/*--------------------------------------------------*/
/** 
  * @brief  Key
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Key(void)
{
	UINT8_t keyTemp;

	keyTemp = (UINT8_t)(P1PRD & 0x04);
	keyTemp ^= 0xFF;
	
	if(keybuf == keyTemp) {
		if(--keychat == 0) {
			keychat = KEYIN_CNT;
			if((keycode != 0x00) && (key_already_flag == 0)) {
				key_already_flag = SET;
				comfirm_flag = SET;
			}
			else {
				/* Do nothing */
			}
			keycode = keybuf;
		}
		else {
			/* Do nothing */
		}
	}
	else {
		keychat = KEYIN_CNT;
		keybuf = keyTemp;
		key_already_flag = CLEAR;
		comfirm_flag = CLEAR;
	}
}
/*--------------------------------------------------*/
/** 
  * @brief  TC0_8bit_PPG_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TC0_8bit_PPG_Config(void)
{
	/* (1) Enable TC02/TC03 Control */
	/* (2) Setting TTF=1(ON),clock T00MOD[5:3]=011(fcgck/2^6),mode T00MOD[1:0]=11(8bit PPG mode) */
	/* (3) Setting T03PPG */
	POFFCR0 = TC0_TC02_03_ENABLE;					/* (1) */

	T03MOD = TC0_T03_TTF | TC0_8bit_PPG_CLK | TC0_T03_MODE;
													/* (2) */
	T03REG = TC0_8BIT_PPG_INTERVAL_2441Hz;
	T03PWM = TC0_8BIT_PPG_DUTY25_2441Hz;			/* (3) */
}
/*--------------------------------------------------*/
/** 
  * @brief  TC0_8bit_PPG_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TC0_8bit_PPG_Start(void)
{
	/* T03RUN = 1 */
	T03RUN = TC0_START;
}

/**
 *  @}
 */ /* End of group TC0_8bit_PPG_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_8bit_PPG_Exported_functions TC0_8bit_PPG Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	
	/*------------------------*/
	/* TC0_8bit_PPG Setting   */
	/*------------------------*/ 
	TC0_8bit_PPG_Config();
	/* Timer Start */
	TC0_8bit_PPG_Start();
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
		Key();
		if(comfirm_flag == 1) {
			comfirm_flag = CLEAR;
			if((keycode & 0x04) != 0) {
				if(ppg_step < 8) {
					ppg_step++;
				}
				else {
					ppg_step = 0;
				}
				
				switch(ppg_step) {
				case 0:					/* 2441HZ,25% duty */
					T03REG = TC0_8BIT_PPG_INTERVAL_2441Hz;
					T03PWM = TC0_8BIT_PPG_DUTY25_2441Hz;
					break;
				case 1:					/* 1221HZ,25% duty */
					T03REG = TC0_8BIT_PPG_INTERVAL_1221Hz;
					T03PWM = TC0_8BIT_PPG_DUTY25_1221Hz;
					break;
				case 2:					/* 610HZ,25% duty */
					T03REG = TC0_8BIT_PPG_INTERVAL_610Hz;
					T03PWM = TC0_8BIT_PPG_DUTY25_610Hz;
					break;
				case 3:					/* 2441HZ,50% duty */
					T03REG = TC0_8BIT_PPG_INTERVAL_2441Hz;
					T03PWM = TC0_8BIT_PPG_DUTY50_2441Hz;
					break;
				case 4:					/* 1221HZ,50% duty */
					T03REG = TC0_8BIT_PPG_INTERVAL_1221Hz;
					T03PWM = TC0_8BIT_PPG_DUTY50_1221Hz;
					break;
				case 5:					/* 610HZ,50% duty */
					T03REG = TC0_8BIT_PPG_INTERVAL_610Hz;
					T03PWM = TC0_8BIT_PPG_DUTY50_610Hz;
					break;
				case 6:					/* 2441HZ,75% duty */
					T03REG = TC0_8BIT_PPG_INTERVAL_2441Hz;
					T03PWM = TC0_8BIT_PPG_DUTY75_2441Hz;
					break;
				case 7:					/* 1221HZ,75% duty */
					T03REG = TC0_8BIT_PPG_INTERVAL_1221Hz;
					T03PWM = TC0_8BIT_PPG_DUTY75_1221Hz;
					break;
			/*	case 8: */
				default:				/* 610HZ,75% duty */
					T03REG = TC0_8BIT_PPG_INTERVAL_610Hz;
					T03PWM = TC0_8BIT_PPG_DUTY75_610Hz;
					break;
				}
			}
			else {
				/* Do nothing */
			}
		}
		else {
			/* Do nothing */
		}
	}
}
/**
 *  @}
 */ /* End of group TC0_8bit_PPG_Exported_functions */

/**
 *  @}
 */ /* End of group TC0_8bit_PPG */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
