/** 
 *******************************************************************************
 * @file    main.c
 * @brief   TC0_8bit_PWM Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-07-19 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup TC0_8bit_PWM TC0_8bit_PWM Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_8bit_PWM_Private_macro TC0_8bit_PWM Private Macro
 *  @{
 */
/** 
 *  @defgroup TC0_TC001EN TC02/03 Enable
 *  @brief    TC02/03 Control Enable.
 *  @{
 */
#define TC0_TC02_03_ENABLE				((UINT8_t)0x20)			/*!< TC02/03 enable */
/**
 *  @}
 */ /* End of group TC0_TC001EN */
/** 
 *  @defgroup TC0_T02PWM Macro Definition
 *  @brief    TC0 PWM Duty.
 *  @{
 */
#define TC0_8BIT_PWM_DUTY_MIN			((UINT8_t)0x02)			/*!< TC02PWM Min */
#define TC0_8BIT_PWM_DUTY_MAX			((UINT8_t)0xFE)			/*!< TC02PWM Max */
/**
 *  @}
 */ /* End of group TC0_T02PWM Macro Definition */
 /** 
 *  @defgroup TC0_TCK0 Macro Definition
 *  @brief    TC0 Clock.
 *  @{
 */
#define TC0_8bit_PWM_CLK				((UINT8_t)0x30)			/*!< TCK0(110=fcgck/2) */
/**
 *  @}
 */ /* End of group TC0_TCK0 Macro Definition */
/** 
 *  @defgroup TC0_TCM0 TC0 Mode Select
 *  @brief    TC0 Mode Select.
 *  @{
 */
#define TC0_T02_MODE					((UINT8_t)0x02)			/*!< TC0 8bit PWM mode */
/**
 *  @}
 */ /* End of group TC0_TCM0 */
/** 
 *  @defgroup TC0_T00RUN TC0 Start
 *  @brief    TC0 Start.
 *  @{
 */
#define TC0_START						((UINT8_t)0x01)			/*!< TC0 Start */
#define TC0_STOP						((UINT8_t)0x00)			/*!< TC0 Stop */
/**
 *  @}
 */ /* End of group TC0_T00RUN */
/** 
 *  @defgroup  Port Port Setting
 *  @brief     Port Setting
 *  @{
 */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
#define PORT_INPUT						((UINT8_t)0x00)			/*!< Port Mode Input */
#define PORT_PWM02						((UINT8_t)0x01)			/*!< Port Function PWM02 */
/** 
 *  @}
 */ /* End of group Port */

/** 
 *  @}
 */ /* End of group TC0_8bit_PWM_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_8bit_PWM_Private_define TC0_8bit_PWM Private Define
 *  @{
 */
#define KEYIN_CNT						((UINT8_t)100)			/*!< keychat=2 */
/** 
 *  @}
 */ /* End of group TC0_8bit_PWM_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_8bit_PWM_Private_variables TC0_8bit_PWM Private Variables
 *  @{
 */
UINT8_t pwm_duty_buf;								/*!< PWM duty buffer           */
static UINT8_t keybuf = (UINT8_t)0;					/*!<  key buffer          */
static UINT8_t keycode = (UINT8_t)0;				/*!<  key confirm         */
static UINT8_t keychat = KEYIN_CNT;					/*!<  key now             */
static UINT8_t key_already_flag = (UINT8_t)0;		/*!<  flag.               */
static UINT8_t comfirm_flag = (UINT8_t)0;			/*!<  flag.               */
/**
 *  @}
 */ /* End of group TC0_8bit_PWM_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_8bit_PWM_Private_fuctions TC0_8bit_PWM Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void Key(void);
static void TC0_8bit_PWM_Config(void);
static void TC0_8bit_PWM_Start(void);
static void TC0_8bit_PWM_Stop(void);
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Port Function Control : PWM02 */
	P8FC0 = PORT_PWM02;
	/* (2) Select output mode on PORT 80 */
	P8CR0 = PORT_OUTPUT;
	/* (3) Select input mode on PORT 12 (SW0) */
	P1CR2 = PORT_INPUT;
}
/*--------------------------------------------------*/
/** 
  * @brief  Key
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Key(void)
{
	UINT8_t keyTemp;

	keyTemp = (UINT8_t)(P1PRD & 0x04);
	keyTemp ^= 0xFF;
	
	if(keybuf == keyTemp) {
		if(--keychat == 0) {
			keychat = KEYIN_CNT;
			if((keycode != 0x00) && (key_already_flag == 0)) {
				key_already_flag = SET;
				comfirm_flag = SET;
			}
			else {
				/* Do nothing */
			}
			keycode = keybuf;
		}
		else {
			/* Do nothing */
		}
	}
	else {
		keychat = KEYIN_CNT;
		keybuf = keyTemp;
		key_already_flag = CLEAR;
		comfirm_flag = CLEAR;
	}
}
/*--------------------------------------------------*/
/** 
  * @brief  TC0_8bit_PWM_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TC0_8bit_PWM_Config(void)
{
	/* (1) Enable TC0"/TC03 Control */
	/* (2) Setting clock T02MOD[5:3]=110(fcgck/2),mode T02MOD[1:0]=10(8bit PWM mode) */
	/* (3) Setting T02PWM */
	POFFCR0 = TC0_TC02_03_ENABLE;					/* (1) */
	T02MOD = TC0_8bit_PWM_CLK | TC0_T02_MODE;		/* (3) */
	T02PWM = TC0_8BIT_PWM_DUTY_MIN;					/* (4) */
	pwm_duty_buf = TC0_8BIT_PWM_DUTY_MIN;
}
/*--------------------------------------------------*/
/** 
  * @brief  TC0_8bit_PWM_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TC0_8bit_PWM_Start(void)
{
	/* T00RUN = 1 */
	T02RUN = TC0_START;
}

/*--------------------------------------------------*/
/** 
  * @brief  TC0_8bit_PWM_Stop
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TC0_8bit_PWM_Stop(void)
{
	/* T02RUN = 0 */
	T02RUN = TC0_STOP;
}

/**
 *  @}
 */ /* End of group TC0_8bit_PWM_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_8bit_PWM_Exported_functions TC0_8bit_PWM Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	
	/*------------------------*/
	/* TC0_8bit_PWM Setting   */
	/*------------------------*/ 
	TC0_8bit_PWM_Config();
	/* Timer Start */
	TC0_8bit_PWM_Start();
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
		Key();
		if(comfirm_flag == 1) {
			comfirm_flag = CLEAR;
			if((keycode & 0x04) != 0) {
				/* PWM stop */
				TC0_8bit_PWM_Stop();
				
				/* PWM Duty 2 increase */
				if(pwm_duty_buf >= TC0_8BIT_PWM_DUTY_MAX) {
					pwm_duty_buf = TC0_8BIT_PWM_DUTY_MIN;
				}
				else {
					pwm_duty_buf += (UINT8_t)2;
				}
				T02PWM = pwm_duty_buf;
				TC0_8bit_PWM_Start();
			}
			else {
				/* Do nothing */
			}
		}
		else {
			/* Do nothing */
		}
	}
}

/**
 *  @}
 */ /* End of group TC0_8bit_PWM_Exported_functions */

/**
 *  @}
 */ /* End of group TC0_8bit_PWM */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
