/** 
 *******************************************************************************
 * @file    main.c
 * @brief   TC0_8bit_TIMER Sample Application.
 * @version V1.0.1.0
 * $Date:: 2019-06-10 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup TC0_8bit_TIMER TC0_8bit_TIMER Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_8bit_TIMER_Private_macro TC0_8bit_TIMER Private Macro
 *  @{
 */

/** 
 *  @defgroup TC0_TC001EN TC00/01 Enable
 *  @brief    TC00/01 Control Enable.
 *  @{
 */
#define TC0_TC00_01_ENABLE				((UINT8_t)0x10)			/*!< TC00/01 enable */
/**
 *  @}
 */ /* End of group TC0_TC001EN */
/** 
 *  @defgroup TC0_T00REG Macro Definition
 *  @brief    TC0 Interval.
 *  @{
 */
#define TC0_8BIT_TIMER_INTERVAL			((UINT8_t)0xA0)			/*!< TC00REG */
/**
 *  @}
 */ /* End of group TC0_T00REG Macro Definition */
 /** 
 *  @defgroup TC0_TCK0 Macro Definition
 *  @brief    TC0 Clock.
 *  @{
 */
#define TC0_8BIT_TIMER_CLK				((UINT8_t)0x28)			/*!< TCK0(101=fcgck/2^2) */
/**
 *  @}
 */ /* End of group TC0_TCK0 Macro Definition */
/** 
 *  @defgroup TC0_TCM0 TC0 Mode Select
 *  @brief    TC0 Mode Select.
 *  @{
 */
#define TC0_T00_MODE					((UINT8_t)0x00)			/*!< TC0 8bit timer mode */
/**
 *  @}
 */ /* End of group TC0_TCM0 */
/** 
 *  @defgroup TC0_T00RUN TC0 Start
 *  @brief    TC0 Start.
 *  @{
 */
#define TC0_START						((UINT8_t)0x01)			/*!< TC0 Start */
/**
 *  @}
 */ /* End of group TC0_T00RUN */
 /** 
 *  @defgroup  Port_Output Port Output ON/OFF
 *  @brief     Port Output ON/OFF
 *  @{
 */
#define PORT_OFF						((UINT8_t)0x00)			/*!< Port OFF */
/** 
 *  @}
 */ /* End of group Port_Output */
 /** 
 *  @defgroup  Port_Mode Port Mode
 *  @brief     Port Mode Input/Output
 *  @{
 */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
/** 
 *  @}
 */ /* End of group Port_Output */

/** 
 *  @}
 */ /* End of group TC0_8bit_TIMER_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_8bit_TIMER_Private_fuctions TC0_8bit_TIMER Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void TC0_8bit_TIMER_Config(void);
static void TC0_8bit_TIMER_Start(void);
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Select output mode on PORT 25 (LED0) */
	P2CR5 = PORT_OUTPUT;
	/* (2) Turn off PORT 25 (LED0) */
	P25 = PORT_OFF;
}

/*--------------------------------------------------*/
/** 
  * @brief  TC0_8bit_TIMER_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TC0_8bit_TIMER_Config(void)
{
	/* (1) Enable TC00/TC01 Control */
	/* (2) Enable interrupt of TC00 */
	/* (3) Setting clock T00MOD[5:3]=101(fcgck/2^2),mode T00MOD[1:0]=00(8bit tomer mode) */
	/* (4) Setting T00REG */
	POFFCR0 = TC0_TC00_01_ENABLE;					/* (1) */
	__DI();
	EIR_INTTC00 = 1;								/* (2) */
	__EI();
	T00MOD = TC0_8BIT_TIMER_CLK | TC0_T00_MODE;		/* (3) */
	T00REG = TC0_8BIT_TIMER_INTERVAL;				/* (4) */
}
/*--------------------------------------------------*/
/** 
  * @brief  TC0_8bit_TIMER_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TC0_8bit_TIMER_Start(void)
{
	/* T00RUN = 1 */
	T00RUN = TC0_START;
}
    
/**
 *  @}
 */ /* End of group TC0_8bit_TIMER_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TC0_8bit_TIMER_Exported_functions TC0_8bit_TIMER Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	
	/*------------------------*/
	/* TC0_8bit_TIMER Setting   */
	/*------------------------*/ 
	TC0_8bit_TIMER_Config();
	/* Timer Start */
	TC0_8bit_TIMER_Start();
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  INTTC00
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntTC00(void)
{
	static UINT8_t i = 0;
	
	i = P25;
	i ^= 1;
	P25 = i;
}
/**
 *  @}
 */ /* End of group TC0_8bit_TIMER_Exported_functions */

/**
 *  @}
 */ /* End of group TC0_8bit_TIMER */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
