/** 
 *******************************************************************************
 * @file    main.c
 * @brief   TCA_EVENT_COUNT Sample Application.
 * @version V1.0.1.0
 * $Date:: 2019-06-10 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup TCA_EVENT_COUNT TCA_EVENT_COUNT Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_EVENT_COUNT_Private_macro TCA_EVENT_COUNT Private Macro
 *  @{
 */

/** 
 *  @defgroup TCA_TCA0EN TCA0 Enable
 *  @brief    TC Control Enable.
 *  @{
 */
#define TC_CONTORL_ENABLE				((UINT8_t)0x01)			/*!< TCA0 enable */
/**
 *  @}
 */ /* End of group TCA_TCA0EN */
/** 
 *  @defgroup TCA_TA0DRA Macro Definition
 *  @brief    TCA Interval.
 *  @{
 */
#define TCA_EVENT_COUNT_INTERVAL		((UINT16_t)0x0180)		/*!< TA0DRA */
/**
 *  @}
 */ /* End of group TCA_TA0DRA Macro Definition */
/** 
 *  @defgroup TCA_TA0MOD Macro Definition
 *  @brief    TA0M Setting.
 *  @{
 */
#define TCA_EVENT_EDGE					((UINT8_t)0x40)			/*!< TCA ext trig edge(down) */
#define TCA_MODE						((UINT8_t)0x02)			/*!< TCA event count mode */
/**
 *  @}
 */ /* End of group TCA_TA0MOD Macro Definition */
/** 
 *  @defgroup TCA_TA0NC TCA Noize cancel
 *  @brief    TC0 Start.
 *  @{
 */
#define TCA_EVENT_COUNT_NOIZE			((UINT8_t)0x00)			/*!< TCA noize cansel(Non) */
/**
 *  @}
 */ /* End of group TCA_TA0NC Macro Definition */
/** 
 *  @defgroup TC0_T00RUN TC0 Start
 *  @brief    TC0 Start.
 *  @{
 */
#define TCA_START						((UINT8_t)0x01)			/*!< TCA Start */
/**
 *  @}
 */ /* End of group TC0_T00RUN */
/** 
 *  @defgroup  Port Port Setting
 *  @brief     Port Setting
 *  @{
 */
#define PORT_OFF						((UINT8_t)0x00)			/*!< Port OFF */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
#define PORT_INPUT						((UINT8_t)0x00)			/*!< Port Mode Input */
#define PORT_TCA0						((UINT8_t)0x01)			/*!< Port Function TCA0 */
/** 
 *  @}
 */ /* End of group Port */

/** 
 *  @}
 */ /* End of group TCA_EVENT_COUNT_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_EVENT_COUNT_Private_fuctions TCA_EVENT_COUNT Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void TCA_EVENT_COUNT_Config(void);
static void TCA_EVENT_COUNT_Start(void);
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Select output mode on PORT 27 (LED2) */
	P2CR7 = PORT_OUTPUT;
	/* (2) Turn off PORT 27 (LED2) */
	P27 = PORT_OFF;
	/* (3) Port Function Control : TCA0 */
	P7FC2 = PORT_TCA0;
	/* (4) Select input mode on PORT 72 */
	P7CR2 = PORT_INPUT;
}

/*--------------------------------------------------*/
/** 
  * @brief  TCA_EVENT_COUNT_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TCA_EVENT_COUNT_Config(void)
{
	/* (1) Enable TCA0 Control */
	/* (2) Enable interrupt of TCA0 */
	/* (3) Setting edge TA0MOD[6]=1(down),mode TA0MOD[2:0]=010(event count mode) */
	/* (4) Setting noize cancel TA0CR[5:4]=00(Non) */
	/* (5) Setting TA0DRAL/H */
	POFFCR0 = TC_CONTORL_ENABLE;									/* (1) */
	__DI();
	EIR_INTTCA0 = 1;												/* (2) */
	__EI();
	TA0MOD = TCA_EVENT_EDGE | TCA_MODE;								/* (3) */
	TA0CR  = TCA_EVENT_COUNT_NOIZE;									/* (4) */
	TA0DRAL = (UINT8_t)(TCA_EVENT_COUNT_INTERVAL & 0xFF);			/* (5) */
	TA0DRAH = (UINT8_t)((TCA_EVENT_COUNT_INTERVAL >> 8) & (0xFF));
}
/*--------------------------------------------------*/
/** 
  * @brief  TCA_EVENT_COUNT_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TCA_EVENT_COUNT_Start(void)
{
	/* TA0s = 1 */
	TA0S = TCA_START;
}

/**
 *  @}
 */ /* End of group TCA_EVENT_COUNT_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_EVENT_COUNT_Exported_functions TCA_EVENT_COUNT Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	
	/*------------------------*/
	/* TCA_EVENT_COUNT Setting      */
	/*------------------------*/ 
	TCA_EVENT_COUNT_Config();
	/* Timer Start */
	TCA_EVENT_COUNT_Start();
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  INTTCA0
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntTCA0(void)
{
	static UINT8_t i = 0;
	
	i = P27;
	i ^= 1;
	P27 = i;
}
/**
 *  @}
 */ /* End of group TCA_EVENT_COUNT_Exported_functions */

/**
 *  @}
 */ /* End of group TCA_EVENT_COUNT */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
