/** 
 *******************************************************************************
 * @file    main.c
 * @brief   TCA_EXT_TRIG Sample Application.
 * @version V1.0.1.0
 * $Date:: 2019-06-10 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup TCA_EXT_TRIG TCA_EXT_TRIG Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_EXT_TRIG_Private_macro TCA_EXT_TRIG Private Macro
 *  @{
 */

/** 
 *  @defgroup TCA_TCA0EN TCA0 Enable
 *  @brief    TC Control Enable.
 *  @{
 */
#define TC_CONTORL_ENABLE				((UINT8_t)0x01)			/*!< TCA0 enable */
/**
 *  @}
 */ /* End of group TCA_TCA0EN */
/** 
 *  @defgroup TCA_TA0DRA Macro Definition
 *  @brief    TCA Interval.
 *  @{
 */
#define TCA_EXT_TRIG_INTERVAL			((UINT16_t)0x0100)		/*!< TA0DRA */
/**
 *  @}
 */ /* End of group TCA_TA0DRA Macro Definition */
/** 
 *  @defgroup TCA_TA0MOD Macro Definition
 *  @brief    TA0M Setting.
 *  @{
 */
#define TCA_EXT_TRIG_EDGE				((UINT8_t)0x40)			/*!< TCA ext trig edge(down) */
#define TCA_EXT_TRIG_CLK				((UINT8_t)0x18)			/*!< TCA source clock(fcgck/2) */
#define TCA_MODE						((UINT8_t)0x04)			/*!< TCA ext trig mode */
/**
 *  @}
 */ /* End of group TCA_TA0MOD Macro Definition */
/** 
 *  @defgroup TCA_TA0NC TCA Noize cancel
 *  @brief    TC0 Start.
 *  @{
 */
#define TCA_EXT_TRIG_NOIZE				((UINT8_t)0x00)			/*!< TCA noize cansel(Non) */
/**
 *  @}
 */ /* End of group TCA_TA0NC Macro Definition */
/** 
 *  @defgroup TC0_T00RUN TC0 Start
 *  @brief    TC0 Start.
 *  @{
 */
#define TCA_START						((UINT8_t)0x01)			/*!< TCA Start */
/**
 *  @}
 */ /* End of group TC0_T00RUN */
/** 
 *  @defgroup  Port Port Setting
 *  @brief     Port Setting
 *  @{
 */
#define PORT_OFF						((UINT8_t)0x00)			/*!< Port OFF */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
#define PORT_INPUT						((UINT8_t)0x00)			/*!< Port Mode Input */
#define PORT_TCA0						((UINT8_t)0x01)			/*!< Port Function TCA0 */
/** 
 *  @}
 */ /* End of group Port */

/** 
 *  @}
 */ /* End of group TCA_EXT_TRIG_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_EXT_TRIG_Private_fuctions TCA_EXT_TRIG Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void TCA_EXT_TRIG_Config(void);
static void TCA_EXT_TRIG_Start(void);
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Select output mode on PORT 26 (LED1) */
	P2CR6 = PORT_OUTPUT;
	/* (2) Turn off PORT 26 (LED1) */
	P26 = PORT_OFF;
	/* (3) Port Function Control : TCA0 */
	P7FC2 = PORT_TCA0;
	/* (4) Select input mode on PORT 72 */
	P7CR2 = PORT_INPUT;
}

/*--------------------------------------------------*/
/** 
  * @brief  TCA_EXT_TRIG_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TCA_EXT_TRIG_Config(void)
{
	/* (1) Enable TCA0 Control */
	/* (2) Enable interrupt of TCA0 */
	/* (3) Setting edge TA0MOD[6]=1(down),source clock TA0MOD[2:0]=11(fcgck/2),mode TA0MOD[2:0]=100(ext trig mode) */
	/* (4) Setting noize cancel TA0CR[5:4]=00(Non) */
	/* (5) Setting TA0DRAL/H */
	POFFCR0 = TC_CONTORL_ENABLE;									/* (1) */
	__DI();
	EIR_INTTCA0 = 1;												/* (2) */
	__EI();
	TA0MOD = TCA_EXT_TRIG_EDGE | TCA_EXT_TRIG_CLK | TCA_MODE;		/* (3) */
	TA0CR  = TCA_EXT_TRIG_NOIZE;									/* (4) */
	TA0DRAL = (UINT8_t)(TCA_EXT_TRIG_INTERVAL & 0xFF);				/* (5) */
	TA0DRAH = (UINT8_t)((TCA_EXT_TRIG_INTERVAL >> 8) & (0xFF));
}
/*--------------------------------------------------*/
/** 
  * @brief  TCA_EXT_TRIG_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TCA_EXT_TRIG_Start(void)
{
	/* TA0s = 1 */
	TA0S = TCA_START;
}
    
/**
 *  @}
 */ /* End of group TCA_EXT_TRIG_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_EXT_TRIG_Exported_functions TCA_EXT_TRIG Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	
	/*------------------------*/
	/* TCA_EXT_TRIG Setting      */
	/*------------------------*/ 
	TCA_EXT_TRIG_Config();
	/* Timer Start */
	TCA_EXT_TRIG_Start();
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  INTTCA0
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntTCA0(void)
{
	static UINT8_t i = 0;
	
	i = P26;
	i ^= 1;
	P26 = i;
}
/**
 *  @}
 */ /* End of group TCA_EXT_TRIG_Exported_functions */

/**
 *  @}
 */ /* End of group TCA_EXT_TRIG */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
