/** 
 *******************************************************************************
 * @file    main.c
 * @brief   TCA_PPG Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-07-19 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdlib.h>
#include "TMP89Fx.h"  /*!< TMP89Fx Group Header file. */
#include "cmn_type.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup TCA_PPG TCA_PPG Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_PPG_Private_macro TCA_PPG Private Macro
 *  @{
 */

/** 
 *  @defgroup TBT_TBTCK Macro Definition
 *  @brief    TBT Interval.
 *  @{
 */
#define TBT_INTERVAL					((UINT8_t)0x04)			/*!< TBT Interrupt Clock(=2^12) */
/**
 *  @}
 */ /* End of group TBT_TBTCK Macro Definition */
/** 
 *  @defgroup TBT_TBTEN TBT Start
 *  @brief    TBT Start.
 *  @{
 */
#define TBT_TBTEN_START					((UINT8_t)0x01)			/*!< TBT Start(Innterrupt request signal enable) */
/**
 *  @}
 */ /* End of group TBT_TBTEN */
/** 
 *  @defgroup TCA_TCA1EN TCA1 Enable
 *  @brief    TC Control Enable.
 *  @{
 */
#define TC_CONTORL_ENABLE				((UINT8_t)0x02)			/*!< TCA1 enable */
/**
 *  @}
 */ /* End of group TCA_TCA1EN */
/** 
 *  @defgroup TCA_TA0DRA Macro Definition
 *  @brief    TCA Interval.
 *  @{
 */
#define TCA_PPG_INTERVAL_LEADER			((UINT16_t)11250)			/*!< LEADER */
#define TCA_PPG_INTERVAL_CODE_ZERO		((UINT16_t)1400)			/*!< SYSTEM CODE 0 */
#define TCA_PPG_INTERVAL_CODE_ONE		((UINT16_t)2813)			/*!< SYSTEM CODE 1 */
/**
 *  @}
 */ /* End of group TCA_TA0DRA Macro Definition */
/** 
 *  @defgroup TCA_TA0DRB Macro Definition
 *  @brief    TCA Interval.
 *  @{
 */
#define TCA_PPG_INTERVAL_LEADER_HI		((UINT16_t)5625)			/*!< LEADER */
#define TCA_PPG_INTERVAL_CODE_HI		((UINT16_t)700)				/*!< SYSTEM CODE 0/1 HI */
/**
 *  @}
 */ /* End of group TCA_TA0DRB Macro Definition */
/** 
 *  @defgroup TCA_TA0MOD Macro Definition
 *  @brief    TA0MOD Setting.
 *  @{
 */
#define TCA_PPG_CLK						((UINT8_t)0x10)			/*!< TCA PPG source clock(fcgck/2^2) */
#define TCA_MODE						((UINT8_t)0x03)			/*!< TCA PPG mode */
/**
 *  @}
 */ /* End of group TCA_TA0MOD Macro Definition */
/** 
 *  @defgroup TCA_TA0NC TCA Noize cancel
 *  @brief    TCA noize cancel.
 *  @{
 */
#define TCA_PPG_TFF						((UINT8_t)0x40)			/*!< TCA F/F control(set) */
#define TCA_PPG_NOIZE					((UINT8_t)0x00)			/*!< TCA noize cansel(Non) */
/**
 *  @}
 */ /* End of group TCA_TA0NC Macro Definition */
/** 
 *  @defgroup TCA_TA0S TCA Start
 *  @brief    TCA Start/Stop.
 *  @{
 */
#define TCA_START						((UINT8_t)0x01)			/*!< TCA Start */
/**
 *  @}
 */ /* End of group TCA_TA0S */
/** 
 *  @defgroup  Port Port Setting
 *  @brief     Port Setting
 *  @{
 */
#define PORT_OFF						((UINT8_t)0x00)			/*!< Port OFF */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
#define PORT_FUNC_ON					((UINT8_t)0x01)			/*!< Port Function ON */
#define PORT_FUNC_OFF					((UINT8_t)0x00)			/*!< Port Function OFF */
/** 
 *  @}
 */ /* End of group Port */

/** 
 *  @}
 */ /* End of group TCA_PPG_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_PPG_Private_define TCA_PPG Private Define
 *  @{
 */
#define STEP_LEADER						((UINT8_t)0x00)			/*!< Step : trans "Leader" */
#define STEP_DATA						((UINT8_t)0x01)			/*!< Step : trans "Data" */

#define BUFF_NUM						((UINT8_t)4)			/*!< Step : trans "Data=1" */
/** 
 *  @}
 */ /* End of group TCA_PPG_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_PPG_Private_variables TCA_PPG Private Variables
 *  @{
 */
static UINT8_t step         = STEP_LEADER;		/*!<  step.                 */
static UINT8_t bit_cnt      = (UINT8_t)0;		/*!<  bit count.            */
static UINT8_t byte_cnt     = (UINT8_t)0;		/*!<  byte count.           */
static UINT8_t timer_flag   = (UINT8_t)0;		/*!< main timer flag        */

static const UINT8_t PpgData[BUFF_NUM] = {
	0x85,0x85,0x37,0xC8							/*!< PPG data*/
};
/**
 *  @}
 */ /* End of group TCA_PPG_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_PPG_Private_fuctions TCA_PPG Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void TBT_Config(void);
static void TBT_Start(void);
static void TCA_PPG_Config(void);
static void TCA_PPG_Start(void);
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Port Function Control : PPGA1 */
	P7FC3 = PORT_FUNC_ON;
	/* (2) Select output mode on PORT 73 */
	P7CR3 = PORT_OUTPUT;
}
/*--------------------------------------------------*/
/** 
  * @brief  TBT_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TBT_Config(void)
{
	/* (1) Enable interrupt of TBT */
	/* (2) Setting interrupt frequency,TBTCR[2:0]=100(fcgck/2^12) */
	/* (3) Enable interrupt request signal,TBTCR[3]=1 */
	__DI();
	EIR_INTTBT = SET;				/* (1) */
	__EI();
	TBTCR  = TBT_INTERVAL;			/* (2) */
	TBTEN  = TBT_TBTEN_START;		/* (3) */
}
/*--------------------------------------------------*/
/** 
  * @brief  TBT_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TBT_Start(void)
{
	/* TBTEN = 1 */
	TBTEN = TBT_TBTEN_START;
}
/*--------------------------------------------------*/
/** 
  * @brief  TCA_PPG_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TCA_PPG_Config(void)
{
	/* (1) Enable TCA1 Control */
	/* (2) Enable interrupt of TCA1 */
	/* (3) Setting TA0MOD,source clock(10=fcgck/2^2)/mode(011=ppg mode) */
	/* (4) Setting TA0CR,noize cancel(00=Non) */
	/* (5) Setting TA0DRA,9ms/0.8us=11250 */
	/* (6) Setting TA0DRB,4.5ms/0.8us=5625 */
	POFFCR0 = TC_CONTORL_ENABLE;									/* (1) */
	__DI();
	EIR_INTTCA1 = 1;												/* (2) */
	__EI();
	TA1MOD = TCA_PPG_CLK | TCA_MODE;								/* (3) */
	TA1CR  = TCA_PPG_TFF | TCA_PPG_NOIZE;							/* (4) */
	TA1DRA = TCA_PPG_INTERVAL_LEADER;								/* (5) */
	TA1DRB = TCA_PPG_INTERVAL_LEADER_HI;							/* (6) */
	
	step = STEP_LEADER;
}
/*--------------------------------------------------*/
/** 
  * @brief  TCA_PPG_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TCA_PPG_Start(void)
{
	/* TA1S = 1 */
	TA1S = TCA_START;
}

/**
 *  @}
 */ /* End of group TCA_PPG_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_PPG_Exported_functions TCA_PPG Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	
	TBT_Config();
	TBT_Start();
	/*------------------------*/
	/* TCA_PPG Setting      */
	/*------------------------*/ 
	TCA_PPG_Config();
	/* Timer Start */
	TCA_PPG_Start();
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
		if(timer_flag == SET) {
			__DI();
			timer_flag = CLEAR;
			__EI();
			Port_Config();
			TCA_PPG_Config();
			/* Timer Start */
			TCA_PPG_Start();
		}
		else {
			/* Do nothing */
		}
	}
}
/*--------------------------------------------------*/
/** 
  * @brief  INTTBT
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntTBT(void)
{
	static UINT8_t i = (UINT8_t)0;
	/* 1ms cycle */
	i++;
	if(i >= 250) {
		i = (UINT8_t)0;
		timer_flag = SET;
	}
	else {
		/* Do nothing */
	}
}
/*--------------------------------------------------*/
/** 
  * @brief  INTTCA0
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntTCA1(void)
{
	if(step == STEP_LEADER) {
		step = STEP_DATA;
		bit_cnt = (UINT8_t)0;
		byte_cnt = (UINT8_t)0;
	}
	else {
		if(++bit_cnt >= 8) {
			bit_cnt = (UINT8_t)0;
			byte_cnt++;
			if(byte_cnt >= 4) {
				P7FC3 = PORT_FUNC_OFF;
				P73 = PORT_OFF;
				return;
			}
			else {
				/* Do nothing */
			}
		}
		else {
			/* Do nothing */
		}
	}
	
	/* PPG setting */
	if((PpgData[byte_cnt] & (0x01 << bit_cnt)) == 0) {
		/* 1.12ms/0.8us=1400 , 0.56ms/0.8us=700 */
		TA1DRA = TCA_PPG_INTERVAL_CODE_ZERO;
		TA1DRB = TCA_PPG_INTERVAL_CODE_HI;
	}
	else {
		/* 2.25ms/0.8us=2812.52813 , 0.56ms/16us=700 */
		TA1DRA = TCA_PPG_INTERVAL_CODE_ONE;
		TA1DRB = TCA_PPG_INTERVAL_CODE_HI;
	}
	
	/* TCA PPG start */
	TCA_PPG_Start();
}
/**
 *  @}
 */ /* End of group TCA_PPG_Exported_functions */

/**
 *  @}
 */ /* End of group TCA_PPG */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
