/** 
 *******************************************************************************
 * @file    main.c
 * @brief   TCA_PULSE_WIDTH_MEASURE Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-07-19 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdio.h>
#include	<string.h>
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"
#include	"uart_io.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup TCA_PULSE_WIDTH_MEASURE TCA_PULSE_WIDTH_MEASURE Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_PULSE_WIDTH_MEASURE_Private_macro TCA_PULSE_WIDTH_MEASURE Private Macro
 *  @{
 */

/** 
 *  @defgroup TCA_TCA1EN TCA1 Enable
 *  @brief    TC Control Enable.
 *  @{
 */
#define TC_CONTORL_ENABLE				((UINT8_t)0x02)			/*!< TCA1 enable */
/**
 *  @}
 */ /* End of group TCA_TCA1EN */
/** 
 *  @defgroup TCA_TA0MOD Macro Definition
 *  @brief    TA0M Setting.
 *  @{
 */
#define TCA_PULSE_WIDTH_MEASURE_EDGE	((UINT8_t)0x40)			/*!< TCA ext trig edge(down) */
#define TCA_PULSE_WIDTH_MEASURE_CLK		((UINT8_t)0x10)			/*!< TCA source clock(fcgck/2^2) */
#define TCA_PULSE_WIDTH_MEASURE_EDGE_CAP	TCA__EDGE_CAP_ONE_EDGE			/*!< TCA cap edge */
#define TCA__EDGE_CAP_ONE_EDGE			((UINT8_t)0x20)			/*!< TCA cap edge(one edge) */
#define TCA__EDGE_CAP_BOTH_EDGE			((UINT8_t)0x00)			/*!< TCA cap edge(both edge) */
#define TCA_MODE						((UINT8_t)0x06)			/*!< TCA pulse width measure mode */
/**
 *  @}
 */ /* End of group TCA_TA0MOD Macro Definition */
/** 
 *  @defgroup TCA_TA0NC TCA Noize cancel
 *  @brief    TC0 Start.
 *  @{
 */
#define TCA_PULSE_WIDTH_MEASURE_OVERFLOW	((UINT8_t)0x01)			/*!< interrupt when over flow */
#define TCA_PULSE_WIDTH_MEASURE_NOIZE	((UINT8_t)0x00)			/*!< TCA noize cansel(Non) */
/**
 *  @}
 */ /* End of group TCA_TA0NC Macro Definition */
/** 
 *  @defgroup TC0_T00RUN TC0 Start
 *  @brief    TC0 Start.
 *  @{
 */
#define TCA_START						((UINT8_t)0x01)			/*!< TCA Start */
/**
 *  @}
 */ /* End of group TC0_T00RUN */
/** 
 *  @defgroup  Port Port Setting
 *  @brief     Port Setting
 *  @{
 */
#define PORT_INPUT						((UINT8_t)0x00)			/*!< Port Mode Input */
/** 
 *  @}
 */ /* End of group Port */

/** 
 *  @}
 */ /* End of group TCA_PULSE_WIDTH_MEASURE_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_PULSE_WIDTH_MEASURE_Private_define TCA_PULSE_WIDTH_MEASURE Private Define
 *  @{
 */
const UINT8_t disp_tbl1[] = {"Pulse width measurement mode Sample Program\n"};
const UINT8_t disp_tbl2[] = {"--------------------------------------------------\n"};
const UINT8_t disp_tbl3[] = {"Capture Edge: Single\n"};
const UINT8_t disp_tbl4[] = {"Capture Edge: Both\n"};
const UINT8_t disp_tbl5[] = {"Capture value:\n"};
/** 
 *  @}
 */ /* End of group TCA_PULSE_WIDTH_MEASURE_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_PULSE_WIDTH_MEASURE_Private_fuctions TCA_PULSE_WIDTH_MEASURE Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void TCA_PULSE_WIDTH_MEASURE_Config(void);
static void TCA_PULSE_WIDTH_MEASURE_Start(void);
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Select input mode on PORT 73 */
	P7CR3 = PORT_INPUT;
}

/*--------------------------------------------------*/
/** 
  * @brief  TCA_PULSE_WIDTH_MEASURE_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TCA_PULSE_WIDTH_MEASURE_Config(void)
{
	/* (1) Enable TCA1 Control */
	/* (2) Enable interrupt of TCA1 */
	/* (3) Setting edge TA0MOD[6]=0(down),source clock TA0MOD[2:0]=10(fcgck/2^2),
	       mode TA0MOD[2:0]=110(pulse width mode) */
	/* (4) Setting noize cancel TA0CR[5:4]=00(Non) */
	POFFCR0 = TC_CONTORL_ENABLE;									/* (1) */
	__DI();
	ILPRS6 = 0x03;
	EIR_INTTCA1 = 1;												/* (2) */
	__EI();
	TA1MOD = TCA_PULSE_WIDTH_MEASURE_EDGE | TCA_PULSE_WIDTH_MEASURE_CLK |TCA_PULSE_WIDTH_MEASURE_EDGE_CAP | TCA_MODE;
																	/* (3) */
	TA1CR  = TCA_PULSE_WIDTH_MEASURE_NOIZE | TCA_PULSE_WIDTH_MEASURE_OVERFLOW;
																	/* (4) */
}
/*--------------------------------------------------*/
/** 
  * @brief  TCA_PULSE_WIDTH_MEASURE_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TCA_PULSE_WIDTH_MEASURE_Start(void)
{
	/* TA1s = 1 */
	TA1S = TCA_START;
}

/**
 *  @}
 */ /* End of group TCA_PULSE_WIDTH_MEASURE_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_PULSE_WIDTH_MEASURE_Exported_functions TCA_PULSE_WIDTH_MEASURE Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	UINT8_t edge = TCA_PULSE_WIDTH_MEASURE_EDGE_CAP;
	
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	/*------------------------*/
	/* UART Setting            */
	/*------------------------*/
	uart_initialize();
	
	Send_Display_Data((UINT8_t)sizeof(disp_tbl1), (UINT8_t *)disp_tbl1);
	Send_Display_Data((UINT8_t)sizeof(disp_tbl2), (UINT8_t *)disp_tbl2);
	if(edge == TCA__EDGE_CAP_ONE_EDGE) {
		Send_Display_Data((UINT8_t)sizeof(disp_tbl3), (UINT8_t *)disp_tbl3);
	}
	else {
		Send_Display_Data((UINT8_t)sizeof(disp_tbl4), (UINT8_t *)disp_tbl4);
	}
	
	Send_Display_Data((UINT8_t)sizeof(disp_tbl5), (UINT8_t *)disp_tbl5);
	/*------------------------*/
	/* TCA_PULSE_WIDTH_MEASURE Setting      */
	/*------------------------*/ 
	TCA_PULSE_WIDTH_MEASURE_Config();
	/* Timer Start */
	TCA_PULSE_WIDTH_MEASURE_Start();
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  INTTCA1
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntTCA1(void)
{
	UINT16_t capture = (UINT16_t)0;
	UINT8_t t_state = (UINT8_t)0;

	UINT8_t a = (UINT8_t)0;
	
	t_state = TA1SR;
	if(((t_state & 0x80) == 0) && ((t_state & 0x01) != 0)) {
		/* capture read */
		capture = TA1DRB;
		
		write = 0;
		tx_length = (UINT8_t)7;
		
		tx_data[0] = (CHAR_t)'0';
		tx_data[1] = (CHAR_t)'x';
		
		a = (UINT8_t)(capture / 4096);
		if(a >= 10) {
			a -= 10;
			a += 65;
		}
		else {
			a += 48;
		}
		tx_data[2] = a;
		a = (UINT8_t)((capture % 4096) / 256);
		if(a >= 10) {
			a -= 10;
			a += 65;
		}
		else {
			a += 48;
		}
		tx_data[3] = a;
		a = (UINT8_t)(((capture % 4096) % 256) / 16);
		if(a >= 10) {
			a -= 10;
			a += 65;
		}
		else {
			a += 48;
		}
		tx_data[4] = a;
		a = (UINT8_t)(((capture % 4096) % 256) % 16);
		if(a >= 10) {
			a -= 10;
			a += 65;
		}
		else {
			a += 48;
		}
		tx_data[5] = a;
		tx_data[6] = (CHAR_t)'\n';
		TX_Start();
		TD2BUF = tx_data[write];
	}
	else {
		/* Do nothing */
	}
}
/**
 *  @}
 */ /* End of group TCA_PULSE_WIDTH_MEASURE_Exported_functions */

/**
 *  @}
 */ /* End of group TCA_PULSE_WIDTH_MEASURE */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
