/** 
 *******************************************************************************
 * @file    main.c
 * @brief   TCA_TIMER Sample Application.
 * @version V1.0.1.0
 * $Date:: 2019-06-10 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup TCA_TIMER TCA_TIMER Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_TIMER_Private_macro TCA_TIMER Private Macro
 *  @{
 */

/** 
 *  @defgroup TCA_TCA0EN TCA0 Enable
 *  @brief    TC Control Enable.
 *  @{
 */
#define TC_CONTORL_ENABLE				((UINT8_t)0x01)			/*!< TCA0 enable */
/**
 *  @}
 */ /* End of group TCA_TCA0EN */
/** 
 *  @defgroup TCA_TA0DRA Macro Definition
 *  @brief    TCA Interval.
 *  @{
 */
#define TCA_TIMER_INTERVAL				((UINT16_t)0x0200)		/*!< TA0DRA */
/**
 *  @}
 */ /* End of group TCA_TA0DRA Macro Definition */
/** 
 *  @defgroup TCA_TA0CK Macro Definition
 *  @brief    TCA Clock.
 *  @{
 */
#define TCA_TIMER_CLK					((UINT8_t)0x18)			/*!< TA0CK(11=fcgck/2) */
/**
 *  @}
 */ /* End of group TCA_TA0CK Macro Definition */
/** 
 *  @defgroup TCA_TA0NC TCA Noize cancel
 *  @brief    TC0 Start.
 *  @{
 */
#define TCA_TIMER_NOIZE					((UINT8_t)0x10)			/*!< TCA noize cansel(fcgck/2) */
/**
 *  @}
 */ /* End of group TCA_TA0NC Macro Definition */
/** 
 *  @defgroup TCA_TA0M Macro Definition
 *  @brief    TA0M Setting.
 *  @{
 */
#define TCA_MODE						((UINT8_t)0x00)			/*!< TCA timer mode */
/**
 *  @}
 */ /* End of group TCA_TA0M Macro Definition */
/** 
 *  @defgroup TC0_T00RUN TC0 Start
 *  @brief    TC0 Start.
 *  @{
 */
#define TCA_START						((UINT8_t)0x01)			/*!< TCA Start */
/**
 *  @}
 */ /* End of group TC0_T00RUN */
 /** 
 *  @defgroup  Port_Output Port Output ON/OFF
 *  @brief     Port Output ON/OFF
 *  @{
 */
#define PORT_OFF						((UINT8_t)0x00)			/*!< Port OFF */
/** 
 *  @}
 */ /* End of group Port_Output */
 /** 
 *  @defgroup  Port_Mode Port Mode
 *  @brief     Port Mode Input/Output
 *  @{
 */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
/** 
 *  @}
 */ /* End of group Port_Output */

/** 
 *  @}
 */ /* End of group TCA_TIMER_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_TIMER_Private_fuctions TCA_TIMER Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void TCA_TIMER_Config(void);
static void TCA_TIMER_Start(void);
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Select output mode on PORT 25 (LED0) */
	P2CR5 = PORT_OUTPUT;
	/* (2) Turn off PORT 25 (LED0) */
	P25 = PORT_OFF;
}

/*--------------------------------------------------*/
/** 
  * @brief  TCA_TIMER_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TCA_TIMER_Config(void)
{
	/* (1) Enable TCA0 Control */
	/* (2) Enable interrupt of TCA0 */
	/* (3) Setting clock TA0MOD[4:3]=11(fcgck/2),mode TA0MOD[1:0]=00(timer mode) */
	/* (4) Setting noize cancel TA0CR[5:4]=01(fcgck/2) */
	/* (5) Setting TA0DRAL/H */
	POFFCR0 = TC_CONTORL_ENABLE;									/* (1) */
	__DI();
	EIR_INTTCA0 = 1;												/* (2) */
	__EI();
	TA0MOD = TCA_TIMER_CLK | TCA_MODE;								/* (3) */
	TA0CR  = TCA_TIMER_NOIZE;										/* (4) */
	TA0DRAL = (UINT8_t)(TCA_TIMER_INTERVAL & 0xFF);					/* (5) */
	TA0DRAH = (UINT8_t)((TCA_TIMER_INTERVAL >> 8) & (0xFF));
}
/*--------------------------------------------------*/
/** 
  * @brief  TCA_TIMER_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TCA_TIMER_Start(void)
{
	/* TA0s = 1 */
	TA0S = TCA_START;
}
    
/**
 *  @}
 */ /* End of group TCA_TIMER_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_TIMER_Exported_functions TCA_TIMER Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	
	/*------------------------*/
	/* TCA_TIMER Setting      */
	/*------------------------*/ 
	TCA_TIMER_Config();
	/* Timer Start */
	TCA_TIMER_Start();
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  INTTCA0
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntTCA0(void)
{
	static UINT8_t i = 0;
	
	i = P25;
	i ^= 1;
	P25 = i;
}
/**
 *  @}
 */ /* End of group TCA_TIMER_Exported_functions */

/**
 *  @}
 */ /* End of group TCA_TIMER */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
