/** 
 *******************************************************************************
 * @file    main.c
 * @brief   TCA_WINDOWS Sample Application.
 * @version V1.0.1.0
 * $Date:: 2019-06-10 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup TCA_WINDOWS TCA_WINDOWS Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_WINDOWS_Private_macro TCA_WINDOWS Private Macro
 *  @{
 */
/** 
 *  @defgroup TCA_TA0DRA Macro Definition
 *  @brief    TCA Interval.
 *  @{
 */
#define TCA_TIMER_INTERVAL				((UINT16_t)0x4E20)		/*!< TA0DRA */
/**
 *  @}
 */ /* End of group TCA_TA0DRA Macro Definition */
/** 
 *  @defgroup TCA_TCA0EN TCA0 Enable
 *  @brief    TC Control Enable.
 *  @{
 */
#define TC_CONTORL_ENABLE				((UINT8_t)0x01)			/*!< TCA0 enable */
/**
 *  @}
 */ /* End of group TCA_TCA0EN */
/** 
 *  @defgroup TCA_TA0MOD Macro Definition
 *  @brief    TA0M Setting.
 *  @{
 */
#define TCA_WINDOWS_EDGE				((UINT8_t)0x00)			/*!< TCA ext trig edge(Hi Level) */
#define TCA_WINDOWS_CLK					((UINT8_t)0x18)			/*!< TCA source clock(fcgck/2) */
#define TCA_MODE						((UINT8_t)0x05)			/*!< TCA window mode */
/**
 *  @}
 */ /* End of group TCA_TA0MOD Macro Definition */
/** 
 *  @defgroup TCA_TA0NC TCA Noize cancel
 *  @brief    TC0 Start.
 *  @{
 */
#define TCA_WINDOWS_NOIZE				((UINT8_t)0x00)			/*!< TCA noize cansel(Non) */
/**
 *  @}
 */ /* End of group TCA_TA0NC Macro Definition */
/** 
 *  @defgroup TC0_T00RUN TC0 Start
 *  @brief    TC0 Start.
 *  @{
 */
#define TCA_START						((UINT8_t)0x01)			/*!< TCA Start */
/**
 *  @}
 */ /* End of group TC0_T00RUN */
/** 
 *  @defgroup  Port Port Setting
 *  @brief     Port Setting
 *  @{
 */
#define PORT_OFF						((UINT8_t)0x00)			/*!< Port OFF */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
#define PORT_INPUT						((UINT8_t)0x00)			/*!< Port Mode Input */
#define PORT_TCA0						((UINT8_t)0x01)			/*!< Port Function TCA0 */
/** 
 *  @}
 */ /* End of group Port */

/** 
 *  @}
 */ /* End of group TCA_WINDOWS_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_WINDOWS_Private_fuctions TCA_WINDOWS Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void TCA_WINDOWS_Config(void);
static void TCA_WINDOWS_Start(void);
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Select output mode on PORT 11 (LED3) */
	P1CR1 = PORT_OUTPUT;
	/* (2) Turn off PORT 11 (LED3) */
	P11 = PORT_OFF;
	/* (3) Port Function Control : TCA1 */
	P7FC2 = PORT_TCA0;
	/* (4) Select input mode on PORT 72 */
	P7CR2 = PORT_INPUT;
}

/*--------------------------------------------------*/
/** 
  * @brief  TCA_WINDOWS_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TCA_WINDOWS_Config(void)
{
	/* (1) Enable TCA0 Control */
	/* (2) Enable interrupt of TCA0 */
	/* (3) Setting edge TA0MOD[6]=0(Hi Level),source clock TA0MOD[2:0]=11(fcgck/2),mode TA0MOD[2:0]=101(window mode) */
	/* (4) Setting noize cancel TA0CR[5:4]=00(Non) */
	/* (5) Setting TA0DRAL/H */
	POFFCR0 = TC_CONTORL_ENABLE;									/* (1) */
	__DI();
	EIR_INTTCA0 = 1;												/* (2) */
	__EI();
	TA0MOD = TCA_WINDOWS_EDGE | TCA_WINDOWS_CLK | TCA_MODE;			/* (3) */
	TA0CR  = TCA_WINDOWS_NOIZE;										/* (4) */
	TA0DRAL = (UINT8_t)(TCA_TIMER_INTERVAL & 0xFF);					/* (5) */
	TA0DRAH = (UINT8_t)((TCA_TIMER_INTERVAL >> 8) & (0xFF));
}
/*--------------------------------------------------*/
/** 
  * @brief  TCA_WINDOWS_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TCA_WINDOWS_Start(void)
{
	/* TA0s = 1 */
	TA0S = TCA_START;
}
    
/**
 *  @}
 */ /* End of group TCA_WINDOWS_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TCA_WINDOWS_Exported_functions TCA_WINDOWS Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	
	/*------------------------*/
	/* TCA_WINDOWS Setting      */
	/*------------------------*/ 
	TCA_WINDOWS_Config();
	/* Timer Start */
	TCA_WINDOWS_Start();
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  INTTCA0
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntTCA0(void)
{
	static UINT8_t i = 0;
	
	i = P11;
	i ^= 1;
	P11 = i;
}
/**
 *  @}
 */ /* End of group TCA_WINDOWS_Exported_functions */

/**
 *  @}
 */ /* End of group TCA_WINDOWS */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
