/** 
 *******************************************************************************
 * @file    main.c
 * @brief   UART Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-07-19 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdio.h>
#include	<string.h>
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup UART UART Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_Private_macro UART Private Macro
 *  @{
 */
/** 
 *  @defgroup Serial_setting Macro Definisiton
 *  @brief    Serial setting
 *  @{
 */
#define SERIAL_SEL_UART_PORT			((UINT8_t)0x10)				/*!< UART0 Port select */
#define UART_PORT_TRXD_P9				((UINT8_t)0x00)				/*!< UART2 TXD,RXD Port select */
/**
 *  @}
 */ /* End of group Serial_setting */
/** 
 *  @defgroup UART_UART0 UART0 Enable
 *  @brief    UART0 Control Enable Macro Definisiton.
 *  @{
 */
#define UART_UART0_ENABLE				((UINT8_t)0x01)				/*!< UART0 Control Enable */
#define UART_UART1_ENABLE				((UINT8_t)0x02)				/*!< UART1 Control Enable */
#define UART_UART2_ENABLE				((UINT8_t)0x04)				/*!< UART2 Control Enable */
/**
 *  @}
 */ /* End of group UART_UART2 */
/** 
 *  @defgroup BaudRateConfiguration BaudRate Configuration
 *  @brief    BaudRate .
 *  @{
 */
#define UART_BAUD_RATE					((UINT8_t)(0x04))
/** 
 *  @}
 */ /* End of group BoundRateConfiguration */
/** 
 *  @defgroup DataLengtheConfiguration Data Length Configuration
 *  @brief    Data Length Configuration.
 *  @{
 */
#define UART_DATA_LENGTH				((UINT8_t)250)
#define DATA_LENGTH_MAX					((UINT8_t)32)
/** 
 *  @}
 */ /* End of group DataLengtheConfiguration */
/** 
 *  @defgroup  ReceiveError Receive Error 
 *  @brief    Receive Error  .
 *  @{
 */
#define ERROR_UART_RECEIVE				((UINT8_t)(1))
#define ERROR_COMPARE					((UINT8_t)(3))
/** 
 *  @}
 */ /* End of group ReceiveError */
/** 
 *  @defgroup UART_DataConfiguration  Data Configuration
 *  @brief    Data Configuration Macro Definisiton.
 *  @{
 */
#define UART_TRANS_STOP_BIT_LENGTH		((UINT8_t)0x00)				/*!< 1 bit */
#define UART_PARITY						((UINT8_t)0x00)				/*!< No Parity  */
#define UART_TXD_UART_OUTPUT			((UINT8_t)0x00)				/*!< UART Output */
#define UART_CLOCK						((UINT8_t)0x00)				/*!< fcgck */
#define UART_RT_CLOCK					((UINT8_t)0x20)				/*!< 17/17clock */
#define UART_NOIZE_CANCEL_TIME			((UINT8_t)0x00)				/*!< No Noize Removal */
#define UART_RECIEVE_STOP_BIT_LENGTH	((UINT8_t)0x00)				/*!< 1 bit */
/**
 *  @}
 */ /* End of group UART_DataConfiguration */
/** 
 *  @defgroup UART_TxEnable  Transmitting Enable
 *  @brief    Transmitting Enable Macro Definisiton.
 *  @{
 */
#define UART_TXE_ENABLE					((UINT8_t)0x01)				/*!< Transmitting Enable */
#define UART_TXE_DISABLE				((UINT8_t)0x00)				/*!< Transmitting Disable */
/**
 *  @}
 */ /* End of group UART_TxEnable */
 /** 
 *  @defgroup UART_RxEnable  Receiving Enable
 *  @brief    Receiving Enable Macro Definisiton.
 *  @{
 */
#define UART_RXE_ENABLE					((UINT8_t)0x01)				/*!< Receiving Enable */
#define UART_RXE_DISABLE				((UINT8_t)0x00)				/*!< Receiving Disable */
/**
 *  @}
 */ /* End of group UART_RxEnable */
/** 
 *  @defgroup  Port Port Setting
 *  @brief     Port Setting I/O,FC,ON/OFF
 *  @{
 */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
#define PORT_INPUT						((UINT8_t)0x00)			/*!< Port Mode Input */
#define PORT_FUNC_ON					((UINT8_t)0x01)			/*!< Port Function ON */
/** 
 *  @}
 */ /* End of group Port_Output */


/** 
 *  @}
 */ /* End of group UART_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_Private_define UART Private Define
 *  @{
 */
const UINT8_t dsp_tbl1[] = {"UART Sample Program\n"};
const UINT8_t dsp_tbl2[] = {"----------------------------------------------\n"};
const UINT8_t dsp_tbl3[] = {"PC -> MCU(RX):"};
const UINT8_t dsp_tbl4[] = {"MCU -> PC(TX):"};
const UINT8_t dsp_tbl5[] = {"\nerror\n"};
/** 
 *  @}
 */ /* End of group UART_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_Private_variables UART Private Variables
 *  @{
 */
static UINT8_t error       = (UINT8_t)0;			/*!< error flag.        */
static UINT8_t cmp_flag    = (UINT8_t)0;			/*!< Receive complete flag.        */
static UINT8_t rx_data[UART_DATA_LENGTH];
static UINT8_t tx_data[UART_DATA_LENGTH];
static UINT8_t write        = (UINT8_t)0;
static UINT8_t write_sub    = (UINT8_t)0;
static UINT8_t rx_length    = (UINT8_t)0;
static UINT8_t tx_length    = (UINT8_t)0;
static UINT8_t tx_length2   = (UINT8_t)0;
/**
 *  @}
 */ /* End of group UART_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_Private_fuctions UART Private Fuctions
 *  @{
 */
static void UART_Config(void);
static void Send_Display_Data(UINT8_t size, UINT8_t *pnt);
/*--------------------------------------------------*/
/** 
  * @brief  UART_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void UART_Config(void)
{
	/* (1) Select TXD2:P93,RXD2:P94 */
	UATCNG = UART_PORT_TRXD_P9;
	/* (2) Enable UART2  */
	POFFCR1 = UART_UART2_ENABLE;
	/* (3) Set TXD2,RXD2 */
    P93 = SET;
	P9FC3 = PORT_FUNC_ON;
	P9CR3 = PORT_OUTPUT;
	P9CR4 = PORT_INPUT;
	/* (4) Enable interrupt of Rx */
	__DI();
	EIR_INTRXD2 = 1;
	EIR_INTTXD2 = 1;
	__EI();
	/* (5) Set Tx 1 Stop Bit,No Parity,TXD=UART,Trans Clock=fcgck  */
	UART2CR1 = UART_TRANS_STOP_BIT_LENGTH | UART_PARITY | UART_TXD_UART_OUTPUT | UART_CLOCK;
	/* (6) Set RT Clock=17/17,No Noize Removal,Rx 1 Stop Bit */
	UART2CR2 = UART_RT_CLOCK | UART_NOIZE_CANCEL_TIME | UART_RECIEVE_STOP_BIT_LENGTH;
	/* (7) Set Baud Rate = 115200bps */
	UART2DR = UART_BAUD_RATE;
	/* (8) Rx Start */
	UART2CR1_RXE = UART_RXE_ENABLE;
}
/*--------------------------------------------------*/
/** 
  * @brief  send display data
  * @param  size:send data size
  * @param  pnt:send buff pointer
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Send_Display_Data(UINT8_t size, UINT8_t *pnt)
{
	UINT16_t	i;
	UINT8_t 	t_state = (UINT8_t)0;

	tx_length = size;
	write = (UINT8_t)0;
	write_sub = (UINT8_t)0;
	for(i=0 ; i<=size ; i++) {
		tx_data[i] = pnt[i];
	}
	for(i=0;i<15;i++){
		__asm("nop");
	}
	t_state = UART2SR;
	while((t_state & 0x02) != 0) {
		t_state = UART2SR;
	}
	UART2CR1_TXE = UART_TXE_ENABLE;
	TD2BUF = tx_data[write];
	for(;;) {
		if(write == 0xFF) {
			break;
		}
		else {
			/* Do nothing */
		}
	}
}

/**
 *  @}
 */ /* End of group UART_Private_functions */

/**
 *  @}
 */ /* End of group UART_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_Exported_functions UART Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	UINT8_t i = (UINT8_t)0;
	UINT8_t j = (UINT8_t)0;
	UINT8_t t_state = (UINT8_t)0;
	
	/*------------------------*/
	/* UART Setting           */
	/*------------------------*/
	UART_Config();
	
	/*------------------------*/
	/* Init Display           */
	/*------------------------*/
	Send_Display_Data((UINT8_t)sizeof(dsp_tbl1), (UINT8_t *)dsp_tbl1);
	Send_Display_Data((UINT8_t)sizeof(dsp_tbl2), (UINT8_t *)dsp_tbl2);
	Send_Display_Data((UINT8_t)sizeof(dsp_tbl3), (UINT8_t *)dsp_tbl3);
	
	/*------------------------*/
	/* Wait Data Receive      */
	/*------------------------*/
	while(cmp_flag == 0){
	}
	for(;;){
		if(cmp_flag == SET) {
			__DI();
			cmp_flag = CLEAR;
			__EI();
			if(error == 0x00) {							/* no error */
				Send_Display_Data((UINT8_t)sizeof(dsp_tbl4), (UINT8_t *)dsp_tbl4);
				
				/* send input data */
				__DI();
				for(i = 0 ; i < UART_DATA_LENGTH ; i++) {
					tx_data[i] = (UINT8_t)0;
				}
				for(i = 0 ; i < tx_length2 ; i++) {
					tx_data[i] = rx_data[i];
				}
				for(i = 0 ; i < UART_DATA_LENGTH ; i++) {
					rx_data[i] = (UINT8_t)0;
				}
				tx_data[tx_length2] = '\n';
				j=0;
				for(i = (UINT8_t)(tx_length2 + 1); i < (UINT8_t)(tx_length2 + 1 + (UINT8_t)sizeof(dsp_tbl3)); i++){
					tx_data[i] = dsp_tbl3[j];
					j++;
				}
				tx_length = (UINT8_t)(tx_length2 + 1 + (UINT8_t)sizeof(dsp_tbl3));
				write = (UINT8_t)0;
				__EI();
				t_state = UART2SR;
				while((t_state & 0x02) != 0) {
					t_state = UART2SR;
				}
				UART2CR1_TXE = UART_TXE_ENABLE;
				TD2BUF = tx_data[write];
			}
			else {
				Send_Display_Data((UINT8_t)sizeof(dsp_tbl5), (UINT8_t *)dsp_tbl5);
				Send_Display_Data((UINT8_t)sizeof(dsp_tbl3), (UINT8_t *)dsp_tbl3);
				__DI();
				error = (UINT8_t)0;
				__EI();
			}
		}
		else {
			/* Do nothing */
		}
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  Interrupt RXD2
  * @param  -
  * @return -
  * @retval -
  * @note   UART RXD2 interrupt
  */
/*--------------------------------------------------*/
void __interrupt IntRXD2(void)
{
	UINT8_t r_state = (UINT8_t)0;
	CHAR_t data = (CHAR_t)0;
	
	r_state = UART2SR;
	data = (CHAR_t)RD2BUF;
	
	/* RBFL0 & error check */
	if((r_state & 0xE0) == 0) {
		if ((data != -1) && (rx_length < DATA_LENGTH_MAX)) {
			rx_data[rx_length] = (char)data;
			rx_length++;
			switch ((char)data) {
			/*--- CR: carriage return ---*/
			case '\r':
				/* no process */
				break;
			/*--- LF: line feed ---*/
			case '\n':
				cmp_flag = (UINT8_t)1;
				tx_length2 = (UINT8_t)(rx_length - 2);
				rx_length = 0;
				break;
			/*--- default ---*/
			default:
				/* no process */
				break;
			}
		}
		else {
			error = ERROR_UART_RECEIVE;
			rx_length = (UINT8_t)0;
			cmp_flag = (UINT8_t)1;
		}
	}
	else
	{
		rx_length = (UINT8_t)0;
		/* error or busy or buff empty */
		cmp_flag = (UINT8_t)1;
		error = ERROR_UART_RECEIVE;
	}
}
/*--------------------------------------------------*/
/** 
  * @brief  Interrupt TXD2
  * @param  -
  * @return -
  * @retval -
  * @note   UART TXD2 interrupt
  */
/*--------------------------------------------------*/
void __interrupt IntTXD2(void)
{
	write++;
	
	if(write >= tx_length ){
		/* Tx Stop */
		tx_length = (UINT8_t)0;
		write = (UINT8_t)0xFF;
		write_sub = (UINT8_t)0xFF;
		UART2CR1_TXE = UART_TXE_DISABLE;
	}
	else{
		TD2BUF = tx_data[write];
	}
}
/**
 *  @}
 */ /* End of group UART_Exported_functions */

/**
 *  @}
 */ /* End of group UART */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
