/** 
 *******************************************************************************
 * @file    main.c
 * @brief   UART_IrDA Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-07-19 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup UART UART Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_IrDA_Private_macro UART_IrDA Private Macro
 *  @{
 */
/** 
 *  @defgroup TBT_TBTCK Macro Definition
 *  @brief    TBT Interval.
 *  @{
 */
#define TBT_INTERVAL					((UINT8_t)0x03)			/*!< TBT Interrupt Clock(011=fcgck/2^13) */
/**
 *  @}
 */ /* End of group TBT_TBTCK Macro Definition */
/** 
 *  @defgroup TBT_TBTEN TBT Start
 *  @brief    TBT Start.
 *  @{
 */
#define TBT_TBTEN_START					((UINT8_t)0x01)			/*!< TBT Start(Innterrupt request signal enable) */
/**
 *  @}
 */ /* End of group TBT_TBTEN */
/** 
 *  @defgroup UART_IrDA_UART2 UART2 Enable
 *  @brief    UART1 Control Enable Macro Definisiton.
 *  @{
 */
#define UART_UART1_ENABLE				((UINT8_t)0x02)				/*!< UART1 Control Enable */
#define UART_UART2_ENABLE				((UINT8_t)0x04)				/*!< UART2 Control Enable */
/**
 *  @}
 */ /* End of group UART_UART2 */
/** 
 *  @defgroup BaudRateConfiguration BaudRate Configuration
 *  @brief    BaudRate .
 *  @{
 */
#define UART_BAUD_RATE					((UINT8_t)(0x04))
/** 
 *  @}
 */ /* End of group BoundRateConfiguration */
/** 
 *  @defgroup DataLengtheConfiguration Data Length Configuration
 *  @brief    Data Length Configuration.
 *  @{
 */
#define UART_DATA_LENGTH				((UINT8_t)1)
/** 
 *  @}
 */ /* End of group DataLengtheConfiguration */

/** 
 *  @defgroup UART_IrDA_DataConfiguration  Data Configuration
 *  @brief    Data Configuration Macro Definisiton.
 *  @{
 */
#define UART_TRANS_STOP_BIT_LENGTH		((UINT8_t)0x00)				/*!< 1 bit */
#define UART_PARITY						((UINT8_t)0x00)				/*!< No Parity  */
#define UART_TXD_UART_IrDA				((UINT8_t)0x04)				/*!< IrDA */
#define UART_CLOCK						((UINT8_t)0x00)				/*!< fcgck */
#define UART_RT_CLOCK					((UINT8_t)0x20)				/*!< 17/17clock */
/**
 *  @}
 */ /* End of group UART_IrDA_DataConfiguration */
 /** 
 *  @defgroup UART_TxEnable  Transmitting Enable
 *  @brief    Transmitting Enable Macro Definisiton.
 *  @{
 */
#define UART_TXE_ENABLE					((UINT8_t)0x01)				/*!< Transmitting Enable */
#define UART_TXE_DISABLE				((UINT8_t)0x00)				/*!< Transmiting Disable */
/**
 *  @}
 */ /* End of group UART_TxEnable */
/** 
 *  @}
 */ /* End of group Port_Output */
 /** 
 *  @defgroup  Port Port Setting
 *  @brief     Port Setting I/O,FC,ON/OFF
 *  @{
 */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
#define PORT_TXD2						((UINT8_t)0x01)			/*!< Port Function TXD2 */
/** 
 *  @}
 */ /* End of group Port_Output */


/** 
 *  @}
 */ /* End of group UART_IrDA_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_IrDA_Private_variables UART_IrDA Private Variables
 *  @{
 */
static UINT8_t timer_flag  = (UINT8_t)0;				/*!< Main timer flag.   */
static UINT8_t tx_data[UART_DATA_LENGTH];
static UINT8_t tbt_cnt     = (UINT8_t)0;				/*!<  tbt count.   */
/**
 *  @}
 */ /* End of group UART_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_IrDA_Private_fuctions UART_IrDA Private Fuctions
 *  @{
 */
static void UART_IrDA_Config(void);
static void TBT_Config(void);
static void TBT_Start(void);
static void UARTTX_Start(void);
static void UARTTX_Trans(void);
static void UARTTX_Stop(void);
/*--------------------------------------------------*/
/** 
  * @brief  UART_IrDA_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void UART_IrDA_Config(void)
{
	/* (1) Enable UART2  */
	POFFCR1 = UART_UART2_ENABLE;
	/* (2) Set TXD2 */
	P93 = SET;
	P9FC3 = PORT_TXD2;
	P9CR3 = PORT_OUTPUT;
	/* (3) Enable interrupt of Rx */
	__DI();
	EIR_INTTXD2 = 1;
	__EI();
	/* (4) Set Tx 1 Stop Bit,No Parity,TXD=UART,Trans Clock=fcgck  */
	UART2CR1 = UART_TRANS_STOP_BIT_LENGTH | UART_PARITY | UART_TXD_UART_IrDA | UART_CLOCK;
	/* (5) Set RT Clock=17/17 */
	UART2CR2 = UART_RT_CLOCK;
	/* (6) Set Baud Rate = 115200bps */
	UART2DR = UART_BAUD_RATE;
	/* (7) Tx Start */
	UART2CR1_TXE = UART_TXE_ENABLE;
	/* (8) Tx Data Set */
	tx_data[0] = 0xA3;
}
/*--------------------------------------------------*/
/** 
  * @brief  TBT_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TBT_Config(void)
{
	/* (1) Enable interrupt of TBT */
	/* (2) Setting interrupt frequency,TBTCR[2:0]=(/2^) */
	__DI();
	EIR_INTTBT = SET;				/* (1) */
	__EI();
	TBTCR  = TBT_INTERVAL;			/* (2) */
}
/*--------------------------------------------------*/
/** 
  * @brief  TBT_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void TBT_Start(void)
{
	/* TBTEN = 1 */
	TBTEN = TBT_TBTEN_START;
}
/*--------------------------------------------------*/
/** 
  * @brief  UARTTX_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void UARTTX_Start(void)
{
	UART2CR1_TXE = SET;
}

/*--------------------------------------------------*/
/** 
  * @brief  UARTTX_Trans
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void UARTTX_Trans(void)
{
	TD2BUF = tx_data[0];
}

/*--------------------------------------------------*/
/** 
  * @brief  UARTTX_Stop
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void UARTTX_Stop(void)
{
	UART2CR1 = UART_TRANS_STOP_BIT_LENGTH | UART_PARITY | UART_TXD_UART_IrDA | UART_CLOCK;
}

/**
 *  @}
 */ /* End of group UART_IrDA_Private_functions */

/**
 *  @}
 */ /* End of group UART_IrDA_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_IrDA_Exported_functions UART_IrDA Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	TBT_Config();
	TBT_Start();
	
	/*------------------------*/
	/* UART_IrDA Setting           */
	/*------------------------*/
	UART_IrDA_Config();
	UARTTX_Start();
	UARTTX_Trans();
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
		if(timer_flag == 1) {
			timer_flag = (UINT8_t)0;
			UARTTX_Start();
			UARTTX_Trans();
		}
		else {
			/* Do nothing */
		}
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  Interrupt TBT
  * @param  -
  * @return -
  * @retval -
  * @note   100ms timer interrupt
  */
/*--------------------------------------------------*/
void __interrupt IntTBT(void)
{
	/* 1ms cycle */
	tbt_cnt++;
	
	if(tbt_cnt >= 122) {
		tbt_cnt = (UINT8_t)0;
		/* 100ms timer flag set */
		timer_flag = (UINT8_t)1;
	}
	else {
		/* Do nothing */
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  Interrupt TXD2
  * @param  -
  * @return -
  * @retval -
  * @note   UART_IrDA TXD2 interrupt
  */
/*--------------------------------------------------*/
void __interrupt IntTXD2(void)
{
	/* TBSY & TBFL check */
	if((UART2SR & 0x03) == 0) {
		UARTTX_Stop();
	}
	else {
		/* busy or buff full */
	}
}
/**
 *  @}
 */ /* End of group UART_IrDA_Exported_functions */

/**
 *  @}
 */ /* End of group UART_IrDA */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
