/** 
 *******************************************************************************
 * @file    main.c
 * @brief   VOLT_DETECT Sample Application.
 * @version V1.0.2.0
 * $Date:: 2019-07-19 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup LVD LVD Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Private_macro LVD Private Macro
 *  @{
 */
/** 
 *  @defgroup RTC_RTCEN RTC Enable
 *  @brief    RTC Control Enable.
 *  @{
 */
#define RTC_RTCEN_ENABLE				((UINT8_t)0x20)			/*!< RTC enable */
/**
 *  @}
 */ /* End of group RTC_RTCEN */
/** 
 *  @defgroup RTC_RTCSEL Macro Definition
 *  @brief    RTC Interval.
 *  @{
 */
#define RTC_SEL_2_15_FS				((UINT8_t)0x00)			/*!< RTC Interval(= 2^15/fs = 1.000[s]) */
/**
 *  @}
 */ /* End of group RTC_RTCSEL Macro Definition */
/** 
 *  @defgroup RTC_RTCRUN RTC Start
 *  @brief    RTC Start.
 *  @{
 */
#define RTC_RTCRUN_START				((UINT8_t)0x01)			/*!< RTC Start */
/**
 *  @}
 */ /* End of group RTC_RTCRUN */
/** 
 *  @defgroup  Port Port Setting
 *  @brief     Port Setting
 *  @{
 */
#define PORT_OFF						((UINT8_t)0x00)			/*!< Port OFF */
#define PORT_ON							((UINT8_t)0x01)			/*!< Port ON */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
/** 
 *  @}
 */ /* End of group Port */

/** 
 *  @}
 */ /* End of group LVD_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Private_define LVD Private Define
 *  @{
 */
#define LVD_VOLTAGE						((UINT8_t)0x00)			/*!< LVD voltage 4.5+-0.2V    */
#define LVD_ST_MASK						((UINT8_t)0x04)			/*!< LVD ST mask              */
#define LVD_UPPER						((UINT8_t)0x00)			/*!< LVD UPPER                */
#define LVD_LOWER						((UINT8_t)0x04)			/*!< LVD LOWER                */
#define LVD_UNKNOWN						((UINT8_t)0xFF)			/*!< LVD UNKNOWN              */
#define LVD_STATUS_CT					((UINT8_t)0x08)			/*!< LVD status count         */
#define LVD_SUCCESS						((UINT8_t)0x00)			/*!< LVD result Success       */
#define LVD_ERROR						((UINT8_t)0x01)			/*!< LVD result Error         */
/** 
 *  @}
 */ /* End of group LVD_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Private_variables LVD Private Variables
 *  @{
 */
UINT8_t lvd_status_now;								/*!< LVD status now           */
UINT8_t lvd_status_old;								/*!< LVD status old           */
UINT8_t lvd_status_count = 0;						/*!< LVD status count         */
UINT8_t lvd_status = LVD_UPPER;						/*!< LVD status               */
UINT8_t lvd_led_pattern = 0;						/*!< LVD LED pattern          */
/**
 *  @}
 */ /* End of group LVD_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Private_fuctions LVD Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void LVD_Config(void);
static void RTC_Config(void);
static void RTC_Start(void);
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Select output mode on PORT 25 (LED0) */
	P2CR5 = PORT_OUTPUT;
	/* (2) Turn off PORT 25 (LED0) */
	P25 = PORT_OFF;
	/* (3) Select output mode on PORT 26 (LED1) */
	P2CR6 = PORT_OUTPUT;
	/* (4) Turn off PORT 26 (LED1) */
	P26 = PORT_OFF;
}

/*--------------------------------------------------*/
/**
  * @brief     LVD_Configure.
  * @param     -
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
static void LVD_Config(void)
{
	VDCR1 = LVD_VOLTAGE;			/* Voltage level select */
	VD1EN = SET;
}
/*--------------------------------------------------*/
/** 
  * @brief  RTC_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void RTC_Config(void)
{
	/* (1) Enable RTC Control */
	/* (2) Enable interrupt of RTC */
	/* (3) Setting interrupt frequency,RTCCR[3:1]=(2^15/fs) */
	POFFCR2 = RTC_RTCEN_ENABLE;		/* (1) */
	__DI();
	EIR_INTRTC = 1;					/* (2) */
	__EI();
	RTCCR  = RTC_SEL_2_15_FS;		/* (3) */
}
/*--------------------------------------------------*/
/** 
  * @brief  RTC_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void RTC_Start(void)
{
	/* RTCRUN = 1 */
	RTCRUN = RTC_RTCRUN_START;
}
/**
 *  @}
 */ /* End of group LVD_Private_fuctions */
/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Exported_functions LVD Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	UINT8_t res = LVD_ERROR;
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	LVD_Config();
	lvd_status = LVD_UNKNOWN;
	RTC_Config();
	RTC_Start();

	for(;;){
		/*----------------------*/
		/* LVD Status Check            */
		/*----------------------*/
		lvd_status_now = (UINT8_t)(VDCR1 & LVD_ST_MASK);
		
		if(lvd_status_now == lvd_status_old) {
			lvd_status_count++;
			if(lvd_status_count >= LVD_STATUS_CT) {
				if(lvd_status != lvd_status_now) {
					lvd_status = lvd_status_now;
					res = LVD_SUCCESS;
				}
				else {
					/* Do nothing */
				}
				lvd_status_count = 0;
			}
			else {
				/* Do nothing */
			}
		}
		else {
			lvd_status_old = lvd_status_now;
			lvd_status_count = 0;
		}
		if(res == LVD_SUCCESS) {
			if(lvd_status == LVD_UPPER) {
				lvd_led_pattern = 0;
			}
			else {
				lvd_led_pattern = 1;
			}
		}
		else {
			/* do nothing */
		}
	}
}
/*--------------------------------------------------*/
/** 
  * @brief  INTRTC
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntRTC(void)
{
	static UINT8_t i = 0;
	
	if(lvd_led_pattern == 0) {
		P25 = PORT_ON;
		P26 = PORT_OFF;
	}
	else {
		P25 = PORT_OFF;
		i = P26;
		i ^= 1;
		P26 = i;
	}
}

/**
 *  @}
 */ /* End of group LVD_Exported_functions */

/**
 *  @}
 */ /* End of group LVD */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
