/** 
 *******************************************************************************
 * @file    main.c
 * @brief   WDT Sample Application.
 * @version V1.0.3.0
 * $Date:: 2019-08-30 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdlib.h>
#include "TMP89Fx.h"  /*!< TMP89Fx Group Header file. */
#include "cmn_type.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup WDT WDT Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup WDT_Private_macro WDT Private Macro
 *  @{
 */

#define WDT_DEMO_CLEAR

/** 
 *  @defgroup RTC_RTCEN RTC Enable
 *  @brief    RTC Control Enable.
 *  @{
 */
#define RTC_RTCEN_ENABLE				((UINT8_t)0x20)			/*!< RTC enable */
/**
 *  @}
 */ /* End of group RTC_RTCEN */
/** 
 *  @defgroup RTC_RTCSEL Macro Definition
 *  @brief    RTC Interval.
 *  @{
 */
#define RTC_INTERVAL					((UINT8_t)0x00)			/*!< RTC Interval(= 2^15/fs = 1.000[s]) */
/**
 *  @}
 */ /* End of group RTC_RTCSEL Macro Definition */
/** 
 *  @defgroup RTC_RTCRUN RTC Start
 *  @brief    RTC Start.
 *  @{
 */
#define RTC_RTCRUN_START				((UINT8_t)0x01)			/*!< RTC Start */
#define RTC_RTCRUN_STOP					((UINT8_t)0x00)			/*!< RTC Stop */
/**
 *  @}
 */ /* End of group RTC_RTCRUN */
/** 
 *  @defgroup WDT_WDTCR Macro Definition
 *  @brief    WDTCR setting.
 *  @{
 */
#define WDT_START						((UINT8_t)0x01)			/*!< WDT start */
#define WDT_COUNT_CLR_TIME				((UINT8_t)0x10)			/*!< WDT clear time(1/2) */
#define WDT_DETECTION_TIME				((UINT8_t)0x06)			/*!< WDT over flow time(2^24/fcgck) */
#define WDT_OVER_FLOW_BEHAVE			((UINT8_t)0x00)			/*!< WDT over flow behave(WDT interrupt) */
/**
 *  @}
 */ /* End of group WDT_WDTCR Macro Definition */
/** 
 *  @defgroup WDT_WDCDR Macro Definition
 *  @brief    WDCDR setting.
 *  @{
 */
#define WDT_CLEAR_CODE					((UINT8_t)0x4E)			/*!< WDT count clear code */
/**
 *  @}
 */ /* End of group WDT_WDCDR Macro Definition */
/** 
 *  @defgroup  Port Port Setting
 *  @brief     Port Setting
 *  @{
 */
#define PORT_OFF						((UINT8_t)0x00)			/*!< Port OFF */
#define PORT_ON							((UINT8_t)0x01)			/*!< Port ON */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
/** 
 *  @}
 */ /* End of group Port */

/** 
 *  @}
 */ /* End of group WDT_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup WDT_Private_variables WDT Private Variables
 *  @{
 */
static UINT8_t blink_flag = 0;					/*!<  blink flag.        */
static UINT8_t WDT_CLR_TIMING_cnt = 0;			/*!<  WDT clear timing count. */
/**
 *  @}
 */ /* End of group WDT_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup WDT_Private_fuctions WDT Private Fuctions
 *  @{
 */
static void Port_Config(void);
static void RTC_Config(void);
static void WDT_Config(void);
static void WDT_Start(void);
/*--------------------------------------------------*/
/** 
  * @brief  Port_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void Port_Config(void)
{
	/* (1) Select output mode on PORT 25 (LED0) */
	P2CR5 = PORT_OUTPUT;
	/* (2) Turn on PORT 25 (LED0) */
	P25 = PORT_ON;
	/* (3) Select output mode on PORT 26 (LED1) */
	P2CR6 = PORT_OUTPUT;
	/* (4) Turn on PORT 26 (LED1) */
	P26 = PORT_ON;
	/* (5) Select output mode on PORT 27 (LED2) */
	P2CR7 = PORT_OUTPUT;
	/* (6) Turn on PORT 27 (LED2) */
	P27 = PORT_ON;
	/* (7) Select output mode on PORT 11 (LED3) */
	P1CR1 = PORT_OUTPUT;
	/* (8) Turn oN PORT 11 (LED3) */
	P11 = PORT_ON;
}
/*--------------------------------------------------*/
/** 
  * @brief  RTC_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void RTC_Config(void)
{
	/* (1) Enable RTC Control */
	/* (2) Enable interrupt of RTC */
	/* (3) Setting interrupt frequency */
	POFFCR2 = RTC_RTCEN_ENABLE;		/* (1) */
	__DI();
	EIR_INTRTC = 1;					/* (2) */
	__EI();
	RTCCR  = RTC_INTERVAL;			/* (3) */
	
	/* RTCRUN = 1 */
	RTCRUN = RTC_RTCRUN_START;
}
/*--------------------------------------------------*/
/** 
  * @brief  WDT_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void WDT_Config(void)
{
	/* Setting WDCTR,clear time(10=under1/2)/over flow time(01=2^24/fcgck)
	/over flow behave(0=WDT interrupt) */
	WDCTR = WDT_COUNT_CLR_TIME | WDT_DETECTION_TIME | WDT_OVER_FLOW_BEHAVE;
}
/*--------------------------------------------------*/
/** 
  * @brief  WDT_Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void WDT_Start(void)
{
	/* WDTEN = 1 */
	WDTEN = WDT_START;
}

/**
 *  @}
 */ /* End of group WDT_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup WDT_Exported_functions WDT Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void main(void)
{
	/*------------------------*/
	/* Port Setting           */
	/*------------------------*/
	Port_Config();
	RTC_Config();
	/*------------------------*/
	/* WDT Setting      */
	/*------------------------*/ 
	WDT_Config();
	/* WDT Start */
	WDT_Start();
	/*------------------------*/
	/* Normal Mode Processing */
	/*------------------------*/
	for(;;){
#if defined(WDT_DEMO_CLEAR)
		if(WDT_CLR_TIMING_cnt >= 4) {
			WDT_CLR_TIMING_cnt = 0;
			
			WDCDR = WDT_CLEAR_CODE;
		}
		else {
			/* Do nothing */
		}
#endif
	}
}
/*--------------------------------------------------*/
/** 
  * @brief  INTRTC
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntRTC(void)
{
	if(blink_flag == 1){
		blink_flag = (UINT8_t)0;
		P25 = PORT_ON;
		P26 = PORT_ON;
		P27 = PORT_ON;
		P11 = PORT_ON;
	}
	else {
		blink_flag = (UINT8_t)1;
		P25 = PORT_OFF;
		P26 = PORT_OFF;
		P27 = PORT_OFF;
		P11 = PORT_OFF;
	}
	
	WDT_CLR_TIMING_cnt++;
}
/*--------------------------------------------------*/
/** 
  * @brief  INTWDT
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void __interrupt IntWDT(void)
{
	/* Turn off PORT 25 (LED0) */
	P25 = PORT_OFF;
	/* Turn off PORT 26 (LED1) */
	P26 = PORT_OFF;
	/* Turn off PORT 27 (LED2) */
	P27 = PORT_OFF;
	/* Turn off PORT 11 (LED3) */
	P11 = PORT_OFF;
	
	RTCRUN = RTC_RTCRUN_STOP;
}
/**
 *  @}
 */ /* End of group WDT_Exported_functions */

/**
 *  @}
 */ /* End of group WDT */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
