/** 
 *******************************************************************************
 * @file    uart_io.h
 * @brief   External reference declaration for Uart Input/Output.
 * @version V1.0.1.0
 * $Date:: 2019-06-10 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __UART_IO_H
#define __UART_IO_H

#ifdef __cplusplus
extern "C" {
#endif

#include	"TMP89Fx.h"		/*!< TMP89Fx Group Header file. */
#include	"cmn_type.h"

/** 
 *  @addtogroup UART_UTILITIES_Exported_define
 *  @{
 */

/** 
 *  @defgroup Serial_setting Macro Definisiton
 *  @brief    Serial setting
 *  @{
 */
#define SERIAL_SEL_UART_PORT			((UINT8_t)0x00)				/*!< UART2 Port select */
#define UART_PORT_TRXD_PB				((UINT8_t)0x00)				/*!< UART2 TXD,RXD Port select */
/**
 *  @}
 */ /* End of group Serial_setting */
/** 
 *  @defgroup UART_UART0 UART0 Enable
 *  @brief    UART0 Control Enable Macro Definisiton.
 *  @{
 */
#define UART_UART0_ENABLE				((UINT8_t)0x01)				/*!< UART0 Control Enable */
#define UART_UART1_ENABLE				((UINT8_t)0x02)				/*!< UART1 Control Enable */
#define UART_UART2_ENABLE				((UINT8_t)0x04)				/*!< UART2 Control Enable */
/**
 *  @}
 */ /* End of group UART_UART0 */
/** 
 *  @defgroup BaudRateConfiguration BaudRate Configuration
 *  @brief    BaudRate .
 *  @{
 */
#define UART_BAUD_RATE					((UINT8_t)(0x04))
/** 
 *  @}
 */ /* End of group BoundRateConfiguration */
/** 
 *  @defgroup DataLengtheConfiguration Data Length Configuration
 *  @brief    Data Length Configuration.
 *  @{
 */
#define UART_DATA_LENGTH				((UINT8_t)250)
/** 
 *  @}
 */ /* End of group DataLengtheConfiguration */
/** 
 *  @defgroup  ReceiveError Receive Error 
 *  @brief    Receive Error  .
 *  @{
 */
#define ERROR_UART_RECEIVE				((UINT8_t)(1))
#define ERROR_COMPARE					((UINT8_t)(3))
/** 
 *  @}
 */ /* End of group ReceiveError */
/** 
 *  @defgroup UART_DataConfiguration  Data Configuration
 *  @brief    Data Configuration Macro Definisiton.
 *  @{
 */
#define UART_TRANS_STOP_BIT_LENGTH		((UINT8_t)0x00)				/*!< 1 bit */
#define UART_PARITY						((UINT8_t)0x00)				/*!< No Parity  */
#define UART_TXD_UART_OUTPUT			((UINT8_t)0x00)				/*!< UART Output */
#define UART_CLOCK						((UINT8_t)0x00)				/*!< fcgck */
#define UART_RT_CLOCK					((UINT8_t)0x20)				/*!< 17/17clock */
#define UART_NOIZE_CANCEL_TIME			((UINT8_t)0x00)				/*!< No Noize Removal */
#define UART_RECIEVE_STOP_BIT_LENGTH	((UINT8_t)0x00)				/*!< 1 bit */
/**
 *  @}
 */ /* End of group UART_DataConfiguration */

/** 
 *  @defgroup UART_TxInterrupt  Tx Interrpt
 *  @brief    Available Transrate Interrupt Macro Definisiton.
 *  @{
 */
#define UART_TX_INT_DISABLE				((UINT8_t)0x00)				/*!< Not Available. */
#define UART_TX_INT_ENABLE				((UINT8_t)0x01)				/*!< Available.     */
/**
 *  @}
 */ /* End of group UART_TxInterrupt */
/** 
 *  @defgroup UART_RxInterrupt  Rx Interrpt
 *  @brief    Available Receive Interrupt Macro Definisiton.
 *  @{
 */
#define UART_RX_INT_DISABLE				((UINT8_t)0x00)				/*!< Not Available. */
#define UART_RX_INT_ENABLE				((UINT8_t)0x01)				/*!< Available.     */
/**
 *  @}
 */ /* End of group UART_RxInterrupt */
/** 
 *  @defgroup UART_TxEnable  Transmitting Enable
 *  @brief    Transmitting Enable Macro Definisiton.
 *  @{
 */
#define UART_TXE_ENABLE					((UINT8_t)0x01)				/*!< Transmitting Enable */
#define UART_TXE_DISABLE				((UINT8_t)0x00)				/*!< Transmitting Disable */
/**
 *  @}
 */ /* End of group UART_TxEnable */
 /** 
 *  @defgroup UART_RxEnable  Receiving Enable
 *  @brief    Receiving Enable Macro Definisiton.
 *  @{
 */
#define UART_RXE_ENABLE					((UINT8_t)0x01)				/*!< Receiving Enable */
#define UART_RXE_DISABLE				((UINT8_t)0x00)				/*!< Receiving Disable */
/**
 *  @}
 */ /* End of group UART_RxEnable */
/** 
 *  @defgroup  Port Port Setting
 *  @brief     Port Setting I/O,FC,ON/OFF
 *  @{
 */
#define PORT_OUTPUT						((UINT8_t)0x01)			/*!< Port Mode Output */
#define PORT_INPUT						((UINT8_t)0x00)			/*!< Port Mode Input */
#define PORT_FUNC_ON					((UINT8_t)0x01)			/*!< Port Function Change */
#define PORT_OFF						((UINT8_t)0x00)			/*!< Port OFF */
#define PORT_ON							((UINT8_t)0x01)			/*!< Port ON */
/** 
 *  @}
 */ /* End of group Port_Output */
/** 
 *  @}
 */ /* End of group UART_UTILITIES_Exported_define */

/*------------------------------------------------------------------------------*/
/*  Member                                                                      */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_variables UART Private Variables
 *  @{
 */
extern UINT8_t error;			/*!< error flag.        */
extern UINT8_t cmp_flag;			/*!< Receive complete flag.        */
extern UINT8_t rx_data[UART_DATA_LENGTH];
extern UINT8_t tx_data[UART_DATA_LENGTH];
extern UINT8_t write;
extern UINT8_t rx_length;
extern UINT8_t tx_length;
/**
 *  @}
 */ /* End of group UART_variables */

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
extern void UART_Config(void);
extern void uart_initialize(void);
extern void TX_Start(void);
extern void uart_data_set(UINT8_t length);
extern void Send_Display_Data(UINT8_t size, UINT8_t *pnt);
/**
 *  @}
 */ /* End of group UART_IO_PUBLIC */

/**
 *  @}
 */ /* End of group UART_IO */

/**
 *  @}
 */ /* End of group Sample_Appli */
#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __UART_IO_H */

